<?php

// <editor-fold defaultstate="collapsed" desc="DESCRIPTION">
/*
 * the common.php:
 * is a protected file.
 * escapes $_GET, $_POST, $_COOKIE data.
 * includes the config file, if present.
 * starts the installion if HCS is not installed.
 * checks PHP version.
 * gets und sets language and style.
 * filles the $_CONFIG array.
 * prepares the $TEMPLATE variables as a global page_template
 */
// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="CALLABLE PROTECTION">
// Make file not callable directly
if (!defined('HCS_CALLABLE_FILE')) {
    header('HTTP/1.0 404 Not Found');
    die('You shall not access this file directly.');
    exit;
}
// </editor-fold>
/* -------------------------------------------------------------------------- */
// Default the site-wide variables
if (!isset($HCS_ROOTPATH)) {
    $HCS_ROOTPATH = './';
}


require_once $HCS_ROOTPATH . 'includes/php/common_functions.php';

// Disable magic quotes and add slashes to global arrays
ini_set('magic_quotes_runtime', 0);
if (get_magic_quotes_gpc() == 0) {
    $_GET = escapeData($_GET);
    $_POST = escapeData($_POST);
    $_COOKIE = escapeData($_COOKIE);
}
checkCookiesEnabled();
//setcookie("cookieCheck", "on", time() + 31536000, '/');

// Get the config file
$conf_file = $HCS_ROOTPATH . 'config.php';
if (!is_file($conf_file)) {
    @fopen($conf_file, "w+");
    if (!is_file($conf_file)) {
        die('Error: could not locate configuration file config.php in the Root Folder. Create an empty config.php or set the install folder to chmod 777.');
    }
} else {
    require_once($conf_file);
}

require_once $HCS_ROOTPATH . 'defaults.php';
require_once $HCS_ROOTPATH . 'includes/php/class_template.php';
require_once $HCS_ROOTPATH . 'includes/php/functions.php';

// check if HCS is allready installed
if (!defined('HCS_INSTALLED')) {
    // if not, redirect to the install.php
    redirect($HCS_ROOTPATH . 'install/index.php');

    //echo "<script>window.location.href = '" . $HCS_ROOTPATH . "install/index.php';</script>";
}

// PHP5 Warning..
if (version_compare(phpversion(), "5.3.0", "<")) {
    die(sprintf("HCS requires PHP %1\$s or higher. Your server runs PHP %2\$s", "5.3.0", phpversion()));
}

$sql = new SQL();
pickupSession();
$loggedInUser = getLoggedInUser($sql);


$defaultLanguage = '';
$defaultStyle = '';
$updateCheck = false;
$checkInterval = 0;
$lastCheck = 0;
$showUpdateOnPage = 0;
$expertView = false;
$answer = $sql->queryRequest("SELECT * FROM " . HCS_TABLE_PREFIX . "Values WHERE Name IN ('DefaultLanguage', 'DefaultStyle', 'UpdateCheck', 'UpdateCheckInterval', 'LastUpdateCheck', 'ShowUpdateOnPage', 'ExpertView');");
foreach ($answer as $default) {
    switch ($default['Name']) {
        case 'DefaultLanguage':
            $defaultLanguage = $default['Value'];
            break;
        case 'DefaultStyle':
            $defaultStyle = $default['Value'];
            break;
        case 'UpdateCheck':
            $updateCheck = (bool)$default['Value'];
            break;
        case 'UpdateCheckInterval':
            $checkInterval = (int)$default['Value'];
            break;
        case 'LastUpdateCheck':
            $lastCheck = (int)$default['Value'];
            break;
        case 'ShowUpdateOnPage':
            $showUpdateOnPage = (bool)$default['Value'];
            break;
        case 'ExpertView':
            $expertView = (bool)$default['Value'];
            break;
    }

}

$defaultLanguage = ($defaultLanguage != '') ? $defaultLanguage : (isset($_DEFAULTS['default_lang']) ? $_DEFAULTS['default_lang'] : 'german');
$defaultStyle = ($defaultStyle != '') ? $defaultStyle : (isset($_DEFAULTS['default_style']) ? $_DEFAULTS['default_style'] : 'homematic');

// get language and style
$language = isset($_SESSION['language']) ? $_SESSION['language'] : $defaultLanguage;
if (!@include_once($HCS_ROOTPATH . 'language/' . $language . '/page.php')) {
    die('Could not include the language files! Check to make sure that "' . $HCS_ROOTPATH . 'language/' . $language . '/page.php" exists!');
}
$style = isset($_SESSION['style']) ? $_SESSION['style'] : $defaultStyle;
if (!@file_exists($HCS_ROOTPATH . 'style/' . $style . '/style.css')) {
    die('Could not include the style files! Check to make sure that "' . $HCS_ROOTPATH . 'style/' . $style . '/style.css" exists!');
}

$newerVersionAvailable = false;
$newestVersion = '';

if ($updateCheck) {
    // only check if update check ist enabled
    if (($lastCheck + $checkInterval) < time()) {
        // only check if interval has passed
        $newestVersion = checkNewestHcsVersion($sql);
        if (($newestVersion != '') && (compareVersion($newestVersion, HCS_VERSION) == 1)) {
            $newerVersionAvailable = true;
        }
    }
}

$_CONFIG = array(
    'language' => $language,
    'style' => $style,
    'user' => $loggedInUser,
    'admin' => (isset($_SESSION['admin']) && ($_SESSION['admin'] === true)) ? true : false,
    'default_language' => $defaultLanguage,
    'default_style' => $defaultStyle,
    'updateCheck' => $updateCheck,
    'updateCheckInterval' => $checkInterval,
    'lastUpdateCheck' => $lastCheck,
    'showUpdateOnPage' => $showUpdateOnPage,
    'newerVersionAvailable' => $newerVersionAvailable,
    'checkedVersion' => $newestVersion,
    'expertView' => $expertView
);

$TEMPLATE = new PageTemplate();

// EOF
?>