// DISPLAY AREA
function HCS_Diagrams_Chart(containerElement, eventHandler, globals, config, selfPath) {
    var _this = this;
    this.config = config;
    this.container = containerElement;
    this.eventHandler = eventHandler;
    this.globals = globals;
    this.path = selfPath;

    yepnope({
        load: [
            _this.globals.rootPath + 'includes/js/highcharts/highstock.js',
            _this.globals.rootPath + 'includes/js/highcharts/highcharts-more.js',
            _this.globals.rootPath + 'language/' + _this.globals.LANGUAGE + '/highcharts-lang.js',
            _this.globals.rootPath + 'style/highcharts-style.js',
            _this.globals.rootPath + 'style/' + _this.globals.STYLE + '/highcharts-style.js'],
        complete: function () {
            if (_this.config.exportEnabled == true){
                yepnope({load: [_this.globals.rootPath + 'includes/js/highcharts/highcharts-exporting.js'], complete: function (){_this.init();}});
            } else {
                _this.init();
            }
        }
    });

}
HCS_Diagrams_Chart.prototype = new Widget;
HCS_Diagrams_Chart.prototype.constructor = HCS_Diagrams_Chart;

HCS_Diagrams_Chart.prototype.init = function () {
    var _this = this;

    Highcharts.setOptions({
        global: {
            useUTC: false
        }
    });

    var yAxisTemp = [];
    _this.config.yAxis = hcs_parseJSON(_this.config.yAxis);
    for (var key in _this.config.yAxis) {
        (function(key){
            yAxisTemp.push({
                id: key,
                title: {text: _this.config.yAxis[key].title},
                opposite: _this.config.yAxis[key].opposite,
                type: _this.config.yAxis[key].type
            });
        })(key);
    }

    var chartOptions = {
        chart: {
            renderTo: this.container,
            zoomType: 'x'
        },
        title: {
            text: _this.config.title
        },
        subtitle: {
            text: _this.config.subTitle
        },
        xAxis: {
            type: 'datetime'
        },
        exporting: {
            enabled: _this.config.exportEnabled
        },
        plotOptions: {
            series: {
                dataGrouping: {
                    enabled: true,
                    smoothed: true,
                    groupPixelWidth: parseInt(_this.config.groupingDistance)
                }
            }
        }
    }

    if(yAxisTemp.length > 0){
        chartOptions.yAxis = yAxisTemp;
    }

    this.container.highcharts(chartOptions);
    this.chart = this.container.highcharts();
    _this.config.series = hcs_parseJSON(_this.config.series);

    for (var key in _this.config.series) {
        var serie = {
            id: 'series-'+key,
            name: _this.config.series[key].name,
            type: _this.config.series[key].type,
            step: _this.config.series[key].step,
            color: _this.config.series[key].color
        }
        if (_this.config.series[key].yAxis != ''){
            serie.yAxis = _this.config.series[key].yAxis;
        }
        _this.chart.addSeries(serie);
    }
};

// CONFIG AREA
function HCS_Diagrams_Chart_CONFIG(containerElement, eventHandler, globals, selfPath) {
    var _this = this;
    this.container = containerElement;
    this.eventHandler = eventHandler;
    this.globals = globals;
    this.path = selfPath;

    this.id = randomString(16, 'aA');
    this.config = {};
    this.config.exportEnabled = false;
    this.config.liveUpdate = true;
    this.config.title = 'Diagramm-Titel';
    this.config.subTitle = '';
    this.config.interval = '';
    this.config.groupingDistance = 15;
    this.config.series = '{}';
    this.config.yAxis = '{}';

    this.standardColor = parseHex(getCSS('color', 'ui-widget-content')).toLowerCase();

    var spinnerPath = "url('" + this.globals["rootPath"] + "data/images/spinner.gif');";

    this.devicesDlg = $('<div id="' + _this.id + '_Devices"></div>')
        .append($('<div style="height:100%; width:100%; background: no-repeat center; background-image: ' + spinnerPath + '"></div>'))
        .dialog({resizable: true, height: 700, width: 'auto', modal: true, title: "Select Device DataPoint", autoOpen: false});
    $.getScript(selfPath + 'includes.js', function(){
        eventHandler.getDataPointDevices(function (param) {fillDevices(_this, param, true);}, '*');
    });


    this.axisObj = [];
    this.axisDlg = $('<div />')
        .append($('<div></div>')
            .append($('<div style="float: left; width: 60px;"></div>')
                .append($('<label for="' + _this.id + '_axisId">ID:</label>'))
                .append($('<br/>'))
                .append($('<input id="' + _this.id + '_axisId" type="text" class="spinner"/>').addClass("ui-widget-content ui-corner-all").val(0))
            )
            .append($('<div style="float: left; width: 235px;"></div>')
                .append($('<label for="' + _this.id + '_axisTitle">Titel:</label>'))
                .append($('<br/>'))
                .append($('<input id="' + _this.id + '_axisTitle" style="width: 225px;" type="text"/>').addClass("ui-widget-content ui-corner-all"))
            )
            .append($('<div class="expertView" style="float: left; width: 130px;"></div>')
                .append($('<label for="' + _this.id + '_axisType">Typ:</label>'))
                .append($('<br/>'))
                .append($('<select id="' + _this.id + '_axisType" class="multiSelect" style="width: 120px;"/>')
                    .append($('<option value="linear">linear</option>'))
                    .append($('<option value="logarithmic">logarithmisch</option>'))
                )
            )
            .append($('<div style="float: left; width: 100px;"></div>')
                .append($('<label for="' + _this.id + '_axisOpposite">Rechts:</label>'))
                .append($('<div style="line-height:30px; height: 30px"></div>')
                    .append($('<input id="' + _this.id + '_axisOpposite" type="checkbox"/>'))


                    .append($('<button style="margin-left: 40px; height: 28px; top: -2px; width: 36px;"/>').button({icons: {primary: "ui-icon-circle-plus"}, text: false}).click(function () {
                        _this.axisObj[$('#' + _this.id + '_axisId').val()] =
                        {
                            title: $('#' + _this.id + '_axisTitle').val(),
                            type: $('#' + _this.id + '_axisType').val(),
                            opposite: ($('#' + _this.id + '_axisOpposite').prop("checked")) ? true : false
                        }
                        _this.axisReset();
                        _this.fillAxis();
                    }))
                )
            )
        )

        .append('<div class="ui-helper-clearfix" />')
        .append('<br/>')
        .append($('<table style="width: 100%" class="ui-widget ui-widget-content StateTable"></table>')
            .append($('<thead></thead>')
                .append($('<tr class="ui-widget-header"></tr>')
                    .append('<th style="width: 25px;">ID</th>')
                    .append('<th>Titel</th>')
                    .append('<th style="width: 90px;">Typ</th>')
                    .append('<th style="width: 60px;">Rechts</th>')
                    .append('<th style="width: 90px;">Entfernen</th>')
                )
            )
            .append($('<tbody id="axisHeader"></tbody>'))
        )
        .dialog({
            resizable: false,
            height: 'auto',
            width: '550',
            modal: true,
            title: "Werte ersetzen",
            autoOpen: false,
            open: function () {
                _this.axisObj = hcs_parseJSON(_this.config.yAxis);
                _this.axisReset();
                _this.fillAxis();
            },
            buttons: {
                "Abbrechen": function () {
                    $(this).dialog('close');
                },
                "Speichern": function () {
                    var axisString = JSON.stringify(_this.axisObj);

                    $('#' + _this.id + '_yAxis').val(axisString);
                    _this.config.yAxis = axisString;
                    $(this).dialog('close');
                }
            }
        });
    this.seriesObj = [];
    this.seriesDlg = $('<div />')
        .append($('<div></div>')
            .append($('<div></div>')
                .append($('<div style="float: left; width: 60px;"></div>')
                    .append($('<label for="' + _this.id + '_ccu">CCU:</label>'))
                    .append($('<br/>'))
                    .append($('<input id="' + _this.id + '_ccu" style="width: 30px;" type="text" disabled/>').addClass("ui-widget-content ui-corner-all").val(this.config.ccu).blur(function () {
                        _this.config.ccu = $(this).val();
                    }))
                )
                .append($('<div style="float: left; width: 160px;"></div>')
                    .append($('<label for="' + _this.id + '_adr">Address:</label>'))
                    .append($('<br/>'))
                    .append($('<input id="' + _this.id + '_adr" style="width: 130px;" type="text" disabled/>').addClass("ui-widget-content ui-corner-all").val(this.config.adr).blur(function () {
                        _this.config.adr = $(this).val();
                    }))
                )
                .append($('<div style="float: left; width: 180px;"></div>')
                    .append($('<label for="' + _this.id + '_dp">DataPoint:</label>'))
                    .append($('<br/>'))
                    .append($('<input id="' + _this.id + '_dp" style="width: 150px;" type="text" disabled/>').addClass("ui-widget-content ui-corner-all"))
                )
                .append($('<div style="float: left; width: 210px;"></div>')
                    .append($('<br/>'))
                    .append($('<button>Select Device DataPoint</button>').button().click(function () {
                        $(_this.devicesDlg).dialog('open');
                    }))
                )
            )
            .append($('<div class="ui-helper-clearfix" />'))
            .append($('<div style="float: left; width: 60px;"></div>')
                .append($('<label for="' + _this.id + '_seriesId">ID:</label>'))
                .append($('<br/>'))
                .append($('<input id="' + _this.id + '_seriesId" type="text" class="spinner"/>').addClass("ui-widget-content ui-corner-all").val(0))
            )
            .append($('<div style="float: left; width: 330px;"></div>')
                .append($('<label for="' + _this.id + '_seriesName">Name:</label>'))
                .append($('<br/>'))
                .append($('<input id="' + _this.id + '_seriesName" style="width: 300px;" type="text"/>').addClass("ui-widget-content ui-corner-all"))
            )
            .append($('<div style="float: left; width: 130px;"></div>')
                .append($('<label for="' + _this.id + '_seriesColor">Farbe:</label>'))
                .append($('<br/>'))
                .append($('<input id="' + _this.id + '_seriesColor" type="text" class="colorPickerNoOp"/>').addClass("ui-widget-content ui-corner-all")).val(_this.standardColor)
            )
            .append($('<div class="ui-helper-clearfix" />'))
            .append($('<div style="float: left; width: 60px;"></div>')
                .append($('<label for="' + _this.id + '_seriesStep">Schritt:</label>'))
                .append($('<br/>'))
                .append($('<input id="' + _this.id + '_seriesStep" type="checkbox"/>'))
            )
            .append($('<div style="float: left; width: 130px;"></div>')
                .append($('<label for="' + _this.id + '_seriesType">Typ:</label>'))
                .append($('<br/>'))
                .append($('<select id="' + _this.id + '_seriesType" class="multiSelect" style="width: 100px;"/>')
                    .append($('<option value="line">line</option>'))
                    .append($('<option value="spline">spline</option>'))
                    .append($('<option value="area">area</option>'))
                    .append($('<option value="areaspline">areaspline</option>'))
                    .append($('<option value="column">column</option>'))
                )
            )
            .append($('<div style="float: left; width: 370px;"></div>')
                .append($('<label for="' + _this.id + '_seriesAxis">Y-Achse:</label>'))
                .append($('<div style="line-height:30px; height: 30px"></div>')
                    .append($('<select id="' + _this.id + '_seriesAxis" class="multiSelect" style="width: 200px;"/>'))

                    .append($('<button id="' + _this.id + '_seriesAddButton" style="margin-left: 10px; height: 28px; top: -2px;">speichern</button>').button({icons: {primary: "ui-icon-circle-plus"}}).click(function () {
                        _this.seriesObj[$('#' + _this.id + '_seriesId').val()] =
                        {
                            name: $('#' + _this.id + '_seriesName').val(),
                            color: ($('#' + _this.id + '_seriesColor').val() != '') ? $('#' + _this.id + '_seriesColor').val() : _this.standardColor,
                            step: ($('#' + _this.id + '_seriesStep').prop("checked")) ? true : false,
                            type: $('#' + _this.id + '_seriesType').val(),
                            yAxis: ($('#' + _this.id + '_seriesAxis').val() != null) ? $('#' + _this.id + '_seriesAxis').val() : '',
                            ccu: $('#' + _this.id + '_ccu').val(),
                            adr: $('#' + _this.id + '_adr').val(),
                            dp: $('#' + _this.id + '_dp').val()
                        }
                        _this.seriesReset();
                        _this.fillSeries();
                    }))
                )
            )
        )

        .append('<div class="ui-helper-clearfix" />')
        .append('<br/>')
        .append($('<table style="width: 100%" class="ui-widget ui-widget-content StateTable"></table>')
            .append($('<thead></thead>')
                .append($('<tr class="ui-widget-header"></tr>')
                    .append('<th style="width: 25px;">ID</th>')
                    .append('<th>Name</th>')
                    .append('<th style="width: 40px;">Farbe</th>')
                    .append('<th style="width: 50px;">Schritt</th>')
                    .append('<th style="width: 70px;">Typ</th>')
                    .append('<th style="width: 40px;">Achse</th>')
                    .append('<th style="width: 30px;">CCU</th>')
                    .append('<th style="width: 110px;">Address</th>')
                    .append('<th>DataPoint</th>')
                    .append('<th style="width: 70px;">Entfernen</th>')
                )
            )
            .append($('<tbody id="seriesHeader"></tbody>'))
        )
        .dialog({
            resizable: true,
            height: 'auto',
            width: 'auto',
            modal: true,
            title: "Werte ersetzen",
            autoOpen: false,
            open: function () {
                _this.seriesObj = hcs_parseJSON(_this.config.series);
                _this.seriesReset();
                _this.fillSeries();
            },
            buttons: {
                "Abbrechen": function () {
                    $(this).dialog('close');
                },
                "Speichern": function () {
                    var seriesString = JSON.stringify(_this.seriesObj);

                    $('#' + _this.id + '_series').val(seriesString);
                    _this.config.series = seriesString;
                    $(this).dialog('close');
                }
            }
        });

    $('.colorPickerNoOp').colorpicker({
        colorFormat: ['#HEX3', '#HEX'],
        regional: 'local',
        showOn: 'span',
        okOnEnter: true
    });

    $('#' + _this.id + '_seriesStep').checkbox();
    $('#' + _this.id + '_axisOpposite').checkbox();
}

HCS_Diagrams_Chart_CONFIG.prototype = new Widget_CONFIG;
HCS_Diagrams_Chart_CONFIG.prototype.constructor = HCS_Diagrams_Chart_CONFIG;


HCS_Diagrams_Chart_CONFIG.prototype.fillAxis = function () {
    var _this = this;
    $('#axisHeader').empty();
    for (var key in this.axisObj){
        $('#axisHeader').append($('<tr>' +
            '<th class="axisId axisItem">' + key + '</th>' +
            '<th class="axisTitle axisItem">' + this.axisObj[key].title + '</th>' +
            '<th class="axisType axisItem">' + this.axisObj[key].type + '</th>' +
            '<th class="axisOpposite axisItem">' + this.axisObj[key].opposite + '</th>' +
            '<th><a class="axisDelete">Entfernen</a></th>' +
            '</tr>'));
    }
    $('.axisDelete').button({
        icons: {primary: "ui-icon-closethick"},
        text: false
    }).click(function () {
        delete _this.axisObj[$(this).parent().siblings('.axisId').text()];
        _this.fillAxis();
    });
    $('.axisItem').click(function () {
        $('#' + _this.id + '_axisId').spinner("value", $(this).parent().children(".axisId").html());
        $('#' + _this.id + '_axisTitle').val($(this).parent().children(".axisTitle").html());
        $('#' + _this.id + '_axisType').val($(this).parent().children(".axisType").html()).multiselect('refresh');
        $('#' + _this.id + '_axisOpposite').checkbox( ($(this).parent().children(".axisOpposite").html() == 'true') ? 'check' : 'uncheck' );

    });
};
HCS_Diagrams_Chart_CONFIG.prototype.axisReset = function () {
    var maxId = -1;
    for (var key in this.axisObj){
        maxId = key > maxId ? key : maxId;
    }
    $('#' + this.id + '_axisId').spinner("value", parseInt(maxId)+1);
    $('#' + this.id + '_axisTitle').val('');
    $('#' + this.id + '_axisType').val('linear').multiselect('refresh');
    $('#' + this.id + '_axisOpposite').checkbox('uncheck');
};

HCS_Diagrams_Chart_CONFIG.prototype.fillSeries = function () {
    var _this = this;
    $('#seriesHeader').empty();
    for (var key in this.seriesObj){
        $('#seriesHeader').append($('<tr>' +
            '<th class="seriesId seriesItem">' + key + '</th>' +
            '<th class="seriesName seriesItem">' + this.seriesObj[key].name + '</th>' +
            '<th class="seriesColor seriesItem" style="background-color: '+this.seriesObj[key].color+' !important;"></th>' +
            '<th class="seriesStep seriesItem">' + this.seriesObj[key].step + '</th>' +
            '<th class="seriesType seriesItem">' + this.seriesObj[key].type + '</th>' +
            '<th class="seriesYAxis seriesItem">' + this.seriesObj[key].yAxis + '</th>' +
            '<th class="seriesCcu seriesItem">' + this.seriesObj[key].ccu + '</th>' +
            '<th class="seriesAdr seriesItem">' + this.seriesObj[key].adr + '</th>' +
            '<th class="seriesDp seriesItem">' + this.seriesObj[key].dp + '</th>' +
            '<th><a class="seriesDelete">Entfernen</a></th>' +
            '</tr>'));
    }
    $('.seriesDelete').button({
        icons: {primary: "ui-icon-closethick"},
        text: false
    }).click(function () {
        delete _this.seriesObj[$(this).parent().siblings('.seriesId').text()];
        _this.seriesReset();
        _this.fillSeries();
    });
    $('.seriesItem').click(function () {
        $('#' + _this.id + '_seriesId').spinner("value", $(this).parent().children(".seriesId").html());
        $('#' + _this.id + '_seriesName').val($(this).parent().children(".seriesName").html());
        $('#' + _this.id + '_seriesColor').colorpicker('setColor', $(this).parent().children(".seriesColor").css('background-color'));
        $('#' + _this.id + '_seriesStep').checkbox( ($(this).parent().children(".seriesStep").html() == 'true') ? 'check' : 'uncheck' );
        $('#' + _this.id + '_seriesType').val($(this).parent().children(".seriesType").html()).multiselect('refresh');
        $('#' + _this.id + '_seriesAxis').val($(this).parent().children(".seriesYAxis").html()).multiselect('refresh');
        $('#' + _this.id + '_ccu').val($(this).parent().children(".seriesCcu").html());
        $('#' + _this.id + '_adr').val($(this).parent().children(".seriesAdr").html());
        $('#' + _this.id + '_dp').val($(this).parent().children(".seriesDp").html());
        $('#' + _this.id + '_seriesAddButton > .ui-button-text').empty().append('speichern');
    });
};
HCS_Diagrams_Chart_CONFIG.prototype.seriesReset = function () {
    var _this = this;
    var maxId = -1;
    for (var key in this.seriesObj){
        maxId = key > maxId ? key : maxId;
    }

    _this.axisObj = hcs_parseJSON(_this.config.yAxis);
    var size = 0, key;
    for (key in _this.axisObj) {
        if (_this.axisObj.hasOwnProperty(key)) size++;
    }

    if (size <= 0) {
        $('#' + _this.id + '_seriesAxis').empty().append($('<option value="">Standard</option>')).val('').multiselect('refresh');
    } else {
        $('#' + _this.id + '_seriesAxis').empty();
        var first = '';
        for (key in _this.axisObj) {
            if (first == ''){
                first = key;
            }
            $('#' + _this.id + '_seriesAxis').append($('<option value="' + key + '">' + _this.axisObj[key].title + '</option>'));
        }
        $('#' + _this.id + '_seriesAxis').val(first).multiselect('refresh');
    }

    $('#' + _this.id + '_seriesId').spinner("value", parseInt(maxId)+1);
    $('#' + _this.id + '_seriesName').val('');
    $('#' + _this.id + '_seriesColor').colorpicker('setColor', _this.standardColor);
    $('#' + _this.id + '_seriesType').val('line').multiselect('refresh');
    $('#' + _this.id + '_seriesStep').checkbox('uncheck');
    $('#' + _this.id + '_ccu').val('');
    $('#' + _this.id + '_adr').val('');
    $('#' + _this.id + '_dp').val('');
    $('#' + _this.id + '_seriesAddButton > .ui-button-text').empty().append('hinzufügen');
};

HCS_Diagrams_Chart_CONFIG.prototype.createContent = function () {

    this.config.liveUpdate = true;
    var _this = this;
    $(this.container)
        .append($('<div></div>')
            .append($('<label>Diagramm-Titel:</label>')
                .append($('<br/>'))
                .append($('<input type="text" style="width:400px"/>').addClass("ui-widget-content ui-corner-all").val(this.config.title).blur(function () {
                    _this.config.title = $(this).val();
                }))
            )
        )
        .append($('<div></div>')
            .append($('<label>Diagramm-Untertitel:</label>')
                .append($('<br/>'))
                .append($('<input type="text" style="width:400px"/>').addClass("ui-widget-content ui-corner-all").val(this.config.subTitle).blur(function () {
                    _this.config.subTitle = $(this).val();
                }))
            )
        )
        .append($('<div></div>')
            .append($('<label for="' + _this.id + '_intervalSelect">Alter der Daten: </label>'))
            .append($('<br />'))
            .append($('<select id="' + _this.id + '_intervalSelect" class="multiSelect" style="width: 120px;"/>')
                .append($('<option value="">Alle</option>'))
                .append($('<option value="60">1 Minute</option>'))
                .append($('<option value="180">3 Minuten</option>'))
                .append($('<option value="300">5 Minuten</option>'))
                .append($('<option value="600">10 Minuten</option>'))
                .append($('<option value="900">15 Minuten</option>'))
                .append($('<option value="1800">30 Minuten</option>'))
                .append($('<option value="2700">45 Minuten</option>'))
                .append($('<option value="3600">1 Stunde</option>'))
                .append($('<option value="10800">3 Stunden</option>'))
                .append($('<option value="21600">6 Stunden</option>'))
                .append($('<option value="43200">12 Stunden</option>'))
                .append($('<option value="86400">1 Tag</option>'))
                .append($('<option value="259200">3 Tage</option>'))
                .append($('<option value="604800">7 Tage</option>'))
                .append($('<option value="1209600">14 Tage</option>'))
                .append($('<option value="2678400">1 Monat</option>'))
                .append($('<option value="8035200">3 Monate</option>'))
                .append($('<option value="16070400">6 Monate</option>'))
                .append($('<option value="32140800">1 Jahr</option>'))
                .val(this.config.interval).bind("multiselectclick", function(event, ui){
                    _this.config.interval = ui.value;
                    $('#'+_this.id + '_interval').val(ui.value);
                }).blur(function () {
                    _this.config.interval = $(this).val();
                    $('#'+_this.id + '_interval').val($(this).val());
                })
            )
            .append('<span class="expertView"> = </span>')
            .append($('<input class="expertView" type="text" id="' + _this.id + '_interval" style="width:180px"/>').addClass("ui-widget-content ui-corner-all").val(this.config.interval).blur(function () {
                _this.config.interval = $(this).val();
            }))
            .append('<span class="expertView"> Sekunden</span>')
        )
        .append($('<div class="expertView"></div>')
            .append($('<label>Live-Updates:</label>')
                .append($('<input type="checkbox" id="' + _this.id + '_liveUpdate"/>').click(function () {
                    _this.config.liveUpdate = ($(this).prop("checked")) ? true : false;
                }))
            )
            .append($('<label style="margin-left: 20px;">Export Knopf:</label>')
                .append($('<input type="checkbox" id="' + _this.id + '_exportEnabled"/>').click(function () {
                    _this.config.exportEnabled = ($(this).prop("checked")) ? true : false;
                }))
            )
        )
        .append($('<div class="expertView"></div>')
            .append($('<label>minimaler Pixelabstand zwischen 2 Werten: </label>')
                .append($('<input type="text" class="spinner"/>').addClass("ui-widget-content ui-corner-all").val(this.config.groupingDistance).blur(function () {
                    _this.config.groupingDistance = $(this).val();
                }))
            )
        )
        .append($('<div></div>')
            .append($('<label for="' + _this.id + '_yAxisButton">Y-Achsen:</label>'))
            .append($('<div style="line-height:30px; height: 30px"></div>')
                .append($('<input id="' + _this.id + '_yAxis" type="text" disabled style="width:355px"/>').addClass("ui-widget-content ui-corner-all").val(this.config.yAxis).blur(function () {
                    _this.config.yAxis = $(this).val();
                }))
                .append($('<button id="' + _this.id + '_yAxisButton" style="margin-left: 5px; height: 28px; top: -2px; width: 36px;"/>').button().button("option", "icons", {primary: 'ui-icon-pencil'}).click(function () {
                    _this.axisDlg.dialog('open');
                }))
            )
        )
        .append($('<div></div>')
            .append($('<label for="' + _this.id + '_seriesButton">Datenreihen:</label>'))
            .append($('<div style="line-height:30px; height: 30px"></div>')
                .append($('<input id="' + _this.id + '_series" type="text" disabled style="width:355px"/>').addClass("ui-widget-content ui-corner-all").val(this.config.series).blur(function () {
                    _this.config.series = $(this).val();
                }))
                .append($('<button id="' + _this.id + '_seriesButton" style="margin-left: 5px; height: 28px; top: -2px; width: 36px;"/>').button().button("option", "icons", {primary: 'ui-icon-pencil'}).click(function () {
                    _this.seriesDlg.dialog('open');
                }))
            )
        )
    ;
    $('#' + _this.id + '_liveUpdate').checkbox().checkbox(boolToChecked(this.config.liveUpdate));
    $('#' + _this.id + '_exportEnabled').checkbox().checkbox(boolToChecked(this.config.exportEnabled));
    $('.spinner').spinner({min: 0, incremental: true}).removeClass("ui-widget-content ui-corner-all").css("width", 20);
    $('.multiSelect').multiselect({multiple: false, header: false, selectedList: 1, minWidth: 'auto', height: 'auto', noneSelectedText: ' '});
};



HCS_Diagrams_Chart_CONFIG.prototype.createComment = function () {
    return this.config.title;
};