function Widget(containerElement, eventHandler, globals, configurationObj, selfPath) {
    // this constructor style must be implemented by every widget
}

function Widget_CONFIG(containerElement, eventHandler, globals, configurationObj, selfPath) {
    // this constructor style must be implemented by every widget config
}

Widget_CONFIG.prototype.setConfig = function (configJSON) {
    $.extend(this.config, hcs_parseJSON(configJSON));
    //this.config = hcs_parseJSON(configJSON);
};

Widget_CONFIG.prototype.createConfigJSON = function () {
    return JSON.stringify(this.config);
};

Widget_CONFIG.prototype.createComment = function () {
    return '';
};

Widget_CONFIG.prototype.createContent = function () {
    // widget config content
};


// EventHandler - Subscribers
function Subscriber(callbackFunction, interests) {
    this.callback = callbackFunction;
    this.interests = interests;
}
Subscriber.prototype.getCallback = function () {
    return this.callback;
};
Subscriber.prototype.getInterests = function () {
    return this.interests;
};


// EventHandler
function EventHandler(globals) {
    // this is the constructor for the eventHandler
    this.globals = globals;
    this.subscribers = new Array();
    this.id = randomString(16,'aA#');
    this.count = 0;
    this.xhr = null;
    this.rate = parseInt(parseFloat(globals["REFRESH_RATE"]) * 1000);
    this.timeout = parseInt(globals["REFRESH_TIMEOUT"] * 1000);
    this.asyncUrl = this.globals["rootPath"] + "control.php" + this.globals["SID"];

}
EventHandler.prototype.register = function (callbackFunction, interests) {
    // a new subscriber will be pushed upon the subscribers array
    this.subscribers.push(new Subscriber(callbackFunction, interests));
};

EventHandler.prototype.getPages = function (callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getPages'},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getControllableDevices = function (callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getDatapoints', ops: 2},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getDataPointDevices = function (callback, datapoints) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getDatapoints', dps: datapoints},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getFolderContent = function (callback, folder) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getFolderContent', folder: folder},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getRooms = function (callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getRooms'},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getCrafts = function (callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getCrafts'},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getPrograms = function (callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getPrograms'},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getSystemVariables = function (callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getSystemVariables'},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getSystemVariablesForCcuId = function (callback, id) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getSystemVariables', ccu: id},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.getDevices = function (callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getDevices'},
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.setValueSn = function (ccu, adr, dp, val) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'setValueBySn', ccu: ccu, adr: adr, dp: dp, val: val });
    this.informSubscribers([{"ccu": ccu, "adr": adr, "dp": dp, "val": val}]);
}

EventHandler.prototype.getValueSn = function (ccu, adr, dp, callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getValueBySn', ccu: ccu, adr: adr, dp: dp },
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};


EventHandler.prototype.setValueName = function (ccu, adr, dp, val) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'setValue', ccu: ccu, adr: adr, dp: dp, val: val });
}

EventHandler.prototype.getValueName = function (ccu, adr, dp, callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getValue', ccu: ccu, adr: adr, dp: dp },
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.setSystemVariable = function (ccu, sysVar, val) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'setSysVar', ccu: ccu, var: sysVar, val: val });
}

EventHandler.prototype.getSystemVariable = function (ccu, sysVar, callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getSysVar', ccu: ccu, var: sysVar },
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

EventHandler.prototype.executeProgram = function (ccu, prog) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'exec', ccu: ccu, prog: prog});
}


EventHandler.prototype.checkSubscribers = function (fullUpdate) {
    var _this = this;
    if (fullUpdate) {
        var postData = { cmd: 'states', page: _this.globals["PAGE"], id: _this.id, f: 1};

    } else {
        var postData = { cmd: 'states', page: _this.globals["PAGE"], id: _this.id};

    }
    this.xhr = $.ajax({
        type: "POST",
        url: _this.globals["rootPath"] + "states.php",
        data: postData,
        timeout: _this.timeout,
        async: true,
        cache: false,
        error: function (jqXHRObj, tStatus, eError) {
            if (tStatus === "timeout") {
                // timeout
                setTimeout(function () {_this.checkSubscribers(true)}, 30000);
            } else {
                // error!
                setTimeout(function () {_this.checkSubscribers(true)}, 5000);
            }
        },
        success: function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                _this.informSubscribers(result["RETURN"]);
                _this.count++;
                if (_this.count > 100){
                    _this.count = 0;
                    setTimeout(function () {_this.checkSubscribers(true)}, _this.rate);
                } else {
                    setTimeout(function () {_this.checkSubscribers()}, _this.rate);
                }
            } else {
                // error!
                setTimeout(function () {_this.checkSubscribers(true)}, 60000);
            }

        }
    });

};

EventHandler.prototype.informSubscribers = function(data){
    var valCcu, valAddress, valDp, val, interests;
    var subscribers = this.subscribers;

    for (var kVal in data) {
        valCcu = data[kVal]["ccu"];
        valAddress = data[kVal]["adr"];
        valDp = data[kVal]["dp"];
        val = data[kVal]["val"];
        for (var kSub in subscribers) {
            interests = subscribers[kSub].getInterests();

            for (var kInter in interests) {
                if ((valCcu == interests[kInter]["ccu"]) && (valAddress == interests[kInter]["adr"]) && (valDp == interests[kInter]["dp"])) {
                    subscribers[kSub].getCallback()(hcs_parseJSON('{"' + kInter + '":"' + val + '"}'));
                }
            }
        }
    }
}

EventHandler.prototype.getHistoryValueBySn = function (ccu, adr, dp, since, callback) {
    var _this = this;
    $.post(_this.asyncUrl, { page: _this.globals["PAGE"], cmd: 'getHistoryValueBySn', ccu: ccu, adr: adr, dp: dp, since: since },
        function (data) {
            var result = hcs_parseJSON(data);
            if (!("ERROR" in result)) {
                callback(result["RETURN"]);
            } else {
                // error
            }
        });
};

