<?php

// <editor-fold defaultstate="collapsed" desc="CALLABLE PROTECTION">
// Make file not callable directly
if (!defined('HCS_CALLABLE_FILE')) {
    header('HTTP/1.0 404 Not Found');
    die('Do not access this file directly.');
    exit;
}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="TEMPLATE CLASS">
/**
 * NOTICE: This class was not written for the Home Control Suite project.
 * It was originally written for the phpBB project and used here
 * under the GNU GPL.
 *
 * It has been modified and adapted to fit the HCS project's needs.
 *
 * The phpBB Group's original notice appears below
 */
/* * *************************************************************************
 *                              template.php
 *                            -------------------
 *   begin                : Saturday, Feb 13, 2001
 *   copyright            : (C) 2001 The phpBB Group
 *   email                : support@phpbb.com
 *
 *   $Id: class_template.php 7443 2010-03-11 12:22:55Z wallenium $
 *
 * ************************************************************************* */

/* * *************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 * ************************************************************************* */

/*
  Template class.

  Nathan Codding - Original version design and implementation
  Crimsonbane - Initial caching proposal and work
  psoTFX - Completion of file caching, decompilation routines and implementation of
  conditionals/keywords and associated changes

  The interface was inspired by PHPLib templates,  and the template file (formats are
  quite similar)

  The keyword/conditional implementation is currently based on sections of code from
  the Smarty templating engine (c) 2001 ispi of Lincoln, Inc. which is released
  (on its own and in whole) under the LGPL. Section 3 of the LGPL states that any code
  derived from an LGPL application may be relicenced under the GPL, this applies
  to this source
 */

class Template
{

    var $classname = 'Template';
    // variable that holds all the data we'll be substituting into
    // the compiled templates. Takes form:
    // --> $this->_tpldata[block.][iteration#][child.][iteration#][child2.][iteration#][variablename] == value
    // if it's a root-level variable, it'll be like this:
    // --> $this->_tpldata[.][0][varname] == value
    var $_tpldata = array();
    // Root dir and hash of filenames for each template handle.
    var $root = '';
    var $files = array();
    // this will hash handle names to the compiled/uncompiled code for that handle.
    var $compiled_code = array();
    var $uncompiled_code = array();
    // Various counters and storage arrays
    var $block_names = array();
    var $block_else_level = array();
    var $include_counter = 1;
    var $block_nesting_level = 0;

    function setTemplate($param_template = '', $param_rootDirectory = '')
    {
        global $HCS_ROOTPATH;

        if ($param_rootDirectory == '') {
            $this->root = $HCS_ROOTPATH . 'templates/' . $param_template;
        } else {
            if (substr($param_rootDirectory, strlen($param_rootDirectory) - 1) != '/')
                $param_rootDirectory .= '/';
            $this->root = $HCS_ROOTPATH . $param_rootDirectory . $param_template;
        }

        if (defined('PLUGIN')) {
            $this->cachedir = $HCS_ROOTPATH . 'templates/cache/' . $param_template . '/' . PLUGIN . '/';
            $admin_folder = $HCS_ROOTPATH . 'templates/cache/' . $param_template . '/' . PLUGIN . '/admin/';
        } else {
            $this->cachedir = $HCS_ROOTPATH . 'templates/cache/' . $param_template . '/';
            $admin_folder = $HCS_ROOTPATH . 'templates/cache/' . $param_template . '/admin/';
        }

        if (!file_exists($this->cachedir)) {
            @umask(0);
            if (!@mkdir($this->cachedir, 0777, true)) {
                // To avoid confusing the user, we're going to replace ./../../ (etc) with just a /
                $cachedir = str_replace($HCS_ROOTPATH, '/', $this->cachedir);
                $cache_parent = preg_replace('/^(.+)\/.+\/$/', '\1/', $this->cachedir);
                //trigger_error('Could not create template cache dir: ' . $cachedir .' - chmod ' . $cache_parent . ' to 777', E_USER_WARNING);
                unset($cachedir, $cache_parent);
            }
        }

        // Special handling for admin folder
        if (!file_exists($admin_folder)) {
            @umask(0);
            if (!@mkdir($admin_folder, 0777, true)) {
                //trigger_error('Could not create template cache dir: ' . $admin_folder . ' - chmod ' . str_replace('/admin/', '', $admin_folder) . ' to 777', E_USER_WARNING);
                unset($admin_folder);
            }
        }

        return true;

    }

    /**
     * Sets the template filenames for handles. $filename_array
     * should be a hash of handle => filename pairs.
     */
    function setFilenames($param_filenameArray)
    {
        if (!is_array($param_filenameArray)) {
            return false;
        }

        $template_names = '';
        foreach ($param_filenameArray as $handle => $filename) {
            if (empty($filename)) {
                trigger_error("Template error - Empty filename specified for $handle", E_USER_ERROR);
            }

            $this->filename[$handle] = $filename;
            $this->files[$handle] = $this->makeFilename($filename);
        }

        return true;

    }

    /**
     * Generates a full path+filename for the given filename, which can either
     * be an absolute name, or a name relative to the rootdir for this Template
     * object.
     */
    function makeFilename($param_filename)
    {
        // Check if it's an absolute or relative path.
        return (substr($param_filename, 0, 1) != '/') ? $this->root . '/' . $param_filename : $param_filename;

    }

    /**
     * If not already done, load the file for the given handle and populate
     * the uncompiled_code[] hash with its code. Do not compile.
     */
    function loadFile($param_handle)
    {
        // If the file for this handle is already loaded and compiled, do nothing.
        if (!empty($this->uncompiled_code[$param_handle])) {
            return true;
        }

        // If we don't have a file assigned to this handle, die.
        // TODO: Give a better error. Define a language string and pass it all the values that would be useful (theme name, file, handle, "using a plugin?", etc.)
        if (!isset($this->files[$param_handle])) {
            trigger_error("Template->loadfile(): No file specified for handle \"$param_handle\"", E_USER_ERROR);
        }

        if (!($fp = @fopen($this->files[$param_handle], 'r'))) {
            trigger_error("Template->loadfile(): Error - the handle \"$param_handle\" template file or directory does not exist or is empty", E_USER_ERROR);
        }

        $str = '';
        $str = fread($fp, filesize($this->files[$param_handle]));
        @fclose($fp);

        $this->uncompiled_code[$param_handle] = trim($str);

        return true;

    }

    /**
     * Destroys this template object. Should be called when you're done with it, in order
     * to clear out the template data so you can load/parse a new template set.
     */
    function destroy()
    {
        $this->_tpldata = array();

    }

    function security()
    {
        return true;

    }

    /**
     * Load the file for the handle, compile the file,
     * and run the compiled code. This will print out
     * the results of executing the template.
     */
    function display($param_handle)
    {

        $_str = '';

        if (!$this->compile_load($_str, $param_handle, true)) {
            global $user;

            if (!$this->loadFile($param_handle)) {
                trigger_error("Template->pparse(): Couldn't load template file for handle $param_handle", E_USER_ERROR);
            }

            // Actually compile the code now.
            $this->compiled_code[$param_handle] = $this->compile($this->uncompiled_code[$param_handle]);
            $this->compile_write($param_handle, $this->compiled_code[$param_handle]);

            @eval($this->compiled_code[$param_handle]);
        }

        return true;

    }

    /**
     * Inserts the uncompiled code for $handle as the
     * value of $varname in the root-level. This can be used
     * to effectively include a template in the middle of another
     * template.
     * Note that all desired assignments to the variables in $handle should be done
     * BEFORE calling this function.
     */
    function assignVariableFromHandle($param_variableName, $param_handle)
    {
        $_str = '';

        if (!($this->compile_load($_str, $param_handle, false))) {
            if (!$this->loadFile($param_handle)) {
                trigger_error("Template->pparse(): Couldn't load template file for handle $param_handle", E_USER_ERROR);
            }

            $code = $this->compile($this->uncompiled_code[$param_handle], true, '_str');
            $this->compile_write($param_handle, $code);

            // evaluate the variable assignment.
            eval($code);
        }

        // assign the value of the generated variable to the given varname.
        $this->assignVariable($param_variableName, $_str);

        return true;

    }

    function assignFromInclude($param_filename)
    {
        $handle = 'include_' . $this->include_counter++;

        $this->filename[$handle] = $param_filename;
        $this->files[$handle] = $this->makeFilename($param_filename);
        $_str = '';

        if (!($this->compile_load($_str, $handle, false))) {
            global $user;

            if (!$this->loadFile($handle)) {
                trigger_error("Template->pparse(): Couldn't load template file for handle $handle", E_USER_ERROR);
            }

            $this->compiled_code[$handle] = $this->compile($this->uncompiled_code[$handle]);
            $this->compile_write($handle, $this->compiled_code[$handle]);

            eval($this->compiled_code[$handle]);
        }

    }

    /**
     * Root-level variable assignment. Adds to current assignments, overriding
     * any existing variable assignment with the same name.
     */
    function assignVariables($param_variableArray)
    {
        foreach ($param_variableArray as $key => $val) {
            $this->_tpldata['.'][0][$key] = $val;
        }

        return true;

    }

    /**
     * Root-level variable assignment. Adds to current assignments, overriding
     * any existing variable assignment with the same name.
     */
    function assignVariable($param_variableName, $param_variableValue)
    {
        $this->_tpldata['.'][0][$param_variableName] = $param_variableValue;

        return true;

    }

    /**
     * Block-level variable assignment. Adds a new block iteration with the given
     * variable assignments. Note that this should only be called once per block
     * iteration.
     */
    function assignBlockVariables($param_blockname, $param_variablesArray)
    {
        if (strpos($param_blockname, '.') !== false) {
            // Nested block.
            $blocks = explode('.', $param_blockname);
            $blockcount = sizeof($blocks) - 1;

            $str = '$this->_tpldata';
            for ($i = 0; $i < $blockcount; $i++) {
                $str .= '[\'' . $blocks[$i] . '.\']';
                eval('$lastiteration = sizeof(' . $str . ') - 1;');
                $str .= '[' . $lastiteration . ']';
            }

            // Now we add the block that we're actually assigning to.
            // We're adding a new iteration to this block with the given
            // variable assignments.
            $str .= '[\'' . $blocks[$blockcount] . '.\'][] = $vararray;';

            // Now we evaluate this assignment we've built up.
            $str = eval($str);

            $s_row_count = isset($str[$blocks[$blockcount]]) ? sizeof($str[$blocks[$blockcount]]) : 0;
            $param_variablesArray['S_ROW_COUNT'] = $s_row_count;

            // Assign S_FIRST_ROW
            if (!$s_row_count) {
                $param_variablesArray['S_FIRST_ROW'] = true;
            }

            // Now the tricky part, we always assign S_LAST_ROW and alter the entry before
            // This is much more clever than going through the complete template data on display (phew)
            $param_variablesArray['S_LAST_ROW'] = true;
            if ($s_row_count > 0) {
                unset($this->_tpldata[$blocks[$blockcount]][($s_row_count - 1)]['S_LAST_ROW']);
            }
        } else {
            // Top-level block.
            $s_row_count = (isset($this->_tpldata[$param_blockname . '.'])) ? sizeof($this->_tpldata[$param_blockname . '.']) : 0;
            $param_variablesArray['S_ROW_COUNT'] = $s_row_count;

            // Assign S_FIRST_ROW
            if (!$s_row_count) {
                $param_variablesArray['S_FIRST_ROW'] = true;
            }

            // We always assign S_LAST_ROW and remove the entry before
            $param_variablesArray['S_LAST_ROW'] = true;
            if ($s_row_count > 0) {
                unset($this->_tpldata[$param_blockname . '.'][($s_row_count - 1)]['S_LAST_ROW']);
            }

            // Add a new iteration to this block with the variable assignments we were given.
            $this->_tpldata[$param_blockname . '.'][] = $param_variablesArray;
        }

        return true;

    }

    /**
     * Compiles the given string of code, and returns
     * the result in a string.
     * If "do_not_echo" is true, the returned code will not be directly
     * executable, but can be used as part of a variable assignment
     * for use in assign_code_from_handle().
     *
     * Parts of this were inspired by Smarty
     */
    function compile($param_code, $param_doNotEcho = false, $param_returnVariable = '')
    {
        // Pull out all block/statement level elements and seperate
        // plain text
        preg_match_all('#<!-- PHP -->(.*?)<!-- ENDPHP -->#s', $param_code, $matches);
        $php_blocks = $matches[1];
        $param_code = preg_replace('#<!-- PHP -->(.*?)<!-- ENDPHP -->#s', '<!-- PHP -->', $param_code);

        preg_match_all('#<!-- (.*?) (.*?)?[ ]?-->#s', $param_code, $blocks);
        $text_blocks = preg_split('#<!-- (.*?) (.*?)?[ ]?-->#s', $param_code);
        for ($i = 0; $i < count($text_blocks); $i++) {
            $this->compileVariableTags($text_blocks[$i]);
        }

        $compile_blocks = array();

        for ($curr_tb = 0; $curr_tb < count($text_blocks); $curr_tb++) {
            if (isset($blocks[1][$curr_tb])) {
                switch ($blocks[1][$curr_tb]) {
                    case 'BEGIN':
                        $this->block_else_level[] = false;
                        $compile_blocks[] = '// BEGIN ' . $blocks[2][$curr_tb] . "\n" . $this->compileTag_Block($blocks[2][$curr_tb]);
                        break;
                    case 'BEGINELSE':
                        $this->block_else_level[sizeof($this->block_else_level) - 1] = true;
                        $compile_blocks[] = "// BEGINELSE\n}} else {\n";
                        break;
                    case 'END':
                        $compile_blocks[] = ((array_pop($this->block_else_level)) ? "}\n" : "}}\n") . '// END ' . array_pop($this->block_names) . "\n";
                        break;
                    case 'IF':
                        $compile_blocks[] = '// IF ' . $blocks[2][$curr_tb] . "\n" . $this->compileTag_If($blocks[2][$curr_tb], false);
                        break;
                    case 'ELSE':
                        $compile_blocks[] = "// ELSE\n} else {\n";
                        break;
                    case 'ELSEIF':
                        $compile_blocks[] = '// ELSEIF ' . $blocks[2][$curr_tb] . "\n" . $this->compileTag_If($blocks[2][$curr_tb], true);
                        break;
                    case 'ENDIF':
                        $compile_blocks[] = "// ENDIF\n}\n";
                        break;
                    case 'INCLUDE':
                        $compile_blocks[] = '// INCLUDE ' . $blocks[2][$curr_tb] . "\n" . $this->compileTag_Include($blocks[2][$curr_tb]);
                        break;
                    case 'INCLUDEPHP':
                        $compile_blocks[] = '// INCLUDEPHP ' . $blocks[2][$curr_tb] . "\n" . $this->compileTag_IncludePHP($blocks[2][$curr_tb]);
                        break;
                    case 'PHP':
                        $temp = '';
                        list(, $temp) = each($php_blocks);
                        $compile_blocks[] = "// PHP START\n" . $temp . "\n// PHP END\n";
                        break;
                    default:
                        $this->compileVariableTags($blocks[0][$curr_tb]);
                        $trim_check = trim($blocks[0][$curr_tb]);
                        $compile_blocks[] = (!$param_doNotEcho) ? ((!empty($trim_check)) ? 'echo \'' . $blocks[0][$curr_tb] . '\';' : '') : ((!empty($trim_check)) ? $blocks[0][$curr_tb] : '');
                        break;
                } // switch
            } // isset
        } // for

        $template_php = '';
        for ($i = 0; $i < count($text_blocks); $i++) {
            $trim_check_text = (isset($text_blocks[$i])) ? trim($text_blocks[$i]) : '';
            $trim_check_block = (isset($compile_blocks[$i])) ? trim($compile_blocks[$i]) : '';
            $template_php .= (!$param_doNotEcho) ? ((!empty($trim_check_text)) ? 'echo \'' . $text_blocks[$i] . '\';' : '') . ((!empty($compile_blocks[$i])) ? $compile_blocks[$i] : '') : ((!empty($trim_check_text)) ? $text_blocks[$i] . "\n" : '') . ((!empty($compile_blocks[$i])) ? $compile_blocks[$i] . "\n" : '');
        }

        return (!$param_doNotEcho) ? $template_php : '$' . $param_returnVariable . '.= \'' . $template_php . '\'';

    }

    function compileVariableTags(&$param_textBlocks)
    {
        // change template varrefs into PHP varrefs
        $varrefs = array();

        $param_textBlocks = str_replace('\\', '\\\\', $param_textBlocks);
        $param_textBlocks = str_replace('\'', '\\\'', $param_textBlocks);

        // This one will handle varrefs WITH namespaces
        preg_match_all('#\{(([a-z0-9\-_]+?\.)+?)([a-z0-9\-_]+?)\}#is', $param_textBlocks, $varrefs);

        for ($j = 0; $j < sizeof($varrefs[1]); $j++) {
            $namespace = $varrefs[1][$j];
            $varname = $varrefs[3][$j];
            $new = $this->generate_block_varref($namespace, $varname);

            $param_textBlocks = str_replace($varrefs[0][$j], $new, $param_textBlocks);
        }

        // This will handle the remaining root-level varrefs
        $param_textBlocks = preg_replace('#\{L_([a-z0-9\-_]*?)\}#is', "' . ((isset(\$this->_tpldata['.'][0]['L_\\1'])) ? \$this->_tpldata['.'][0]['L_\\1'] : ((isset(\$user->lang['\\1'])) ? \$user->lang['\\1'] : '{ ' . ucfirst(strtolower(str_replace('_', ' ', '\\1'))) . ' 	}')) . '", $param_textBlocks);
        $param_textBlocks = preg_replace('#\{([a-z0-9\-_]*?)\}#is', "' . ((isset(\$this->_tpldata['.'][0]['\\1'])) ? \$this->_tpldata['.'][0]['\\1'] : '') . '", $param_textBlocks);

        return;

    }

    function compileTag_Block($param_tagArguments)
    {
        $tag_template_php = '';
        array_push($this->block_names, $param_tagArguments);

        if (sizeof($this->block_names) < 2) {
            // Block is not nested.
            $tag_template_php = '$_' . $param_tagArguments . '_count = (isset($this->_tpldata[\'' . $param_tagArguments . '.\'])) ?  sizeof($this->_tpldata[\'' . $param_tagArguments . '.\']) : 0;' . "\n";
            $tag_template_php .= 'if ($_' . $param_tagArguments . '_count) {' . "\n";
            $tag_template_php .= 'for ($_' . $param_tagArguments . '_i = 0; $_' . $param_tagArguments . '_i < $_' . $param_tagArguments . '_count; $_' . $param_tagArguments . '_i++)';
        } else {
            // This block is nested.
            // Generate a namespace string for this block.
            $namespace = implode('.', $this->block_names);

            // Get a reference to the data array for this block that depends on the
            // current indices of all parent blocks.
            $varref = $this->generate_block_data_ref($namespace, false);

            // Create the for loop code to iterate over this block.
            $tag_template_php = '$_' . $param_tagArguments . '_count = (isset(' . $varref . ')) ? sizeof(' . $varref . ') : 0;' . "\n";
            $tag_template_php .= 'if ($_' . $param_tagArguments . '_count) {' . "\n";
            $tag_template_php .= 'for ($_' . $param_tagArguments . '_i = 0; $_' . $param_tagArguments . '_i < $_' . $param_tagArguments . '_count; $_' . $param_tagArguments . '_i++)';
        }

        $tag_template_php .= "\n{\n";

        return $tag_template_php;

    }

    //
    // Compile IF tags - much of this is from Smarty with
    // some adaptions for our block level methods
    //
    function compileTag_If($param_tagArguments, $param_elseIf)
    {
        /* Tokenize args for 'if' tag. */
        preg_match_all('/(?:
                         "[^"\\\\]*(?:\\\\.[^"\\\\]*)*"         |
                         \'[^\'\\\\]*(?:\\\\.[^\'\\\\]*)*\'     |
                         [(),]                                  |
                         [^\s(),]+)/x', $param_tagArguments, $match);

        $tokens = $match[0];
        $is_arg_stack = array();

        for ($i = 0, $size = count($tokens); $i < $size; $i++) {
            $token = & $tokens[$i];

            switch ($token) {
                case '==':
                case 'eq':
                    $token = '==';
                    break;

                case '!=':
                case '<>':
                case 'ne':
                case 'neq':
                    $token = '!=';
                    break;

                case '<':
                case 'lt':
                    $token = '<';
                    break;

                case '<=':
                case 'le':
                case 'lte':
                    $token = '<=';
                    break;

                case '>':
                case 'gt':
                    $token = '>';
                    break;

                case '>=':
                case 'ge':
                case 'gte':
                    $token = '>=';
                    break;

                case '&&':
                case 'and':
                    $token = '&&';
                    break;

                case '||':
                case 'or':
                    $token = '||';
                    break;

                case '!':
                case 'not':
                    $token = '!';
                    break;

                case '%':
                case 'mod':
                    $token = '%';
                    break;

                case '(':
                    array_push($is_arg_stack, $i);
                    break;

                case 'is':
                    $is_arg_start = ($tokens[$i - 1] == ')') ? array_pop($is_arg_stack) : $i - 1;
                    $is_arg = implode('    ', array_slice($tokens, $is_arg_start, $i - $is_arg_start));

                    $new_tokens = $this->parseExpression_is($is_arg, array_slice($tokens, $i + 1));

                    array_splice($tokens, $is_arg_start, count($tokens), $new_tokens);

                    $i = $is_arg_start;

                // no break

                default:
                    if (preg_match('#^(([a-z]+[a-zA-Z0-9\-_]+?\.)+?)?([A-Z]+[A-Z0-9\-_]+?)$#s', $token, $varrefs)) {
                        $token = (!empty($varrefs[1])) ? $this->generate_block_data_ref(substr($varrefs[1], 0, -1), true) . '[\'' . $varrefs[3] . '\']' : '$this->_tpldata[\'.\'][0][\'' . $varrefs[3] . '\']';
                    }

                    break;
            }
        }

        return (($param_elseIf) ? '} else if (' : 'if (') . (implode(' ', $tokens) . ') { ' . "\n");

    }

    function compileTag_Include($param_tagArguments)
    {
        return "\$this->assignFromInclude('$param_tagArguments');\n";

    }

    function compileTag_IncludePHP($param_tagArguments)
    {
        return "include('" . $this->root . '/' . $param_tagArguments . "');\n";

    }

    // This is from Smarty
    function parseExpression_is($param_isArgument, $param_tokens)
    {
        $expr_end = 0;
        $negate_expr = false;

        if (($first_token = array_shift($param_tokens)) == 'not') {
            $negate_expr = true;
            $expr_type = array_shift($param_tokens);
        } else {
            $expr_type = $first_token;
        }

        switch ($expr_type) {
            case 'even':
                if (@$param_tokens[$expr_end] == 'by') {
                    $expr_end++;
                    $expr_arg = $param_tokens[$expr_end++];
                    $expr = "!(($param_isArgument	/ $expr_arg) % $expr_arg)";
                } else {
                    $expr = "!($param_isArgument % 2)";
                }
                break;

            case 'odd':
                if (@$param_tokens[$expr_end] == 'by') {
                    $expr_end++;
                    $expr_arg = $param_tokens[$expr_end++];
                    $expr = "(($param_isArgument / $expr_arg)	% $expr_arg)";
                } else {
                    $expr = "($param_isArgument %	2)";
                }
                break;

            case 'div':
                if (@$param_tokens[$expr_end] == 'by') {
                    $expr_end++;
                    $expr_arg = $param_tokens[$expr_end++];
                    $expr = "!($param_isArgument % $expr_arg)";
                }
                break;

            default:
                break;
        }

        if ($negate_expr) {
            $expr = "!($expr)";
        }

        array_splice($param_tokens, 0, $expr_end, $expr);

        return $param_tokens;

    }

    /**
     * Generates a reference to the given variable inside the given (possibly nested)
     * block namespace. This is a string of the form:
     * ' . $this->_tpldata['parent'][$_parent_i]['$child1'][$_child1_i]['$child2'][$_child2_i]...['varname'] . '
     * It's ready to be inserted into an "echo" line in one of the templates.
     * NOTE: expects a trailing "." on the namespace.
     */
    function generate_block_varref($namespace, $varname)
    {
        // Strip the trailing period.
        $namespace = substr($namespace, 0, -1);

        // Get a reference to the data block for this namespace.
        $varref = $this->generate_block_data_ref($namespace, true);
        // Prepend the necessary code to stick this in an echo line.
        // Append the variable reference.
        $varref .= '[\'' . $varname . '\']';

        $varref = '\' . ((isset(' . $varref . ')) ? ' . $varref . ' : \'\') . \'';

        return $varref;

    }

    /**
     * Generates a reference to the array of data values for the given
     * (possibly nested) block namespace. This is a string of the form:
     * $this->_tpldata['parent'][$_parent_i]['$child1'][$_child1_i]['$child2'][$_child2_i]...['$childN']
     *
     * If $include_last_iterator is true, then [$_childN_i] will be appended to the form shown above.
     * NOTE: does not expect a trailing "." on the blockname.
     */
    function generate_block_data_ref($blockname, $include_last_iterator)
    {
        // Get an array of the blocks involved.
        $blocks = explode('.', $blockname);
        $blockcount = sizeof($blocks) - 1;
        $varref = '$this->_tpldata';

        // Build up the string with everything but the last child.
        for ($i = 0; $i < $blockcount; $i++) {
            $varref .= '[\'' . $blocks[$i] . '.\'][$_' . $blocks[$i] . '_i]';
        }

        // Add the block reference for the last child.
        $varref .= '[\'' . $blocks[$blockcount] . '.\']';

        // Add the iterator for the last child if requried.
        if ($include_last_iterator) {
            $varref .= '[$_' . $blocks[$blockcount] . '_i]';
        }

        return $varref;

    }

    //
    // Compilation stuff
    //
    function compile_load(&$_str, &$handle, $do_echo)
    {
        $filename = $this->cachedir . $this->filename[$handle] . '.php';

        // Recompile page if the original template is newer, otherwise load the compiled version
        if (file_exists($filename) && @filemtime($filename) >= @filemtime($this->files[$handle])) {

            $_str = '';
            include($filename);

            if ($do_echo && $_str != '') {
                echo $_str;
            }

            return true;
        }
        return false;

    }

    function compile_write(&$handle, $data)
    {
        $handle_filename = $this->filename[$handle];

        if ($m = preg_match("/^\.{1,2}\/(\.{1,2}\/)*/", $handle_filename)) {
            $handle_filename_array = split("/", $handle_filename);
            $handle_filename = $handle_filename_array[count($handle_filename_array) - 1];
        }

        $filename = $this->cachedir . $handle_filename . '.php';

        $data = '<?php' . "\nif (\$this->security()) {\n" . $data . "\n}\n?" . ">";

        $fp = @fopen($filename, 'w+');
        @fwrite($fp, $data);
        @fclose($fp);

        @touch($filename, filemtime($this->files[$handle]));
        @chmod($filename, 0644);

        return;

    }

}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="INSTALL_TEMPLATE WRAPER">

class InstallTemplate extends Template
{

    var $errorMessages = array();
    var $messages = array();
    var $pageHeaderPrepared = false;
    var $pageTailPrepared = false;
    var $templateFile = '';
//    var $errorIcon = '<img src="../templates/install/images/file_conflict.gif">&nbsp;';
//    var $okIcon = '<img src="../templates/install/images/file_up_to_date.gif">&nbsp;';
    var $errorIcon = '<span class="ui-icon ui-icon-alert icon"></span>&nbsp;';
    var $okIcon = '<span class="ui-icon ui-icon-check icon"></span>&nbsp;';
    var $warningIcon = '<span class="ui-icon ui-icon-info icon"></span>&nbsp;';

    public function __construct($param_templateFile)
    {
        global $_LANGUAGE, $HCS_ROOTPATH, $installStyle;
        $this->templateFile = $param_templateFile;
        if (!isset($installStyle)) {
            $installStyle = (isset($_DEFAULTS)) ? $_DEFAULTS['default_style'] : '';
        }

        $this->setTemplate('install');

        $this->assignVariables(array(
                'CURRENT_YEAR' => date('Y', time()),
                'HCS_ROOT_DIR' => $HCS_ROOTPATH,
                'HCS_ROOTPATH' => $HCS_ROOTPATH,
                'HCS_STYLE' => $installStyle,
                'MSG_TITLE' => '',
                'MSG_TEXT' => '',
                'L_BUTTON_BACK' => $_LANGUAGE['inst_button_back'])
        );

        $this->setFilenames(array('body' => $this->templateFile));

    }

    public function __destruct()
    {
        $this->destroy();

    }

    function StatusIcon($mystat = 'ok')
    {
        return ($mystat == 'ok') ? $this->okIcon : (($mystat == 'warning') ? $this->warningIcon : $this->errorIcon);

    }

    function displayDieMessage($param_title = '', $param_text = '')
    {
        $this->setFilenames(array('body' => 'install_message.html'));

        $this->assignVariables(array(
            'MSG_TITLE' => ($param_title != '') ? $param_title : '&nbsp;',
            'MSG_TEXT' => ($param_text != '') ? $param_text : '&nbsp;',
        ));

        if (!$this->pageHeaderPrepared) {
            $this->preparePageHeader();
        }

        $this->prepratePageTailAndDisplay();

    }

    function appendMessage($param_message)
    {
        $this->messages[sizeof($this->messages) + 1] = $param_message;

    }

    function outputMessage($param_die = false)
    {
        global $_LANGUAGE;
        sort($this->messages);
        reset($this->messages);

        $messages = implode('<br /><br />', $this->messages);

        if ($param_die) {
            $this->displayDieMessage(((sizeof($this->messages) == 1) ? $_LANGUAGE['installation_message'] : $_LANGUAGE['installation_messages']), $messages);
        } else {
            $this->assignVariables(array(
                    'MSG_TITLE' => ((sizeof($this->messages) == 1) ? $_LANGUAGE['installation_message'] : $_LANGUAGE['installation_messages']),
                    'MSG_TEXT' => $messages)
            );
        }

    }

    function appendError($param_error)
    {
        $this->errorMessages[(sizeof($this->errorMessages) + 1)] = $param_error;

    }

    function outputError($die = false)
    {
        global $_LANGUAGE;
//        sort($this->errorMessages);
        reset($this->errorMessages);

        $errorMessages = implode('<br /><br />', $this->errorMessages);

        if ($die) {
            $this->displayDieMessage(((sizeof($this->errorMessages) == 1) ? $_LANGUAGE['error'] : $_LANGUAGE['errors']), $errorMessages);
        } else {
            $this->assignVariables(array(
                    'MSG_TITLE' => ((sizeof($this->errorMessages) == 1) ? $_LANGUAGE['error'] : $_LANGUAGE['errors']),
                    'MSG_TEXT' => $errorMessages)
            );
        }

    }

    function preparePageHeader()
    {
        global $STEP, $_LANGUAGE;

        $this->pageHeaderPrepared = true;

        /*
          $now = gmdate('D, d M Y H:i:s', time()) . ' GMT';
          @header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
          @header('Last-Modified: ' . $now);
          @header('Cache-Control: no-store, no-cache, must-revalidate');
          @header('Cache-Control: post-check=0, pre-check=0', false);
          @header('Pragma: no-cache');
          @header('Content-Type: text/html; charset=iso-8859-1');
         */

        $this->assignVariables(array(
                'INSTALL_STEP' => $STEP,
                'L_STEP' => $_LANGUAGE['inst_step'],
                'L_HEADER' => $_LANGUAGE['install_header']
            )
        );

        $stepname = array(
            1 => $_LANGUAGE['L_STEPNAME_1'],
            2 => $_LANGUAGE['L_STEPNAME_2'],
            3 => $_LANGUAGE['L_STEPNAME_3'],
            4 => $_LANGUAGE['L_STEPNAME_4'],
            5 => $_LANGUAGE['L_STEPNAME_5'],
            6 => $_LANGUAGE['L_STEPNAME_6'],
            7 => $_LANGUAGE['L_STEPNAME_7'],
        );

        // status indicator
        foreach ($stepname as $stepno => $sstep) {
            $this->assignBlockVariables('step_row', array(
                    'VALUE' => $sstep,
                    'INDEX' => $stepno,
                )
            );
        }

    }

    function prepratePageTailAndDisplay()
    {
        global $DEFAULTS, $_LANGUAGE;

        $this->assignVariable('S_SHOW_BUTTON', true);

        if (sizeof($this->messages) > 0) {
            $this->outputMessage(false);
        }

        if (sizeof($this->errorMessages) > 0) {
            $this->assignVariable('S_SHOW_BUTTON', false);
            $this->outputError(false);
        }

        $this->assignVariable('HCS_VERSION', HCS_VERSION);
        $this->assignVariable('HCS_AUTHOR', HCS_AUTHOR);
        $this->display('body');
        $this->destroy();
        exit;

    }

}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="PAGE_TEMPLATE WRAPER">

class PageTemplate extends Template
{

    var $pageHeaderPrepared = false;
    var $pageTailPrepared = false;
    var $templateFile = '';
    var $messages = array();
    var $errorIcon = '<span class="ui-icon ui-icon-alert icon"></span>&nbsp;';
    var $okIcon = '<span class="ui-icon ui-icon-check icon"></span>&nbsp;';
    var $warningIcon = '<span class="ui-icon ui-icon-info icon"></span>&nbsp;';

    public function __construct($param_template = 'page', $param_templateFile = '')
    {
        global $_LANGUAGE, $HCS_ROOTPATH, $_DEFAULTS, $_CONFIG;
        if (isset($_CONFIG)) {
            $style = $_CONFIG['style'];
            $lang = $_CONFIG['language'];
        } else {
            $style = isset($_DEFAULTS) ? $_DEFAULTS['default_style'] : 'homematic';
            $lang = isset($_DEFAULTS) ? $_DEFAULTS['default_lang'] : 'german';
        }
        $this->setTemplate($param_template);

        $this->assignVariables(array(
            'HCS_ROOTPATH' => $HCS_ROOTPATH,
            'MESSAGE_TITLE' => '',
            'MESSAGE_TEXT' => '',
            'HCS_VERSION' => HCS_VERSION,
            'HCS_AUTHOR' => HCS_AUTHOR,
            'DIE_MESSAGE' => false,
            'HCS_STYLE' => $style,
            'HCS_LANGUAGE' => $lang,
            'L_WARNING' => $_LANGUAGE['L_WARNING'],
            'L_CANCEL' => $_LANGUAGE['L_CANCEL'],
            'CURRENT_DATETIME' => date(SQL_SERVER_DATETIME_FORMAT),
            'HCS_NEW_VERSION_AVAILABLE' => $_CONFIG['newerVersionAvailable'],
            'HCS_NEWEST_VERSION' => $_CONFIG['checkedVersion'],
            'HCS_SHOW_UPDATE_ON_PAGE' => $_CONFIG['showUpdateOnPage'],
            'L_UPDATE_NEW_VERSION_INFO_TEXT' => sprintf($_LANGUAGE['L_UPDATE_NEW_VERSION_INFO_TEXT'], '<a href="http://hcs.xenorate.com/">' . $_CONFIG['checkedVersion'] . '</a>')

        ));

        $this->setFilename($param_templateFile);

    }

    public function setFilename($param_templateFile = '')
    {
        $this->templateFile = $param_templateFile;
        if ($this->templateFile != '') {
            $this->setFilenames(array('body' => $this->templateFile));
        }

    }

    public function __destruct()
    {
        $this->destroy();

    }

    function StatusIcon($param_status)
    {
        switch ($param_status) {
            case 'warning':
                return $this->warningIcon;
                break;

            case 'error':
                return $this->errorIcon;
                break;

            case 'ok':
            default:
                return $this->okIcon;
                break;
        }
        return false;

    }

    function appendMessage($param_message)
    {
        $this->messages[sizeof($this->messages) + 1] = $param_message;

    }

    function outputMessage($param_die = false)
    {
        global $_LANGUAGE;
        sort($this->messages);
        reset($this->messages);

        $messages = implode('<br /><br />', $this->messages);

        if ($param_die) {
            $this->displayDieMessage(((sizeof($this->messages) == 1) ? $_LANGUAGE['L_MESSAGE'] : $_LANGUAGE['L_MESSAGES']), $messages);
        } else {
            $this->assignVariables(array(
                    'MESSAGE_TITLE' => ((sizeof($this->messages) == 1) ? $_LANGUAGE['L_MESSAGE'] : $_LANGUAGE['L_MESSAGES']),
                    'MESSAGE_TEXT' => $messages)
            );
        }

    }

    function displayDieMessage($param_title = '', $param_text = '')
    {
        $this->setFilename('message.html');

        $this->assignVariables(array(
            'DIE_MESSAGE' => true,
            'MESSAGE_TITLE' => ($param_title != '') ? $param_title : '&nbsp;',
            'MESSAGE_TEXT' => ($param_text != '') ? $param_text : '&nbsp;',
        ));

        if (!$this->pageHeaderPrepared) {
            $this->preparePageHeader();
        }

        $this->prepratePageTailAndDisplay();

    }

    function preparePageHeader()
    {
        global $_LANGUAGE;

        $this->pageHeaderPrepared = true;

        /*
          $now = gmdate('D, d M Y H:i:s', time()) . ' GMT';
          @header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
          @header('Last-Modified: ' . $now);
          @header('Cache-Control: no-store, no-cache, must-revalidate');
          @header('Cache-Control: post-check=0, pre-check=0', false);
          @header('Pragma: no-cache');
          @header('Content-Type: text/html; charset=iso-8859-1');
         */

    }

    function prepratePageTailAndDisplay()
    {
        global $DEFAULTS, $_LANGUAGE;

        if (sizeof($this->messages) > 0) {
            $this->outputMessage(false);
        }
        $this->assignVariables(array(
            'CURRENT_YEAR' => date('Y', time()),
            'REQUEST_URI' => getRequestUri(),
            'SID' => insertSessionId(''),
            'SID_ADD' => insertSessionId('?'),
        ));

        if (getLoggedInUser() != false) {

            $this->assignVariables(array(
                'LOGGED_IN' => true,
                'USER_NAME' => $_SESSION['username'],
                'USER_IS_ADMIN' => $_SESSION['admin'],
                'USER_STYLE' => $_SESSION['style'],
                'USER_LANGUAGE' => $_SESSION['language'],
                'L_LOGOUT_BUTTON' => $_LANGUAGE['L_LOGOUT_BUTTON'],
            ));
        } else {
            $this->assignVariables(array(
                'LOGGED_IN' => false,
                'L_LOGIN_BUTTON' => $_LANGUAGE['L_LOGIN_BUTTON'],
            ));
        }

        if ($this->templateFile == '') {
            $this->setFilenames(array('body' => 'empty.html'));
        }
        $this->display('body');
        $this->destroy();
        exit;

    }

}

// </editor-fold>

/* -------------------------------------------------------------------------- */
?>