<?php

// <editor-fold defaultstate="collapsed" desc="CALLABLE PROTECTION">
// Make file not callable directly
if (!defined('HCS_CALLABLE_FILE')) {
    header('HTTP/1.0 404 Not Found');
    die('You shall not access this file directly.');
    exit;
}
// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="DEFINES">
if (!isset($HCS_ROOTPATH)) {
    $HCS_ROOTPATH = '../../';
}
if (is_file($HCS_ROOTPATH . 'config.php')) {
    include_once($HCS_ROOTPATH . 'config.php');
}

defined('LOGGING_INFO_MESSAGE') or define('LOGGING_INFO_MESSAGE', false);
defined('LOGGING_WARNING_MESSAGE') or define('LOGGING_WARNING_MESSAGE', true);
defined('LOGGING_DEBUG_MESSAGE') or define('LOGGING_DEBUG_MESSAGE', false);
defined('LOGGING_ERROR_MESSAGE') or define('LOGGING_ERROR_MESSAGE', true);


defined('LOGGING_PRINT_TO_FILE') or define('LOGGING_PRINT_TO_FILE', true);
defined('LOGGING_ECHO_TO_SCREEN') or define('LOGGING_ECHO_TO_SCREEN', false);
defined('LOGGING_LOG_TO_GLOBAL') or define('LOGGING_LOG_TO_GLOBAL', true);

defined('LOGGING_FILE_PATH') or define('LOGGING_FILE_PATH', 'data/log');
defined('LOGGING_DATE_TIME_STRING') or define('LOGGING_DATE_TIME_STRING', 'Y/m/d H:i:s');
defined('PING_TIMEOUT') or define('PING_TIMEOUT', 1);

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="expanded" desc="CLASSES">
// <editor-fold defaultstate="collapsed" desc="CLASS LOGGING SINGLETON">
class LoggingSingleton
{

    public $info = array();
    public $warning = array();
    public $debug = array();
    public $error = array();
    static private $instance = null;
    public $logInfo = LOGGING_INFO_MESSAGE;
    public $logWarning = LOGGING_WARNING_MESSAGE;
    public $logDebug = LOGGING_DEBUG_MESSAGE;
    public $logError = LOGGING_ERROR_MESSAGE;
    public $logToFile = LOGGING_PRINT_TO_FILE;
    public $logToScreen = LOGGING_ECHO_TO_SCREEN;
    public $logToArray = LOGGING_LOG_TO_GLOBAL;

    static public function getInstance()
    {
        if (null === self::$instance) {
            self::$instance = new self;
        }
        return self::$instance;

    }

    public static function reset()
    {
        $instance = null;
        $instance = self::getInstance();

        $instance->info = array();
        $instance->warning = array();
        $instance->debug = array();
        $instance->error = array();

        $instance->logInfo = LOGGING_INFO_MESSAGE;
        $instance->logWarning = LOGGING_WARNING_MESSAGE;
        $instance->logDebug = LOGGING_DEBUG_MESSAGE;
        $instance->logError = LOGGING_ERROR_MESSAGE;

        $instance->logToFile = LOGGING_PRINT_TO_FILE;
        $instance->logToScreen = LOGGING_ECHO_TO_SCREEN;
        $instance->logToArray = LOGGING_LOG_TO_GLOBAL;

    }

    public static function getInfoArray()
    {
        $instance = null;
        $instance = self::getInstance();
        return $instance->info;

    }

    public static function getWarningArray()
    {
        $instance = null;
        $instance = self::getInstance();
        return $instance->warning;

    }

    public static function getDebugArray()
    {
        $instance = null;
        $instance = self::getInstance();
        return $instance->debug;

    }

    public static function getErrorArray()
    {
        $instance = null;
        $instance = self::getInstance();
        return $instance->error;

    }

    private function __construct()
    {

    }

    private function __clone()
    {

    }

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="CLASS LOGGING">
class Logging
{

    private function __construct()
    {

    }

    private function __clone()
    {

    }

    static public function info($param_message, $param_die = false)
    {
        if (LoggingSingleton::getInstance()->logInfo) {
            self::logMessage($param_message, "info");
        }
        if ($param_die) {
            die($param_message);
        }

    }

    static public function warning($param_message, $param_die = false)
    {
        if (LoggingSingleton::getInstance()->logWarning) {
            self::logMessage($param_message, "warning");
        }
        if ($param_die) {
            die($param_message);
        }

    }

    static public function debug($param_message, $param_die = false)
    {
        if (LoggingSingleton::getInstance()->logDebug) {
            self::logMessage($param_message, "debug");
        }
        if ($param_die) {
            die($param_message);
        }

    }

    static public function error($param_message, $param_die = false)
    {
        if (LoggingSingleton::getInstance()->logError) {
            self::logMessage($param_message, "error");
        }
        if ($param_die) {
            die($param_message);
        }

    }

    static private function logMessage($param_message, $param_type)
    {
        $instance = LoggingSingleton::getInstance();
        if ($instance->logToFile) {
            self::printLogMessageToFile("[" . strtoupper($param_type) . "] " . $param_message, LOGGING_FILE_PATH, 'log.txt');
        }
        if ($instance->logToScreen) {
            self::echoToScreen("[" . strtoupper($param_type) . "] " . $param_message);
        }
        if ($instance->logToArray) {
            switch ($param_type) {
                case "info":
                    $instance->info[] = $param_message;
                    break;
                case "warning":
                    $instance->warning[] = $param_message;
                    break;
                case "debug":
                    $instance->debug[] = $param_message;
                    break;
                case "error":
                    $instance->error[] = $param_message;
                    break;

                default:
                    break;
            }
        }

    }

    static private function echoToScreen($param_Message)
    {
        echo('[' . date(LOGGING_DATE_TIME_STRING) . '] ' . htmlspecialchars($param_Message) . "\n<br/>");

    }

    static private function printLogMessageToFile($param_Message, $param_Folder, $param_File)
    {
        global $HCS_ROOTPATH;
        // make logging folder relative to root path
        $param_Folder = $HCS_ROOTPATH . $param_Folder . '/';
        $inhalt = '[' . date(LOGGING_DATE_TIME_STRING) . '] ' . $param_Message;
        if (!is_dir($param_Folder)) {
            @mkdir($param_Folder, 0777, true);
        }
        $handle = @fopen($param_Folder . $param_File, 'a');
        if (($handle !== false) && is_file($param_Folder . $param_File)) {
            fwrite($handle, $inhalt . "\n");
            fclose($handle);
        }

    }

}

// </editor-fold>
/* -------------------------------------------------------------------------- */
// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="expanded" desc="FUNCTIONS">
// <editor-fold defaultstate="collapsed" desc="FUNCTION XML->ARRAY">
/**
 * convert an XML object to an PHP array
 * @param SimpleXMLElement $xml <br>the XML object
 * @param array $arr <br>an PHP array equal keys will be overwritten by the XML
 * @return array an PHP array that is structured equivalent to the XML object
 */
function xml2phpArray($xml, $arr)
{
    $iter = 0;
    foreach ($xml->children() as $b) {
        $a = $b->getName();
        if (!$b->children()) {
            $arr[$a] = trim($b[0]);
        } else {
            $arr[$a][$iter] = array();
            $arr[$a][$iter] = xml2phpArray($b, $arr[$a][$iter]);
        }
        $iter++;
    }
    return $arr;

}

function xml2phpArray_utf8_decode($xml, $arr)
{
    $iter = 0;
    foreach ($xml->children() as $b) {
        $a = $b->getName();
        if (!$b->children()) {
            $arr[$a] = utf8_decode(trim($b[0]));
        } else {
            $arr[$a][$iter] = array();
            $arr[$a][$iter] = xml2phpArray($b, $arr[$a][$iter]);
        }
        $iter++;
    }
    return $arr;

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION SANITIZE">
/**
 * Convert a string to the file/URL safe "slug" form
 *
 * @param string $string the string to clean
 * @param bool $is_filename TRUE will allow additional filename characters
 * @return string
 */
function sanitize($string = '', $is_filename = FALSE)
{
    // Replace all weird characters with dashes
    $string = preg_replace('/[^\w\-'. ($is_filename ? '~_\.' : ''). ']+/u', '-', $string);

    // Only allow one dash separator at a time (and make string lowercase)
    return function_exists('mb_strtolower') ? mb_strtolower(preg_replace('/--+/u', '-', $string), 'UTF-8') : strtolower(preg_replace('/--+/u', '-', $string));
}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION FNMATCH">
// fnmatch checks if the passed string would match the given shell wildcard pattern.
// but for now it is not available on non-POSIX compliant systems.
if (!function_exists('fnmatch')) {
    define('FNM_PATHNAME', 1);
    define('FNM_NOESCAPE', 2);
    define('FNM_PERIOD', 4);
    define('FNM_CASEFOLD', 16);

    function fnmatch($pattern, $string, $flags = 0)
    {
        return pcre_fnmatch($pattern, $string, $flags);

    }

}

function pcre_fnmatch($pattern, $string, $flags = 0)
{
    $modifiers = null;
    $transforms = array(
        '\*' => '.*',
        '\?' => '.',
        '\[\!' => '[^',
        '\[' => '[',
        '\]' => ']',
        '\.' => '\.',
        '\\' => '\\\\'
    );

// Forward slash in string must be in pattern:
    if ($flags & FNM_PATHNAME) {
        $transforms['\*'] = '[^/]*';
    }

// Back slash should not be escaped:
    if ($flags & FNM_NOESCAPE) {
        unset($transforms['\\']);
    }

// Perform case insensitive match:
    if ($flags & FNM_CASEFOLD) {
        $modifiers .= 'i';
    }

// Period at start must be the same as pattern:
    if ($flags & FNM_PERIOD) {
        if (strpos($string, '.') === 0 && strpos($pattern, '.') !== 0)
            return false;
    }

    $pattern = '#^'
        . strtr(preg_quote($pattern, '#'), $transforms)
        . '$#'
        . $modifiers;

    return (boolean)preg_match($pattern, $string);

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION BOOLEAN->STRING">
function booleanToString(&$param_value)
{
    if ($param_value === true) {
        $param_value = "true";
        return "true";
    } else if ($param_value === false) {
        $param_value = "false";
        return "false";
    } else {
        return $param_value;
    }

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION PAGE RIGHTS->STRING">
function pageRightsToString($param_rightInt, $param_languageArray)
{
    $strings = array("" => "Unknown", 0 => $param_languageArray['L_PAGE_RIGHTS_NONE'], 1 => $param_languageArray['L_PAGE_RIGHTS_DISPLAY'], 2 => $param_languageArray['L_PAGE_RIGHTS_CONTROL'], 3 => $param_languageArray['L_PAGE_RIGHTS_EDIT']);
    return $strings[(int)$param_rightInt];
}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION INT->TIME STRING">
function intToTimeString($param_int, $param_timeArray = array('s', 'm', 'h', 'd'))
{
    $param_int = (int)$param_int;
    $days = (int)($param_int / 86400);
    $hours = (int)(($param_int - ($days * 86400)) / 3600);
    $minutes = (int)(($param_int - ($days * 86400) - ($hours * 3600)) / 60);
    $seconds = $param_int - ($days * 86400) - ($hours * 3600) - ($minutes * 60);

    $result = '';
    $result .= ($days != 0) ? $days . $param_timeArray[3] : '';
    $result .= ($hours != 0) ? ' ' . $hours . $param_timeArray[2] : '';
    $result .= ($minutes != 0) ? ' ' . $minutes . $param_timeArray[1] : '';
    $result .= ($seconds != 0) ? ' ' . $seconds . $param_timeArray[0] : '';

    if ($result == '') {
        return $seconds . $param_timeArray[0];
    }
    return $result;
}

// </editor-fold>
/* -------------------------------------------------------------------------- */


// <editor-fold defaultstate="collapsed" desc="FUNCTION BOOLEAN STRING->INT">
function booleanToInt(&$param_value)
{
    if ($param_value == 'true') {
        $param_value = 1;
        return 1;
    } else if ($param_value == 'false') {
        $param_value = 0;
        return 0;
    } else {
        return $param_value;
    }

}

function XStringToSqlBoolean($param_string)
{
    if (!(strpos($param_string, 'x') === false)) {
        return '1';
    } else if (!(strpos($param_string, '-') === false)) {
        return '0';
    }
    return '0';

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION UTF-8 ARRAY">
function utf8_array_encode($inArray, $encodeKey = false)
{
    /* our return object */
    $newArray = array();
    /* step through inArray */
    foreach ($inArray as $key => $val) {
        if (is_array($val)) {
            /* recurse on array elements */
            if ($encodeKey) {
                $newArray[utf8_encode($key)] = utf8_array_encode($val, $encodeKey);
            } else {
                $newArray[$key] = utf8_array_encode($val, $encodeKey);
            }
        } else {
            /* encode string values */
            if ($encodeKey) {
                $newArray[utf8_encode($key)] = utf8_encode($val);
            } else {
                $newArray[($key)] = utf8_encode($val);
            }
        }
    }

    /* return utf8 encoded array */
    return $newArray;

}

function to_utf8_deep($in)
{
    if (is_array($in)) {
        foreach ($in as $key => $value) {
            $out[to_utf8_deep($key)] = to_utf8_deep($value);
        }
    } elseif (is_string($in)) {
        if (mb_detect_encoding($in) != "UTF-8")
            return utf8_encode($in);
        else
            return $in;
    } else {
        return $in;
    }
    return $out;
}

function from_utf8_deep($in)
{
    if (is_array($in)) {
        foreach ($in as $key => $value) {
            $out[from_utf8_deep($key)] = from_utf8_deep($value);
        }
    } elseif (is_string($in)) {
        if (mb_detect_encoding($in) == "UTF-8")
            return utf8_decode($in);
        else
            return $in;
    } else {
        return $in;
    }
    return $out;
}

function utf8_array_decode($inArray, $encodedKey = false)
{
    /* our return object */
    $newArray = array();

    /* step through inArray */
    foreach ($inArray as $key => $val) {
        if (is_array($val)) {
            /* recurse on array elements */
            if ($encodedKey) {
                $newArray[utf8_decode($key)] = utf8_array_decode($val, $encodedKey);
            } else {
                $newArray[$key] = utf8_array_decode($val, $encodedKey);
            }
        } else {
            /* encode string values */
            if ($encodedKey) {
                $newArray[utf8_decode($key)] = utf8_decode($val);
            } else {
                $newArray[$key] = utf8_decode($val);
            }
        }
    }

    /* return utf8 encoded array */
    return $newArray;
}

function array_urldecode($inArray, $encodedKey = false)
{
    /* our return object */
    $newArray = array();

    /* step through inArray */
    foreach ($inArray as $key => $val) {
        if (is_array($val)) {
            /* recurse on array elements */
            if ($encodedKey) {
                $newArray[urldecode($key)] = array_urldecode($val, $encodedKey);
            } else {
                $newArray[$key] = array_urldecode($val, $encodedKey);
            }
        } else {
            /* encode string values */
            if ($encodedKey) {
                $newArray[urldecode($key)] = urldecode($val);
            } else {
                $newArray[$key] = urldecode($val);
            }
        }
    }

    /* return utf8 encoded array */
    return $newArray;
}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION PING">
function ping($param_host, $param_timeout = PING_TIMEOUT)
{
    $ts = microtime(true);
    $filehandle = fsockopen($param_host, 80, $errno, $errstr, $param_timeout);
    if (!$filehandle) {
        return false;
    } else {
        return microtime(true) - $ts;
    }
}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION ESCAPE DATA">

/**
 * Applies addslashes() to the provided data
 *
 * @param    mixed $param_data    Array of data or a single string
 * @return   mixed                  Array or string of escaped data
 */
function escapeData($param_data)
{
    if (is_array($param_data)) {
        foreach ($param_data as $key => $value) {
            $param_data[$key] = (is_array($value)) ? escapeData($value) : addslashes($value);
        }
    } else {
        $param_data = addslashes($param_data);
    }
    return $param_data;

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION UNESCAPE DATA">

/**
 * Applies addslashes() to the provided data
 *
 * @param    mixed $param_data    Array of data or a single string
 * @return   mixed                  Array or string of escaped data
 */
function unescapeData($param_data)
{
    if (is_array($param_data)) {
        foreach ($param_data as $key => $value) {
            $param_data[$key] = (is_array($value)) ? unescapeData($value) : stripslashes($value);
        }
    } else {
        $param_data = stripslashes($param_data);
    }
    return $param_data;

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTION GET $_POST[] OR ARRAY[] ">

/**
 * Checks if a POST field value exists;
 * If it does, we use that one, otherwise we use the optional array field value,
 * or return a null string
 *
 * @param    string $param_postIndex POST field name
 * @param    array $param_Array     Array of DB values
 * @param    string $param_ArrayIndex   DB field name
 * @return   string
 */
function getPostIndexOrArrayValue($param_postIndex, $param_Array = array(), $param_ArrayIndex = '')
{
    if (@sizeof($param_Array) > 0) {
        if ($param_ArrayIndex == '') {
            $param_ArrayIndex = $param_postIndex;
        }

        $arrayValue = $param_Array[$param_ArrayIndex];
    } else {
        $arrayValue = '';
    }

    return ((isset($_POST[$param_postIndex])) && (!empty($_POST[$param_postIndex]))) ? $_POST[$param_postIndex] : $arrayValue;

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="CHECKED SET">
function checkedSet(&$param_variable, &$param_data)
{
    if (isset($param_data) && ($param_data != '')) {
        if (get_magic_quotes_gpc() == 0) {
            // weil die common.php schon ein escaping macht.
            $param_variable = $param_data;

        } else {
            $param_variable = $param_data;
            //$param_variable = escapeData($param_data);

        }

    }

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="CLEAR LOGIN COOKIE">
function clearLoginCookies()
{
    if (isset($_COOKIE['userToken'])) {
        unset($_COOKIE['hashToken']);
    }
    if (isset($_COOKIE['userToken'])) {
        unset($_COOKIE['hashToken']);
    }
    setcookie('userToken', '', -1, '/');
    setcookie('hashToken', '', -1, '/');

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="CLEAR SESSION">
function clearSession()
{
    $_SESSION = array();
    if (session_id() != '') {
        session_write_close();
        @session_destroy();
    }
    if (isset($_COOKIE['PHPSESSID'])) {
        unset($_COOKIE['PHPSESSID']);
    }
    if (isset($_COOKIE[session_name()])) {
        unset($_COOKIE[session_name()]);
    }
    setcookie('PHPSESSID', '', -1, '/');
    setcookie(session_name(), '', -1, '/');

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="PICKUP SESSION">
function pickupSession()
{
    $id = '';
    if (session_id() == '') {
        checkedSet($id, $_COOKIE["PHPSESSID"]);
        checkedSet($id, $_GET['s']);
        checkedSet($id, $_GET['PHPSESSID']);
        checkedSet($id, $_POST['s']);
        checkedSet($id, $_POST['PHPSESSID']);

        if ($id != '') {
            session_id($id);
            @session_start();
        } else {
            session_id('0');
            @session_start();
        }
    }
    if ((session_id() != '') && isset($_SESSION['serverMade']) && ($_SESSION['serverMade'] === true)) {
        $time = time();
        $_SESSION['lastUse'] = $time;
        if (($time - $_SESSION['created']) >= 43200) {
            // if session is older than 12 hours (43200 s) regenerate id
            regenerateSession();
        }

        return true;
    } else {
        $_SESSION = array();
        return false;
    }

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="REGENERATE SESSION">
function regenerateSession()
{
    if (session_id() != '') {
        $_SESSION['created'] = time();
        @session_regenerate_id(TRUE);
    }
}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="GET REQUEST URI">
function getRequestUri()
{
    return insertSessionId($_SERVER['REQUEST_URI']);
}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="GET REFERER URI">
function getRefererUri()
{
    return insertSessionId($_SERVER['HTTP_REFERER']);
}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="INSERT SESSION ID">
function insertSessionId($param_link)
{
    $param_link = removeUriKey($param_link, session_name());

    $autoLogin = null;
    checkedSet($autoLogin, $_SESSION['autoLogin']);
    if (defined('SID') && (SID != '') && (($autoLogin == false) || !checkCookiesEnabled()) ) {
        // find 1st accurance of ? 
        $pos = strpos($param_link, '?');
        if ($pos !== false) {

            $param_link = substr($param_link, 0, $pos + 1) . SID . '&' . substr($param_link, $pos + 1);
        } else {
            $param_link .= '?' . SID;
        }
    }
    return $param_link;

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="CHECK COOKIES ENABLED">
function checkCookiesEnabled()
{
    if (isset($_COOKIE['cookieCheck'])) {
        return true;
    }
    setcookie("cookieCheck", "on", time() + 31536000, '/');
    if (isset($_COOKIE['userToken'])) {
        return true;
    }
    if (isset($_COOKIE['hashToken'])) {
        return true;
    }
    return false;
}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="HASHER">
// harry at simans dot net from http://www.php.net/manual/de/function.crypt.php#105949
function hasher($info, $param_encryptedData = false)
{
    $strength = "08";
    //if encrypted data is passed, check it against input ($info)
    if ($param_encryptedData) {
        if (substr($param_encryptedData, 0, 60) == crypt($info, "$2a$" . $strength . "$" . substr($param_encryptedData, 60))) {
            return true;
        } else {
            return false;
        }
    } else {
        //make a salt and hash it with input, and add salt to end
        $salt = "";
        for ($i = 0; $i < 22; $i++) {
            $salt .= substr("./ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", mt_rand(0, 63), 1);
        }
        //return 82 char string (60 char hash & 22 char salt)
        return crypt($info, "$2a$" . $strength . "$" . $salt) . $salt;
    }

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="REMOVE QUERYSTRING KEY">
// Dustin Hansen; http://maveno.us/; http://davidwalsh.name/php-remove-variable
function removeUriKey($param_uri, $param_key)
{
    if (!is_array($param_key)) {
        $param_key = array($param_key);
    }
    $url = $param_uri;
    foreach ($param_key as $key) {
        $url = preg_replace('/(?:&|(\\?|\\G))' . urlencode($key) . '(?:\\[[^]]*\\])*=[^&]*(?(1)&|)?/i', "$1", $url);
    }
    if ((substr($url, -1) == '?') && (substr($param_uri, -1) != '?')) {
        $url = substr($url, 0, -1);
    }
    return $url;

}

// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="DELETE CACHE FILE">
function deleteCacheFile($param_file, $param_token){
    if (file_exists($param_token) && is_readable($param_token) && !is_dir($param_token)) {
        @unlink($param_token);
    }

    if (file_exists($param_file) && is_readable($param_file) && !is_dir($param_file)) {
        if (!unlink($param_file)){
            $fp = @fopen($param_token, 'wb');
            @fclose($fp);
        }
    }
}
// </editor-fold>



/* -------------------------------------------------------------------------- */
// </editor-fold>
?>
