<?php

// <editor-fold defaultstate="collapsed" desc="CALLABLE PROTECTION">
// Make file not callable directly
if (!defined('HCS_CALLABLE_FILE')) {
    header('HTTP/1.0 404 Not Found');
    die('You shall not access this file directly.');
}
// </editor-fold>
/* -------------------------------------------------------------------------- */

if (!isset($HCS_ROOTPATH)) {
    $HCS_ROOTPATH = '../../';
}
require_once('xmlrpc.inc');
require_once('xmlrpcs.inc');
require_once('xmlrpc_wrappers.inc');

require_once('functions.php');
require_once('globals.php');

defined('REGAHSS_TIMEOUT') or define('REGAHSS_TIMEOUT', 30);
defined('XMLRPC_TIMEOUT') or define('XMLRPC_TIMEOUT', 30);

// Homematic Interfaces Enum
class Homematic_Interface
{

    const RF = 2001;
    const Wired = 2000;
    const System = 2002;
    const ReGaHSS = 8181;
    const CUxD = 8701;

}

class Homematic_ValueTypes
{

    const Unknown = "";
    const Integer = "int";
    const Float = "double";
    const Boolean = "boolean";
    const String = "string";
    const Enum = self::Integer;

}

class Homematic_Device
{
    public $channelAddress;
    public $channelName;
    public $deviceType;
    public $deviceAddress;
    public $deviceName;
    public $interface = Homematic_Interface::RF;
    public $craftIds = array();
    public $roomIds = array();

    function __construct($param_address, $param_name, $param_type, $param_deviceName, $param_interface, array $param_craftIds, array $param_roomIds)
    {
        $this->channelAddress = $param_address;
        $this->channelName = $param_name;
        $this->deviceType = $param_type;
        $this->deviceName = $param_deviceName;
        $this->interface = $param_interface;
        $this->deviceAddress = substr($param_address, 0, strrpos($param_address, ":"));
        $this->craftIds = $param_craftIds;
        $this->roomIds = $param_roomIds;
    }
}

class Homematic_SystemVariable
{
    public $id;
    public $name;
    public $type;
    public $interface = Homematic_Interface::ReGaHSS;

    function __construct($param_id, $param_name, $param_type)
    {
        $this->id = $param_id;
        $this->name = $param_name;
        $this->type = $param_type;
    }
}

// static Homematic XML-RPC communication class
class Homematic_XmlRpc
{

    private function __construct()
    {

    }

    private function __clone()
    {

    }

    public static function init($param_server, $param_port, $param_callbackAddress, $param_callbackId)
    {
        $xmlRpcValArray = array();
        $xmlRpcValArray[0] = new xmlrpcval($param_callbackAddress, "string");
        $xmlRpcValArray[1] = new xmlrpcval($param_callbackId, "string");

        $xmlRpcResp = self::executeXmlRpcMethod($param_server, $param_port, "init", $xmlRpcValArray);

        if ($xmlRpcResp->faultCode() == 0) {
            // no error while executing/sending
            Logging::info("[Homematic_XmlRpc->init] subscribed to event notification layer on server: \"" . $param_server . "\"; port: \"" . $param_port . "\"; Callback adresse: \"" . $param_callbackAddress . "\"; Callback ID: \"" . $param_callbackId . "\".");
            return true;
        } else {
            Logging::error("[Homematic_XmlRpc->init] could not subscribe to event notification layer on server: \"" . $param_server . "\"; port: \"" . $param_port . "\"; Callback adresse: \"" . $param_callbackAddress . "\"; Callback ID: \"" . $param_callbackId . "\". ErrorCode: \"" . $xmlRpcResp->faultCode() . "\"; ErrorString: \"" . $xmlRpcResp->faultString() . "\".");
            return false;
        }

    }

    public static function deinit($param_server, $param_port, $param_callbackAddress)
    {
        $xmlRpcValArray = array();
        $xmlRpcValArray[0] = new xmlrpcval($param_callbackAddress, "string");


        // check if wired port is open, if not, don't try to unsubscribe
        $portTest = @fsockopen($param_server, $param_port);
        if ($portTest === false) {
            return false;
        }
        $xmlRpcResp = self::executeXmlRpcMethod($param_server, $param_port, "init", $xmlRpcValArray);

        if ($xmlRpcResp->faultCode() == 0) {
            // no error while executing/sending
            Logging::info("[Homematic_XmlRpc->deinit] unsubscribed from event notification layer on server: \"" . $param_server . "\"; port: \"" . $param_port . "\" for Callback address: \"" . $param_callbackAddress . "\".");
            return true;
        } else {
            Logging::error("[Homematic_XmlRpc->deinit] could not unsubscribe from event notification layer on server: \"" . $param_server . "\"; port: \"" . $param_port . "\"; Callback address: \"" . $param_callbackAddress . "\". ErrorCode: \"" . $xmlRpcResp->faultCode() . "\"; ErrorString: \"" . $xmlRpcResp->faultString() . "\".");
            return false;
        }

    }

    /**
     * executes a XML-RPC method on a specified server:port
     * @param string $param_server the XML-RPC server address
     * @param string $param_port the XML-RPC server port
     * @param string $param_methodName the XML-RPC method name
     * @param array $param_methodParameters the parameters for the XML-RPC method. array of xmlrpcval (part of the PHP XML-RPC library)
     * @param int $param_debug Debug values 0, 1 and 2 are supported (2 = echo sent msg too, before received response)
     * @return xmlrpcresp an XML-RPC Response object holding the response values; false if the command could not be sent.
     */
    public static function executeXmlRpcMethod($param_server, $param_port, $param_methodName, array $param_methodParameters, $param_debug = 0)
    {
        if (($param_port == Homematic_Interface::RF) || ($param_port == Homematic_Interface::Wired) || ($param_port == Homematic_Interface::System)) {
            $client = new xmlrpc_client($param_server . ":" . $param_port);

            $client->setDebug($param_debug);
            $xmlRpcValArray = array();

            foreach ($param_methodParameters as $value) {
                if ($value instanceof xmlrpcval) {
                    $xmlRpcValArray[] = $value;
                } else {
                    Logging::error("[Homematic_XmlRpc->executeXmlRpcMethod] method parameter is not an instance of 'xmlrpcval'");
                }
            }
            $xmlRpcMsg = new xmlrpcmsg($param_methodName, $xmlRpcValArray);
            $xmlRpcResp = $client->send($xmlRpcMsg, XMLRPC_TIMEOUT);

            Logging::debug("[Homematic_XmlRpc->executeXmlRpcMethod ($param_methodName)] sent: " . $xmlRpcMsg->serialize());
            Logging::debug("[Homematic_XmlRpc->executeXmlRpcMethod ($param_methodName)] received: " . $xmlRpcResp->serialize());

            if ($xmlRpcResp === 0) {
                // die because some error occurred during sending
                Logging::error("[Homematic_XmlRpc->executeXmlRpcMethod ($param_methodName)] could not send command to XML-RPC server: \".$param_server . ':' . $param_port.\". ErrorNumber: \"" . $client->errno . "\"; ErrorString: \"" . $client->$errstr . "\".");
                return false;
            }
            return $xmlRpcResp;
        } else {
            Logging::error('[Homematic_XmlRpc->executeXmlRpcMethod] Port (' . $param_port . ') is not valid. Port-Range: 2000-2002');
            return false;
        }

    }

    public static function setValue($param_server, $param_port, $param_device, $param_valueKey, $param_value, $param_valueType = Homematic_ValueTypes::Unknown)
    {
        if ($param_valueType == Homematic_ValueTypes::Unknown) {
            $valueArray = self::getValueTypeByDp($param_valueKey, $param_value);

            $param_value = $valueArray[1];
            $param_valueType = $valueArray[0];
        }

        $xmlRpcValArray = array();
        $xmlRpcValArray[0] = new xmlrpcval($param_device, "string");
        $xmlRpcValArray[1] = new xmlrpcval($param_valueKey, "string");
        $xmlRpcValArray[2] = new xmlrpcval($param_value, $param_valueType);

        $xmlRpcResp = self::executeXmlRpcMethod($param_server, $param_port, "setValue", $xmlRpcValArray);

        if ($xmlRpcResp->faultCode() == 0) {
            // no error while executing/sending
            // make false string "0" not ""
            if (($param_valueType == Homematic_ValueTypes::Boolean) && ($xmlRpcValArray[2]->scalarval() == 0)) {
                $param_value = "0";
            }

            Logging::info("[Homematic_XmlRpc->setValue] set value \"" . $param_value . "\" (" . $param_valueType . ") to data point \"" . $param_valueKey . "\" on device \"" . $param_device . "\".");
            return true;
        } else {
            Logging::error("[Homematic_XmlRpc->setValue] could not set value \"" . $param_value . "\"(" . $param_valueType . ") to data point \"" . $param_valueKey . "\" on device \"" . $param_device . "\".  ErrorCode: \"" . $xmlRpcResp->faultCode() . "\"; ErrorString: \"" . $xmlRpcResp->faultString() . "\".");
            return false;
        }

    }

    public static function getValue($param_server, $param_port, $param_device, $param_valueKey)
    {
        $xmlRpcValArray = array();
        $xmlRpcValArray[0] = new xmlrpcval($param_device, "string");
        $xmlRpcValArray[1] = new xmlrpcval($param_valueKey, "string");

        $xmlRpcResp = self::executeXmlRpcMethod($param_server, $param_port, "getValue", $xmlRpcValArray);

        if ($xmlRpcResp->faultCode() == 0) {
            // no error while executing/sending
            $respValue = $xmlRpcResp->value();

            if ($respValue->kindOf() != "scalar") {
                Logging::error("[Homematic_XmlRpc->getValue] returned value is not a scalar. This should not happen.");
                return false;
            }

            $respScalarType = $respValue->scalartyp();
            $respScalarValue = $respValue->scalarval();

            // make false string "0" not ""
            if (($respScalarType == Homematic_ValueTypes::Boolean) && ($respScalarValue == 0)) {
                $respScalarValue = "0";
            }

            Logging::info("[Homematic_XmlRpc->getValue] got value \"" . $respScalarValue . "\" (" . $respScalarType . ") from data point \"" . $param_valueKey . "\" on device \"" . $param_device . "\".");
            return $respScalarValue;
        } else {
            Logging::error("[Homematic_XmlRpc->getValue] could not get value from data point \"" . $param_valueKey . "\" on device \"" . $param_device . "\".  ErrorCode: \"" . $xmlRpcResp->faultCode() . "\"; ErrorString: \"" . $xmlRpcResp->faultString() . "\".");
            return false;
        }

    }

    public static function getValueTypeByDp($param_valueKey, $param_value)
    {
        $value = $param_value;
        $param_valueKey = strtoupper($param_valueKey);

        switch ($param_valueKey) {
            // int
            case "HUMIDITY":
            case "ADJUSTING_COMMAND":
            case "ADJUSTING_DATA":
            case "VALVE_STATE":
            case "WIND_DIRECTION":
            case "WIND_DIRECTION_RANGE":
            case "SUNSHINEDURATION":
            case "BRIGHTNESS":
            case "ALARM_COUNT":
            case "SERVICE_COUNT":
            case "NEXT_TRANSMISSION":
            case "AIR_PRESSURE":
            case "RSSI_PEER":
            case "RSSI_DEVICE":

                // option
            case "ERROR":
            case "UNIT":
            case "BEEP":
            case "BACKLIGHT":
            case "STATUS":
                $valueType = Homematic_ValueTypes::Integer;
                $value = (int)$value;
                break;

            // double
            case "TEMPERATURE":
            case "LEVEL":
            case "SETPOINT":
            case "ON_TIME":
            case "RAMP_TIME":
            case "RELOCK_DELAY":
            case "RAIN_COUNTER":
            case "WIND_SPEED":
            case "SPEED":
            case "BAT_LEVEL":
                $valueType = Homematic_ValueTypes::Float;
                $value = (float)$value;
                break;

            // boolean
            case "INHIBIT":
            case "WORKING":
            case "STATE":
            case "ERROR_REDUCED":
            case "ERROR_OVERLOAD":
            case "ERROR_OVERHEAT":
            case "STATE_UNCERTAIN":
            case "RAINING":
            case "MOTION":
            case "SENSOR":
                $valueType = Homematic_ValueTypes::Boolean;
                if (($value == false) || (is_int($value) && ($value <= 0)) || (strtolower($value) == "false")) {
                    $value = false;
                } else {
                    $value = true;
                }

                break;

            // action
            case "STOP":
            case "INSTALL_TEST":
            case "PRESS_LONG":
            case "PRESS_SHORT":
            case "OLD_LEVEL":
            case "OPEN":
            case "BULB":
            case "SWITCH":
            case "WINDOW":
            case "DOOR":
            case "BLIND":
            case "SCENE":
            case "PHONE":
            case "BELL":
            case "CLOCK":
            case "ARROW_UP":
            case "ARROW_DOWN":
            case "SUBMIT":
            case "SEQUENCE_OK":
            case "PRESS":
                $valueType = Homematic_ValueTypes::Boolean;
                $value = true;
                break;

            // string
            case "TEXT":
                $valueType = Homematic_ValueTypes::String;
                $value = (string)$value;
                break;

            default:
                $valueType = Homematic_ValueTypes::String;
                $value = (string)$value;
                break;
        }
        return array($valueType, $value);

    }

}

class Homematic_XmlRpcInterface
{

    // <editor-fold defaultstate="collapsed" desc=" member variables ">
    public function getCallbackAddress()
    {
        return $this->callbackAddress;

    }

    public function setCallbackAddress($callbackAdress)
    {
        $this->callbackAddress = $callbackAdress;

    }

    public function getCallbackId()
    {
        return $this->callbackId;

    }

    public function setCallbackId($callbackId)
    {
        $this->callbackId = $callbackId;

    }

    public function getServer()
    {
        return $this->server;

    }

    public function getPort()
    {
        return $this->port;

    }

    private $server = "127.0.0.1";
    private $port = Homematic_Interface::RF;
    private $callbackAddress = null;
    private $callbackId = "0";
    private $initialized = false;

// </editor-fold>

    function __construct($param_server, $param_port = Homematic_Interface::RF, $param_callbackAddress = null, $param_callbackId = "0")
    {
        $this->server = $param_server;
        $this->callbackId = $param_callbackId;
        $this->callbackAddress = $param_callbackAddress;
        if (($param_port == Homematic_Interface::RF) || ($param_port == Homematic_Interface::Wired) || ($param_port == Homematic_Interface::System)) {
            $this->port = $param_port;
        } else {

            Logging::error('[Homematic_XmlRpcInterface] XML-RPC Port (' . $param_port . ') is not valid. Port-Range: 2000-2002');
        }

    }

    public function init()
    {
        if (!is_null($this->callbackAddress)) {
            if (Homematic_XmlRpc::init($this->server, $this->port, $this->callbackAddress, $this->callbackId)) {
                $this->initialized = true;
                return true;
            } else {
                return false;
            }
        } else {
            Logging::error("[Homematic_XmlRpcInterface->init] no callback address set.", false);
            return false;
        }

    }

    public function deinit()
    {
        if (!is_null($this->callbackAddress)) {
            if (Homematic_XmlRpc::deinit($this->server, $this->port, $this->callbackAddress)) {
                $this->initialized = false;
                return true;
            } else {
                return false;
            }
        } else {
            Logging::error("[Homematic_XmlRpcInterface->deinit] no callback address set.", false);
            return false;
        }

    }

    public function executeXmlRpcMethode($param_methodName, array $param_methodParameters, $param_debug = 0)
    {
        return Homematic_XmlRpc::executeXmlRpcMethod($this->server, $this->port, $param_methodName, $param_methodParameters, $param_debug);

    }

    public function setValue($param_device, $param_valueKey, $param_value, $param_valueType = Homematic_ValueTypes::Unknown)
    {
        return Homematic_XmlRpc::setValue($this->server, $this->port, $param_device, $param_valueKey, $param_value, $param_valueType);

    }

    public function getValue($param_device, $param_valueKey)
    {
        return Homematic_XmlRpc::getValue($this->server, $this->port, $param_device, $param_valueKey);

    }

}

// <editor-fold defaultstate="collapsed" desc="Commands">

abstract class Command
{

    protected $commandString;

    public function getCommandString()
    {
        return $this->commandString;

    }

    protected function setCommandString($commandString)
    {
        $this->commandString = $commandString;

    }

    private $hssObject;

    public function getHssObject()
    {
        return $this->hssObject;

    }

    protected function setHssObject($hssObject)
    {
        $this->hssObject = $hssObject;

    }

}

class SetCommand extends Command
{

    private $value;

    public function getValue()
    {
        return $this->value;

    }

    protected function setValue($value)
    {
        $this->value = $value;

    }

    function __construct($param_systemVariable, $param_value)
    {
        $this->setHssObject($param_systemVariable);
        $this->setValue($param_value);

        if (is_int($param_systemVariable)) {
            $command = "dom.GetObject(" . $param_systemVariable . ")";
        } else {
            $command = "dom.GetObject(\"" . $param_systemVariable . "\")";
        }

        if (is_numeric($param_value) || is_int($param_value) || is_float($param_value) || is_double($param_value) || is_integer($param_value) || is_long($param_value) || is_real($param_value)) {
            $command .= ".State(" . $param_value . ");";
        } elseif (is_bool($param_value)) {
            if ($param_value) {
                $command .= ".State(true);";
            } else {
                $command .= ".State(false);";
            }
        } else {
            $command .= ".State(\"" . $param_value . "\");";
        }
        $this->setCommandString($command);

    }

}

class SetCommandByName extends Command
{

    private $value;

    public function getValue()
    {
        return $this->value;

    }

    protected function setValue($value)
    {
        $this->value = $value;

    }

    function __construct($param_device, $param_valueKey, $param_value)
    {
        $this->setHssObject($param_device);
        $this->setValue($param_value);

        if (is_int($param_device)) {
            $command = "dom.GetObject(" . $param_device . ")";
        } else {
            $command = "dom.GetObject(\"" . $param_device . "\")";
        }
        $command .= ".DPByHssDP(\"" . $param_valueKey . "\")";
        if (is_numeric($param_value) || is_int($param_value) || is_float($param_value) || is_double($param_value) || is_integer($param_value) || is_long($param_value) || is_real($param_value)) {
            $command .= ".State(" . $param_value . ");";
        } elseif (is_bool($param_value)) {
            if ($param_value) {
                $command .= ".State(true);";
            } else {
                $command .= ".State(false);";
            }
        } else {
            $command .= ".State(\"" . $param_value . "\");";
        }
        $this->setCommandString($command);

    }

}

class GetCommand extends Command
{

    function __construct($param_object)
    {
        $this->setHssObject($param_object);

        if (is_int($param_object)) {
            $this->setCommandString("dom.GetObject(" . $param_object . ").Value();");
        } else {
            $this->setCommandString("dom.GetObject(\"" . $param_object . "\").Value();");
        }

    }

}

class GetCommandByName extends Command
{

    function __construct($param_device, $param_valueKey)
    {
        $this->setHssObject($param_device);

        if (is_int($param_device)) {
            $this->setCommandString("dom.GetObject(" . $param_device . ").DPByHssDP(\"" . $param_valueKey . "\").Value();");
        } else {
            $this->setCommandString("dom.GetObject(\"" . $param_device . "\").DPByHssDP(\"" . $param_valueKey . "\").Value();");
        }

    }

}

class GetVarCommand extends Command
{

    function __construct($param_systemVariable)
    {
        $this->setHssObject($param_systemVariable);

        if (is_int($param_systemVariable)) {
            $this->setCommandString("dom.GetObject(" . $param_systemVariable . ").Variable();");
        } else {
            $this->setCommandString("dom.GetObject(\"" . $param_systemVariable . "\").Variable();");
        }

    }

}

class ExecuteCommand extends Command
{

    function __construct($param_program)
    {
        $this->setHssObject($param_program);

        if (is_int($param_program)) {
            $this->setCommandString("dom.GetObject(" . $param_program . ").ProgramExecute();");
        } else {
            $this->setCommandString("dom.GetObject(\"" . $param_program . "\").ProgramExecute();");
        }

    }

}

class MultiCommandObject
{

    private $commands = array();

    public function getCommands()
    {
        return $this->commands;

    }

    public function getCommandsString()
    {
        $commandsString = "";
        foreach ($this->commands as $key => $command) {
            $commandsString .= "var t_" . $key . " = " . $command->getCommandString();
        }
        return $commandsString;

    }

    function addCommand(Command $param_command, $param_key = null)
    {
        if (is_null($param_key)) {
            $this->commands[] = $param_command;
        } else {
            $this->commands[$param_key] = $param_command;
        }

    }

    function __construct(array $commandsArray)
    {
        foreach ($commandsArray as $key => $commandArray) {
            if ((isset($commandArray[0])) && (isset($commandArray[1]))) {
                if ($commandArray[0] == "set") {
                    if (isset($commandArray[2])) {
                        $this->addCommand(new SetCommand($commandArray[1], $commandArray[2]), $key);
                    } else {
                        Logging::error("[MultiCommandObject] value parameter (array index [2]) is not set.");
                    }
                } elseif ($commandArray[0] == "get") {
                    $this->addCommand(new GetCommand($commandArray[1]), $key);
                } elseif ($commandArray[0] == "execute") {
                    $this->addCommand(new ExecuteCommand($commandArray[1]), $key);
                } else {
                    Logging::error("[MultiCommandObject] \"$commandArray[0]\" is no known command");
                }
            } else {
                Logging::error("[MultiCommandObject] type parameter (array index [0]) or object parameter (array index [1]) is not set.");
            }
        }

    }

}

// </editor-fold>


class Homematic_ReGaHSS
{

    private $server = "127.0.0.1";
    private $port = Homematic_Interface::ReGaHSS;

    function __construct($param_server, $param_port = Homematic_Interface::ReGaHSS)
    {
        $this->server = $param_server;
        $this->port = $param_port;

    }

    function getValueBySn($param_interface, $param_deviceSn, $param_valueKey)
    {
        $command = new GetCommand(HomematicControl::homematicInterfaceToString($param_interface) . "." . $param_deviceSn . "." . $param_valueKey);
        $return = $this->sendCommand($command->getCommandString());
        if ($return == "null") {
            Logging::error("[Homematic_ReGaHSS->getValueBySn] something went wrong while getting: \"" . $param_deviceSn . "." . $param_valueKey . "\". (Avoid using 'null' as a value. This will be interpreted as an error)", false);
            return false;
        }
        Logging::info("[Homematic_ReGaHSS->getValueBySn] got value \"" . $return . "\" from : \"" . $param_deviceSn . "." . $param_valueKey . "\".");
        return $return;

    }

    function getValueByName($param_device, $param_valueKey)
    {
        $command = new GetCommandByName($param_device, $param_valueKey);
        $return = $this->sendCommand($command->getCommandString());
        if ($return == "null") {
            Logging::error("[Homematic_ReGaHSS->getValueByName] something went wrong while getting: \"" . $param_device . ":" . $param_valueKey . "\". (Avoid using 'null' as a value. This will be interpreted as an error)", false);
            return false;
        }
        Logging::info("[Homematic_ReGaHSS->getValueByName] got value \"" . $return . "\" from: \"" . $param_device . ":" . $param_valueKey . "\".");
        return $return;

    }

    function setValueBySn($param_interface, $param_sn, $param_valueKey, $param_value)
    {
        $command = new SetCommand(HomematicControl::homematicInterfaceToString($param_interface) . "." . $param_sn . "." . $param_valueKey, $param_value);
        $return = $this->sendCommand($command->getCommandString());
        if ($return == "true") {
            Logging::info("[Homematic_ReGaHSS->setValueBySn] set value \"" . $param_value . "\" to data point \"" . $param_valueKey . "\" on device \"" . $param_sn . "\".");
            return true;
        } elseif ($return == "false") {
            Logging::warning("[Homematic_ReGaHSS->setValueBySn] probably set value \"" . $param_value . "\" to data point \"" . $param_valueKey . "\" on device \"" . $param_sn . "\". But received 'false' as a response. It is also possible that an error occurred.");
            return true;
        } else {
            // something went wrong
            Logging::error("[Homematic_ReGaHSS->setValueBySn] could not set value \"" . $param_value . "\" to data point \"" . $param_valueKey . "\" on device \"" . $param_sn . "\".");
            return false;
        }

    }

    function setValueByName($param_device, $param_valueKey, $param_value)
    {
        $command = new SetCommandByName($param_device, $param_valueKey, $param_value);
        $return = $this->sendCommand($command->getCommandString());
        if ($return == "true") {
            Logging::info("[Homematic_ReGaHSS->setValueByName] set value \"" . $param_value . "\" to data point \"" . $param_valueKey . "\" on device \"" . $param_device . "\".");
            return true;
        } elseif ($return == "false") {
            Logging::warning("[Homematic_ReGaHSS->setValueByName] probably set value \"" . $param_value . "\" to data point \"" . $param_valueKey . "\" on device \"" . $param_device . "\". But received 'false' as a response. It is also possible that an error occurred.");
            return true;
        } else {
            // something went wrong
            Logging::error("[Homematic_ReGaHSS->setValueByName] could not set value \"" . $param_value . "\" to data point \"" . $param_valueKey . "\" on device \"" . $param_device . "\".");
            return false;
        }

    }

    function getAllSystemVariables()
    {
        global $HCS_ROOTPATH;
        $response = array_urldecode(utf8_array_decode(json_decode($this->sendScript(file_get_contents($HCS_ROOTPATH . 'includes/scripts/variablesValues.fn'), true), true), true));

        if (($response == "null") || ($response == false)) {
            Logging::error("[Homematic_ReGaHSS->getAllSystemVariables] something went wrong while getting system variables. (Avoid using 'null' as a value. This will be interpreted as an error)", false);
            return false;
        }
        Logging::info("[Homematic_ReGaHSS->getAllSystemVariables] received all system variable values:\n" . var_export($response, true));
        return $response;
    }

    function getSystemVariable($param_systemVariable)
    {
        $command = new GetVarCommand($param_systemVariable);
        $return = $this->sendCommand($command->getCommandString());
        if ($return == "null") {
            Logging::error("[Homematic_ReGaHSS->setSystemVariable] something went wrong while getting system variable: \"" . $param_systemVariable . "\". (Avoid using 'null' as a value. This will be interpreted as an error)", false);
            return false;
        }
        Logging::info("[Homematic_ReGaHSS->getSystemVariable] got value \"" . $return . "\" from system variable: \"" . $param_systemVariable . "\".");
        return $return;

    }

    function setSystemVariable($param_systemVariable, $param_value)
    {
        $command = new SetCommand($param_systemVariable, $param_value);
        $return = $this->sendCommand($command->getCommandString());
        if ($return == "true") {
            Logging::info("[Homematic_ReGaHSS->setSystemVariable] set system variable: \"" . $param_systemVariable . "\" to \"" . $param_value . "\".");
            return true;
        } else {
            // something went wrong
            Logging::error("[Homematic_ReGaHSS->setSystemVariable] something went wrong while setting system variable: \"" . $param_systemVariable . "\" to \"" . $param_value . "\".", false);
            return false;
        }

    }

    function executeProgram($param_program)
    {
        $command = new ExecuteCommand($param_program);
        $return = $this->sendCommand($command->getCommandString());
        if ($return == "true") {
            Logging::info("[Homematic_ReGaHSS->executeProgram] executed program: \"" . $param_program . "\".");
            return true;
        } else {
            // something went wrong
            Logging::error("[Homematic_ReGaHSS->executeProgram] something went wrong while executing program: \"" . $param_program . "\".", false);
            return false;
        }

    }

    function multiCommand(MultiCommandObject $param_commands)
    {
        if (LOGGING_INFO_MESSAGE) {
            $commands = $param_commands->getCommands();
            foreach ($commands as $key => $command) {
                if ($command instanceof SetCommand) {
                    Logging::info("[Homematic_ReGaHSS->multiCommand] [" . $key . "] setting \"" . $command->getValue() . "\" to \"" . $command->getHssObject() . "\".");
                } elseif ($command instanceof GetCommand) {
                    Logging::info("[Homematic_ReGaHSS->multiCommand] [" . $key . "] getting \"" . $command->getHssObject() . "\".");
                } elseif ($command instanceof GetVarCommand) {
                    Logging::info("[Homematic_ReGaHSS->multiCommand] [" . $key . "] getting \"" . $command->getHssObject() . "\".");
                } elseif ($command instanceof ExecuteCommand) {
                    Logging::info("[Homematic_ReGaHSS->multiCommand] [" . $key . "] executing \"" . $command->getHssObject() . "\".");
                }
            }
        }
        $resultArray = $this->sendMultipleCommands($param_commands->getCommandsString());

        if (is_array($resultArray)) {
            foreach ($resultArray as $key => $value) {
                if (substr($key, 0, 2) == "t_") {
                    $resultArray[substr($key, 2)] = $value;
                    unset($resultArray[$key]);
                }
            }
        }
        Logging::info("[Homematic_ReGaHSS->multiCommand] received: \"" . var_export($resultArray, true) . "\".");
        return $resultArray;

    }

    function sendMultipleCommands($param_commands, $param_trimmed = true)
    {
        $errorNumber = 0;
        $errorString = "";
        $fileHandle = @fsockopen($this->server, $this->port, $errorNumber, $errorString, REGAHSS_TIMEOUT);
        if (!$fileHandle) {
            // couldn't create fileHandle
            Logging::error("[Homematic_ReGaHSS->sendCommand] could not open socket at server: \"" . $this->server . "\" on port: \"" . $this->port . "\". ErrorNumber: \"" . $errorNumber . "\", ErrorString\"" . $errorString . "\".");
            return false;
        } else {
            Logging::debug("[Homematic_ReGaHSS->sendCommand] opened socket at server: \"" . $this->server . "\" on port: \"" . $this->port . "\".");

            $xml = $this->send($param_commands, $fileHandle, $param_trimmed);
            // return XML as PHP-Array
            return xml2phpArray_utf8_decode($xml, array());
        }

    }

    function sendCommand($param_command, $param_trimmed = true)
    {
        $errorNumber = 0;
        $errorString = "";
        $fileHandle = @fsockopen($this->server, $this->port, $errorNumber, $errorString, REGAHSS_TIMEOUT);
        if (!$fileHandle) {
            // couldn't create fileHandle
            Logging::error("[Homematic_ReGaHSS->sendCommand] could not open socket at server: \"" . $this->server . "\" on port: \"" . $this->port . "\". ErrorNumber: \"" . $errorNumber . "\", ErrorString\"" . $errorString . "\".");
            return false;
        } else {
            Logging::debug("[Homematic_ReGaHSS->sendCommand] opened socket at server: \"" . $this->server . "\" on port: \"" . $this->port . "\".");

            $xml = $this->send("result = " . $param_command, $fileHandle, $param_trimmed);
            // return XML value result
            return utf8_decode($xml->result);
        }

    }

    function sendScript($param_script, $onlyReturnOutput = false)
    {
        $errorNumber = 0;
        $errorString = "";
        $fileHandle = @fsockopen($this->server, $this->port, $errorNumber, $errorString, REGAHSS_TIMEOUT);
        if (!$fileHandle) {
            // couldn't create fileHandle
            Logging::error("[Homematic_ReGaHSS->sendCommand] could not open socket at server: \"" . $this->server . "\" on port: \"" . $this->port . "\". ErrorNumber: \"" . $errorNumber . "\", ErrorString\"" . $errorString . "\".");
            return false;
        } else {
            Logging::debug("[Homematic_ReGaHSS->sendCommand] opened socket at server: \"" . $this->server . "\" on port: \"" . $this->port . "\".");

            // return receive Message
            $received = $this->send($param_script, $fileHandle, false);
            if ($onlyReturnOutput) {
                return utf8_encode(substr($received, 0, strripos($received, '<xml>')));
            } else {
                return utf8_encode($received);
            }
        }
    }

    function getRoomList()
    {
        global $HCS_ROOTPATH;
        $response = utf8_array_decode(json_decode($this->sendScript(file_get_contents($HCS_ROOTPATH . 'includes/scripts/rooms.fn'), true), true), true);
        Logging::info("[Homematic_ReGaHSS->getRoomList] received room list:\n" . var_export($response, true));
        return $response;
    }

    function getCraftsList()
    {
        global $HCS_ROOTPATH;
        $response = utf8_array_decode(json_decode($this->sendScript(file_get_contents($HCS_ROOTPATH . 'includes/scripts/crafts.fn'), true), true), true);
        Logging::info("[Homematic_ReGaHSS->getCraftsList] received crafts list:\n" . var_export($response, true));
        return $response;
    }

    function getProgramsList()
    {
        global $HCS_ROOTPATH;
        $response = array_urldecode(utf8_array_decode(json_decode($this->sendScript(file_get_contents($HCS_ROOTPATH . 'includes/scripts/programs.fn'), true), true), true));
        Logging::info("[Homematic_ReGaHSS->getProgramsList] received programs list:\n" . var_export($response, true));
        return $response;
    }

    function getSystemVariableList()
    {
        global $HCS_ROOTPATH;
        $response = array_urldecode(utf8_array_decode(json_decode($this->sendScript(file_get_contents($HCS_ROOTPATH . 'includes/scripts/variables.fn'), true), true), true));
        Logging::info("[Homematic_ReGaHSS->getSystemVariableList] received system variable list:\n" . var_export($response, true));
        return $response;
    }

    function getDeviceList()
    {
        global $HCS_ROOTPATH;
        $response = array_urldecode(utf8_array_decode(json_decode($this->sendScript(file_get_contents($HCS_ROOTPATH . 'includes/scripts/devices.fn'), true), true), true));
        Logging::info("[Homematic_ReGaHSS->getDeviceList] received device list:\n" . var_export($response, true));
        return $response;
    }

    private function send($param_command, $fileHandle, $param_trimmed = true)
    {
        // set no limit for execution time. important for device updates.
        set_time_limit(0);
        $postString = "POST /tclrega.exe HTTP/1.1\r\n";
        $postString = $postString . "Content-type: application/x-www-form-urlencoded\r\n";
        $postString = $postString . "Connection: Close\r\n";
        $postString = $postString . "Content-Length: " . strlen($param_command) . "\r\n\r\n";

        // fwrite(handle, string) writes the contents of string to the file stream pointed to by handle. 
        fwrite($fileHandle, $postString . $param_command);
        Logging::debug("[Homematic_ReGaHSS->send] sent: " . $postString . $param_command);

        $tempString = fgets($fileHandle);
//        Logging::debug("[Homematic_ReGaHSS->send] DEBUG fgets(): " . $tempString);
        $debugString = $tempString;
        // skip over the unimportant messages to the next empty line
        while ($tempString != "\r\n") {
            $tempString = fgets($fileHandle);
//            Logging::debug("[Homematic_ReGaHSS->send] DEBUG fgets(): " . $tempString);
            $debugString .= $tempString;
        }


        // get lines
        $answer = "";
        while (!feof($fileHandle)) {
            $tempString = fgets($fileHandle);
//            Logging::debug("[Homematic_ReGaHSS->send] DEBUG fgets(): " . $tempString);
            $answer .= $tempString;
        }
        // close socket connection
        fclose($fileHandle);
        Logging::debug("[Homematic_ReGaHSS->send] closed socket at server: \"" . $this->server . "\" on port: \"" . $this->port . "\".");

        $debugString .= $answer;
        Logging::debug("[Homematic_ReGaHSS->send] received: " . $debugString);

        // parse result
        if ($param_trimmed) {
            $pos1 = (strpos($answer, "</exec>") + strlen("</exec>"));
            $pos2 = (strpos($answer, "</sessionId>") + strlen("</sessionId>"));
            $pos3 = (strpos($answer, "</httpUserAgent>") + strlen("</httpUserAgent>"));
            return new SimpleXMLElement(utf8_encode("<xml>" . substr($answer, max($pos1, $pos2, $pos3))));
        } else {
            return $answer;
        }
    }
}

?>