<?php
// <editor-fold defaultstate="collapsed" desc="CALLABLE PROTECTION">
// Make file not callable directly
if ( !defined('HCS_CALLABLE_FILE') ) {
    header('HTTP/1.0 404 Not Found');
    die('You shall not access this file directly.');
    exit;
}
// </editor-fold>
/* -------------------------------------------------------------------------- */

require_once 'sql.php';
require_once 'common_functions.php';
require_once 'tasker_functions.php';

// Default the site-wide variables
if (!isset($HCS_ROOTPATH)) {
    $HCS_ROOTPATH = '../../';
}

function LogEvent($param_ccuId, $param_device, $param_dataPoint, $param_value, SQL $param_sql = null) {
    $sql = null;
    if ( $param_sql != null ) {
        $sql = $param_sql;
    } else {
        $sql = new SQL();
    }

    $loggingRulesArray = getLoggingRulesArray($sql);
    $loggingRules = new HomematicLoggingRules();
    $loggingRules->rulesFromArray($loggingRulesArray);

    $Timestamp = "";
    $LastValue = "";
    $LastEvent = "";
    if ( $sql->type == "MSSQL" ) {
        $LastEvent = $sql->queryRequest("SELECT TOP 1 Timestamp, Value FROM " . HCS_TABLE_PREFIX . "HistoryStates WHERE CcuId = '" . $param_ccuId . "' AND ChannelAddress = '" . $param_device . "' AND DataPoint = '" . $param_dataPoint . "' ORDER BY Timestamp DESC");
    } else {
        $LastEvent = $sql->queryRequest("SELECT Timestamp, Value FROM " . HCS_TABLE_PREFIX . "HistoryStates WHERE CcuId = '" . $param_ccuId . "' AND ChannelAddress = '" . $param_device . "' AND DataPoint = '" . $param_dataPoint . "' ORDER BY Timestamp DESC LIMIT 1");
    }
    if ( count($LastEvent) > 0 ) {
        $Timestamp = $LastEvent[0]["Timestamp"];
        $LastValue = $LastEvent[0]["Value"];
    }
    $current_time = date(SQL_SERVER_DATETIME_FORMAT);
    $logging = $loggingRules->checkLoggingCondition($param_ccuId, $param_device, $param_dataPoint, $param_value, $Timestamp, $LastValue);
    if ( !($logging === false) ) {
        $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'HistoryStates', array(
            'ChannelAddress' => $param_device,
            'DataPoint' => $param_dataPoint,
            'Value' => $param_value,
            'Timestamp' => $current_time,
            'Expiration' => $logging,
            'CcuId' => $param_ccuId
        ));
    }
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'CurrentStates', array(
        'ChannelAddress' => $param_device,
        'DataPoint' => $param_dataPoint,
        'LastUpdate' => $current_time,
        'Value' => $param_value,
        'CcuId' => $param_ccuId), array('CcuId', 'ChannelAddress', 'DataPoint'));

    // Delete old entries
    $answer = $sql->queryRequest("SELECT Value FROM " . HCS_TABLE_PREFIX . "Values WHERE Name = 'LastHistoryCleanup'");
    if ( ((int)$answer[0]['Value'] + MIN_CLEANUP_INTERVAL) < time()){
        $sql->query("DELETE FROM " . HCS_TABLE_PREFIX . "HistoryStates WHERE Expiration <= '" . $current_time . "';");
        $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'LastHistoryCleanup', 'Value' => time()), array('Name'));
    }
    executeTasker('', false, $sql);
}

function getLoggingRulesArray($param_sql = null){
    global $HCS_ROOTPATH;
    $sql = null;
    $writeNew = false;
    $loggingRulesArray = array();


    if ($param_sql != null) {
        $sql = $param_sql;
    } else {
        $sql = new SQL();
    }

    $fileToken = $HCS_ROOTPATH . 'data/cache/rules_delete';
    $file = $HCS_ROOTPATH . 'data/cache/rules.json';
    if ( (file_exists($fileToken) && !is_dir($fileToken)) || (!is_readable($file)) ) {
        $writeNew = true;
        deleteCacheFile($file, $fileToken);
    }

    if(!$writeNew){
        $content = file_get_contents($file);
        if ($content !== false ){
            $loggingRulesArray = json_decode($content, true);
            Logging::info("[Homematic_Logging->getLoggingRulesArray] Got logging rules array from cache.");
        } else {
            $writeNew = true;
            Logging::warning("[Homematic_Logging->getLoggingRulesArray] Could not get logging rules array from cache.");
        }
    }

    if ($writeNew){
        deleteCacheFile($file, $fileToken);
        $loggingRulesArray = $sql->queryRequest("SELECT CcuId, Device, DataPoint, Value, LoggingInterval, LifeSpan, OnlyLogChanges FROM " . HCS_TABLE_PREFIX . "LoggingRules ORDER BY Priority DESC;");
        file_put_contents($file, json_encode($loggingRulesArray));
        Logging::info("[Homematic_Logging->getLoggingRulesArray] Got logging rules array from Database.");
    } else {
    }

    return $loggingRulesArray;
}

class HomematicLoggingRule {

    private $ccuIdCondition = "*";
    private $deviceCondition = "*";
    private $dataPointCondition = "*";
    private $valueCondition = "*";
    private $loggingInterval = 600;    // every 10 minutes
    public $loggingExpiration = 1209600;    // 14 days
    private $onlyLoggingChange = false;

    function __construct($param_ccuIdCondition = "*", $param_deviceCondition = "*", $param_dataPointCondition = "*", $param_valueCondition = "*", $param_loggingInterval = 600, $param_loggingExpiration = 1209600, $param_onlyLoggingChange = false) {
        $this->ccuIdCondition = $param_ccuIdCondition;
        $this->deviceCondition = $param_deviceCondition;
        $this->dataPointCondition = $param_dataPointCondition;
        $this->valueCondition = $param_valueCondition;
        $this->loggingInterval = $param_loggingInterval;
        $this->loggingExpiration = $param_loggingExpiration;
        $this->onlyLoggingChange = (boolean) $param_onlyLoggingChange;
        if ( "$param_onlyLoggingChange" == "false" ) {
            $this->onlyLoggingChange = false;
        }

    }

    function checkLoggingCondition($param_ccuId, $param_device, $param_dataPoint, $param_value, $param_timestamp, $param_lastValue) {
        if ( fnmatch($this->ccuIdCondition, $param_ccuId, FNM_CASEFOLD) && 
                fnmatch($this->deviceCondition, $param_device, FNM_CASEFOLD) &&
                fnmatch($this->dataPointCondition, $param_dataPoint, FNM_CASEFOLD) &&
                fnmatch($this->valueCondition, $param_value, FNM_CASEFOLD) ) {

            if ( ($this->loggingExpiration <= 0) || ($this->loggingExpiration == "0") ) {
                // found matching rule, but expiration time is to low.
                return -1;
            } elseif ( (strtotime("now") - strtotime($param_timestamp)) <= $this->loggingInterval ) {
                // found matching rule, but last event is not long enough ago 
                return -1;
            } elseif ( ($this->onlyLoggingChange) && ($param_lastValue != "") && ($param_lastValue == $param_value) ) {
                // found matching rule, but last value is the same and we won't logg same values.
                return -1;
            } else {
                // found matching rule!
                return 1;
            }
        } else {
            return 0;
        }

    }

    function ruleFromArray(array $param_ruleArray) {
        if ( count($param_ruleArray) == 7 ) {
            $this->ccuIdCondition = $param_ruleArray["CcuId"];
            $this->deviceCondition = $param_ruleArray["Device"];
            $this->dataPointCondition = $param_ruleArray["DataPoint"];
            $this->valueCondition = $param_ruleArray["Value"];
            $this->loggingInterval = $param_ruleArray["LoggingInterval"];
            $this->loggingExpiration = $param_ruleArray["LifeSpan"];
            $this->onlyLoggingChange = (boolean) $param_ruleArray["OnlyLogChanges"];
            if ( ((string) $param_ruleArray["OnlyLogChanges"] == "false") || ((string) $param_ruleArray["OnlyLogChanges"] == "0") ) {
                $this->onlyLoggingChange = false;
            }
            return true;
        } else {
            return false;
        }

    }

}

class HomematicLoggingRules {

    private $rules;

    function __construct() {
        $this->rules = array();

    }

    function addRule(HomematicLoggingRule $param_rule) {
        $this->rules[] = $param_rule;

    }

    function rulesFromArray($param_ruleArrays) {
        foreach ( $param_ruleArrays as $ruleArray ) {
            $tempRule = new HomematicLoggingRule();
            if ( $tempRule->ruleFromArray($ruleArray) ) {
                $this->addRule($tempRule);
            }
        }

    }

    function checkLoggingCondition($param_ccuId, $param_device, $param_dataPoint, $param_value, $param_timestamp, $param_lastValue) {
        $result = "";
        foreach ( $this->rules as $rule ) {
            if ( $rule instanceof HomematicLoggingRule ) {
                $result = $rule->checkLoggingCondition($param_ccuId, $param_device, $param_dataPoint, $param_value, $param_timestamp, $param_lastValue);
                if ( $result == 1 ) {
                    return date(SQL_SERVER_DATETIME_FORMAT, time() + $rule->loggingExpiration);
                } elseif ( $result == -1 ) {
                    return false;
                }
            }
        }
        return false;

    }

}

?>