<?php

// <editor-fold defaultstate="collapsed" desc="CALLABLE PROTECTION">
// Make file not callable directly
if (!defined('HCS_CALLABLE_FILE')) {
    header('HTTP/1.0 404 Not Found');
    die('Do not access this file directly.');
}
// </editor-fold>
/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="INCLUDES">
require_once('functions.php');
require_once('common_functions.php');
require_once('homematic_logging.php');
require_once('sql.php');

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="FUNCTIONS TASKER">

function executeTasker($param_ids = '', $param_force = false, SQL $param_sql = null)
{
    LoggingSingleton::getInstance()->logError = true;
    LoggingSingleton::getInstance()->logToArray = true;

    $current_time = date(SQL_SERVER_DATETIME_FORMAT);
    $idQueryString = "";
    if ($param_ids != '') {
        $searchIds = array();
        $ids = explode(",", sqlEscapeString($param_ids));
        foreach ($ids as $id) {
            if (!(strpos($id, "-") === false)) {
                $range = explode("-", $id);
                if (count($range) == 2) {
                    if ($range[0] <= $range [1]) {
                        $min = (int)$range[0];
                        $max = (int)$range[1];
                    } else {
                        $min = (int)$range[1];
                        $max = (int)$range[0];
                    }
                    for ($i = $min; $i <= $max; $i++) {
                        $searchIds[] = "'" . $i . "'";
                    }
                }
            } else {
                $searchIds[] = "'" . $id . "'";
            }
        }

        $idQueryString = "(" . HCS_TABLE_PREFIX . "Tasks.Id IN (" . implode(",", $searchIds) . ") )";
    }
    $sql = null;
    if ( $param_sql != null ) {
        $sql = $param_sql;
    } else {
        $sql = new SQL();
    }

    if ($param_force){
        $tasks = $sql->queryRequest("SELECT " . HCS_TABLE_PREFIX . "Tasks.Id, CcuId, CcuAddress, CallbackAddress, Type, Interface, Designator, Parameter, NextExecution, ExecutionInterval FROM (" . HCS_TABLE_PREFIX . "Tasks LEFT JOIN " . HCS_TABLE_PREFIX . "CcuList ON " . HCS_TABLE_PREFIX . "Tasks.CcuId = " . HCS_TABLE_PREFIX . "CcuList.Id) WHERE " . $idQueryString . ";");
    } else {
        if($idQueryString == ''){
            $tasks = $sql->queryRequest("SELECT " . HCS_TABLE_PREFIX . "Tasks.Id, CcuId, CcuAddress, CallbackAddress, Type, Interface, Designator, Parameter, NextExecution, ExecutionInterval FROM (" . HCS_TABLE_PREFIX . "Tasks LEFT JOIN " . HCS_TABLE_PREFIX . "CcuList ON " . HCS_TABLE_PREFIX . "Tasks.CcuId = " . HCS_TABLE_PREFIX . "CcuList.Id) WHERE ( (NextExecution != '') AND (NextExecution <= '" . $current_time . "') AND (ExecutionInterval >= 0) );");
        } else {
            $tasks = $sql->queryRequest("SELECT " . HCS_TABLE_PREFIX . "Tasks.Id, CcuId, CcuAddress, CallbackAddress, Type, Interface, Designator, Parameter, NextExecution, ExecutionInterval FROM (" . HCS_TABLE_PREFIX . "Tasks LEFT JOIN " . HCS_TABLE_PREFIX . "CcuList ON " . HCS_TABLE_PREFIX . "Tasks.CcuId = " . HCS_TABLE_PREFIX . "CcuList.Id) WHERE ( (NextExecution != '') AND (NextExecution <= '" . $current_time . "') AND (ExecutionInterval >= 0) ) AND " . $idQueryString . ";");
        }
    }

    // get all CCU addresses
    $ccuAddresses = array();
    foreach ($tasks as $checkTask) {
        if (!in_array($checkTask["CcuAddress"], $ccuAddresses)) {
            $ccuAddress = $checkTask["CcuAddress"];
            // check if CCUs are reachable if not delete all related tasks from list.
            $ccuAddresses[] = $ccuAddress;
            if (ping($ccuAddress) === false) {
                foreach ($tasks as $key => $task) {
                    if ($ccuAddress == $task["CcuAddress"]) {
                        unset($tasks[$key]);
                    }
                }
            }
        }
    }

    foreach ($tasks as $task) {
        // set interval and remove next execution until finished
        $interval = $task["ExecutionInterval"];
        if ($interval <= 0) {
            $interval = -1;
            $nextExecution = '';
        } else {
            $nextExecution = date(SQL_SERVER_DATETIME_FORMAT, time() + $interval);
        }

        $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Tasks', array(
                'Id' => $task["Id"],
                'NextExecution' => '',
                'ExecutionInterval' => $interval)
            , array('Id'));

        switch ($task["Type"]) {
            case "init":
                if (($task["CcuId"] != null) && ($task["Designator"] != null)) {
                    $interfaces = explode(",", $task["Designator"]);
                    if (count($interfaces) > 0) {
                        Logging::info('[Tasker] subscribing to ' . $task["Designator"] . ' events ...');
                        foreach ($interfaces as $interface) {
                            if (strtoupper($interface) == 'RF') {
                                $sql->queryExec("UPDATE " . HCS_TABLE_PREFIX . "CcuList SET SubscribeRfEvents = 1 WHERE Id = " . (int)$task["CcuId"] . ";");
                            } elseif (strtoupper($interface) == 'WIRED') {
                                $sql->queryExec("UPDATE " . HCS_TABLE_PREFIX . "CcuList SET SubscribeWiredEvents = 1 WHERE Id = " . (int)$task["CcuId"] . ";");
                            } elseif (strtoupper($interface) == 'SYSTEM') {
                                $sql->queryExec("UPDATE " . HCS_TABLE_PREFIX . "CcuList SET SubscribeSystemEvents = 1 WHERE Id = " . (int)$task["CcuId"] . ";");
                            } else {
                                Logging::error("[Tasker] 'Designator' pattern is wrong: " . print_r($task["Designator"], true) . "\n Must be: 'INTERFACE,INTERFACE,INTERFACE' (INTERFACE=RF|WIRED|SYSTEM)");
                            }
                        }
                        HomematicControl::updateCcuSubscription((int)$task["CcuId"], $sql);
                    } else {
                        Logging::error("[Tasker] 'Designator' pattern is wrong: " . print_r($task["Designator"], true) . "\n must be: 'INTERFACE,INTERFACE,INTERFACE' (INTERFACE=RF|WIRED|SYSTEM)");
                    }
                } else {
                    Logging::error("[Tasker] could not subscribed to XML-RPC interface. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true));
                }
                break;
            case "deinit":
                if (($task["CcuId"] != null) && ($task["Designator"] != null)) {
                    $interfaces = explode(",", $task["Designator"]);
                    if (count($interfaces) > 0) {
                        Logging::info('[Tasker] unsubscribing from ' . $task["Designator"] . ' events ...');
                        foreach ($interfaces as $interface) {
                            if (strtoupper($interface) == 'RF') {
                                $sql->queryExec("UPDATE " . HCS_TABLE_PREFIX . "CcuList SET SubscribeRfEvents = 0 WHERE Id = " . (int)$task["CcuId"] . ";");
                            } elseif (strtoupper($interface) == 'WIRED') {
                                $sql->queryExec("UPDATE " . HCS_TABLE_PREFIX . "CcuList SET SubscribeWiredEvents = 0 WHERE Id = " . (int)$task["CcuId"] . ";");
                            } elseif (strtoupper($interface) == 'SYSTEM') {
                                $sql->queryExec("UPDATE " . HCS_TABLE_PREFIX . "CcuList SET SubscribeSystemEvents = 0 WHERE Id = " . (int)$task["CcuId"] . ";");
                            } else {
                                Logging::error("[Tasker] 'Designator' pattern is wrong: " . print_r($task["Designator"], true) . "\n Must be: 'INTERFACE,INTERFACE,INTERFACE' (INTERFACE=RF|WIRED|SYSTEM)");
                            }
                        }
                        HomematicControl::updateCcuSubscription((int)$task["CcuId"], $sql);
                    } else {
                        Logging::error("[Tasker] 'Designator' pattern is wrong: " . print_r($task["Designator"], true) . "\n must be: 'INTERFACE,INTERFACE,INTERFACE' (INTERFACE=RF|WIRED|SYSTEM)");
                    }
                } else {
                    Logging::error("[Tasker] could not subscribed to XML-RPC interface. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true));
                }
                break;
                break;
            case "update":
            case "updateinit":
            case "updateInit":
            case "updateSubscription":
                if ($task["CcuId"] != null) {
                    Logging::info('[Tasker] updating CCU event subscriptions for CCU ID:' . (int)$task["CcuId"]);
                    HomematicControl::updateCcuSubscription((int)$task["CcuId"], $sql);
                } else {
                    Logging::error("[Tasker] could not update event subscription. Not all parameters set. " . print_r($task["CcuId"], true));
                }

                break;
            case "updateallinit":
            case "updateAllinit":
            case "updateAllInit":
            case "updateAllSubscriptions":
                Logging::info('[Tasker] updating all CCU event subscriptions.');
                HomematicControl::updateAllCcuSubscriptions($sql);
                break;
            case "getValue":
            case "getValueByName":
                if (($task["CcuId"] != null) && ($task["Designator"] != null) && ($task["Parameter"] != null)) {
                    $interface = "XML-RPC";
                    if (($task["Interface"] != null) && ($task["Interface"] == "HSS")) {
                        $interface = "HSS";
                    }
                    Logging::info("[Tasker] getting value from: " . $task["Designator"] . ":" . $task["Parameter"] . '...');
                    HomematicControl::getValueByName($task["CcuId"], $task["Designator"], $task["Parameter"], $interface, $sql);
                } else {
                    Logging::error("[Tasker] could not read device. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true) . " " . print_r($task["Parameter"], true));
                }

                break;
            case "getValueBySn":
                if (($task["CcuId"] != null) && ($task["Designator"] != null) && ($task["Parameter"] != null)) {
                    $interface = "XML-RPC";
                    if (($task["Interface"] != null) && ($task["Interface"] == "HSS")) {
                        $interface = "HSS";
                    }
                    Logging::info("[Tasker] getting value from: " . $task["Designator"] . ":" . $task["Parameter"] . '...');
                    HomematicControl::getValueBySn($task["CcuId"], $task["Designator"], $task["Parameter"], $interface, $sql);
                } else {
                    Logging::error("[Tasker] could not read device. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true) . " " . print_r($task["Parameter"], true));
                }

                break;
            case "setValue":
            case "setValueByName":
                if (($task["CcuId"] != null) && ($task["Designator"] != null) && ($task["Parameter"] != null)) {
                    $parameters = explode("=", $task["Parameter"]);
                    if ((count($parameters) == 2) && ($parameters[0] != null) && ($parameters[1] != null)) {
                        $interface = "XML-RPC";
                        if (($task["Interface"] != null) && ($task["Interface"] == "HSS")) {
                            $interface = "HSS";
                        }
                        Logging::info("[Tasker] setting value: " . $parameters[1] . " to: " . $task["Designator"] . ":" . $parameters[0] . '...');
                        HomematicControl::setValueByName($task["CcuId"], $task["Designator"], $parameters[0], $parameters[1], $interface, $sql);
                    } else {
                        Logging::error("[Tasker] 'Parameter' pattern is wrong: " . print_r($task["Parameter"], true) . "\n Must be: 'DATAPOINT=VALUE'");
                    }
                } else {
                    Logging::error("[Tasker] could not read device. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true) . " " . print_r($task["Parameter"], true));
                }

                break;
            case "setValueBySn":
                if (($task["CcuId"] != null) && ($task["Designator"] != null) && ($task["Parameter"] != null)) {
                    $parameters = explode("=", $task["Parameter"]);
                    if ((count($parameters) == 2) && ($parameters[0] != null) && ($parameters[1] != null)) {
                        $interface = "XML-RPC";
                        if (($task["Interface"] != null) && ($task["Interface"] == "HSS")) {
                            $interface = "HSS";
                        }
                        Logging::info("[Tasker] setting value: " . $parameters[1] . " to: " . $task["Designator"] . ":" . $parameters[0] . '...');
                        HomematicControl::setValueBySn($task["CcuId"], $task["Designator"], $parameters[0], $parameters[1], $interface, $sql);
                    } else {
                        Logging::error("[Tasker] 'Parameter' pattern is wrong: " . print_r($task["Parameter"], true) . "\n Must be: 'DATAPOINT=VALUE'");
                    }
                } else {
                    Logging::error("[Tasker] could not read device. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true) . " " . print_r($task["Parameter"], true));
                }

                break;
            case "logAllSystemVariables":
                if (($task["CcuId"] != null)) {
                    $vars = explode(';', $task["Designator"]);
                    foreach ($vars as $var) {
                        Logging::info("[Tasker] getting all system variables " . $var . '...');
                        HomematicControl::getAllSystemVariables($task["CcuId"], $sql);
                    }
                } else {
                    Logging::error("[Tasker] could not get all system variables. Not all parameters set. " . print_r($task["CcuId"], true));
                }
                break;
            case "logSystemVariable":
            case "getSystemVariable":
                if (($task["CcuId"] != null) && ($task["Designator"] != null)) {
                    $vars = explode(';', $task["Designator"]);
                    foreach ($vars as $var) {
                        Logging::info("[Tasker] getting system variable " . $var . '...');
                        HomematicControl::getSystemVariable($task["CcuId"], $var, $sql);
                    }
                } else {
                    Logging::error("[Tasker] could not get system variable. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true));
                }
                break;
            case "setSystemVariable":
                if (($task["CcuId"] != null) && ($task["Designator"] != null) && ($task["Parameter"] != null)) {
                    Logging::info("[Tasker] setting system variable " . $task["Designator"] . " to value: " . $task["Parameter"] . '...');
                    HomematicControl::setSystemVariable($task["CcuId"], $task["Designator"], $task["Parameter"], $sql);
                } else {
                    Logging::error("[Tasker] could not set system variable. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true) . " " . print_r($task["Parameter"], true));
                }
                break;
            case "executeProgram":
                if (($task["Designator"] != null) && ($task["CcuId"] != null)) {
                    Logging::info("[Tasker] executing program: " . $task["Designator"] . '...');
                    HomematicControl::executeProgram($task["CcuId"], $task["Designator"], $sql);
                } else {
                    Logging::error("[Tasker] could not execute program. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["Designator"], true));
                }
                break;
            case "updateRoomList":
                if (($task["CcuId"] != null)) {
                    Logging::info("[Tasker] updating rooms list...");
                    HomematicControl::updateRoomList($task["CcuId"], $sql);
                } else {
                    Logging::error("[Tasker] could not update rooms list. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["CcuAddress"], true));
                }
                break;
            case "updateCraftList":
                if (($task["CcuId"] != null)) {
                    Logging::info("[Tasker] updating crafts list...");
                    HomematicControl::updateCraftList($task["CcuId"], $sql);
                } else {
                    Logging::error("[Tasker] could not update crafts list. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["CcuAddress"], true));
                }
                break;
            case "updateProgramList":
                if (($task["CcuId"] != null)) {
                    Logging::info("[Tasker] updating program list...");
                    HomematicControl::updateProgramsList($task["CcuId"], $sql);
                } else {
                    Logging::error("[Tasker] could not update program list. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["CcuAddress"], true));
                }
                break;
            case "updateDeviceList":
                if (($task["CcuId"] != null)) {
                    Logging::info("[Tasker] updating devices list...");
                    HomematicControl::updateDeviceList($task["CcuId"], $sql);
                } else {
                    Logging::error("[Tasker] could not update devices list. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["CcuAddress"], true));
                }
                break;
            case "updateSystemVariableList":
                if (($task["CcuId"] != null)) {
                    Logging::info("[Tasker] updating system variables list...");
                    HomematicControl::updateSystemVariableList($task["CcuId"], $sql);
                } else {
                    Logging::error("[Tasker] could not update system variables list. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["CcuAddress"], true));
                }
                break;
            case "updateLists":
                if (($task["CcuId"] != null)) {
                    Logging::info("[Tasker] updating all lists...");
                    HomematicControl::updateAllList($task["CcuId"], $sql);
                } else {
                    Logging::error("[Tasker] could not update lists. Not all parameters set. " . print_r($task["CcuId"], true) . " " . print_r($task["CcuAddress"], true));
                }
                break;

            default:
                break;
        }
        // set next execution
        $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Tasks', array(
                'Id' => $task["Id"],
                'NextExecution' => $nextExecution)
            , array('Id'));
    }
    LoggingSingleton::getInstance()->logError = LOGGING_ERROR_MESSAGE;
    LoggingSingleton::getInstance()->logToArray = LOGGING_LOG_TO_GLOBAL;
    if (count(LoggingSingleton::getErrorArray()) > 0) {
        return false;
    } else {
        return true;
    }

}

// </editor-fold>

/* -------------------------------------------------------------------------- */


// EOF
?>