<?php

// <editor-fold defaultstate="collapsed" desc="CALLABLE PROTECTION">
// Make file not callable directly
if ( !defined('HCS_CALLABLE_FILE') ) {
    header('HTTP/1.0 404 Not Found');
    die('You shall not access this file directly.');
    exit;
}
// </editor-fold>
/* -------------------------------------------------------------------------- */

require_once("xmlrpc.inc");
require_once("xmlrpcs.inc");
require_once("xmlrpc_wrappers.inc");

require_once("sql.php");
require_once("functions.php");
require_once("homematic_logging.php");
require_once("globals.php");

defined('XMLRPC_TIMEOUT') or define('XMLRPC_TIMEOUT', 30);

function extractScalarParams($param_xmlRpcMsg) {
    $MsgParamScalars = array();
    $MsgNumParams = $param_xmlRpcMsg->getNumParams();

    for ( $i = 0; $i < $MsgNumParams; $i++ ) {
        $Param = $param_xmlRpcMsg->getParam($i);
        if ( $Param->kindOf() == "scalar" ) {
            if ( $Param->scalarTyp() == "boolean" ) {
                if ( $Param->scalarVal() == true ) {
                    $MsgParamScalars[$i] = "true";
                } else if ( $Param->scalarVal() == false ) {
                    $MsgParamScalars[$i] = "false";
                }
            } else if ( $Param->scalarTyp() == "int" ) {
                $MsgParamScalars[$i] = (int) ($Param->scalarVal());
            } else if ( $Param->scalarTyp() == "double" ) {
                $MsgParamScalars[$i] = (double) ($Param->scalarVal());
            } else if ( $Param->scalarTyp() == "string" ) {
                $MsgParamScalars[$i] = (string) ($Param->scalarVal());
            } else {
                $MsgParamScalars[$i] = (string) ($Param->scalarVal());
            }
//        } else if( $Param->kindOf() == "array" ) {
//            for($j = 0; $i < $Param->arraySize(); $j++) {
//                // each arrayElement.kindOf() can be again "struct", "array" or "scalar"
//                //$arrayElement = $Param->arrayMem($j);
//            }
//        } else if( $Param->kindOf() == "struct" ) {
//            $Param->structreset();
//            while( list($structElementKey, $structElementValue) = $Param->structEach() ) {
//                // each structElementValue.kindOf() can be again "struct", "array" or "scalar"
//                //echo($structElementKey.":".$structElementValue);
//            }
        }
    }
    return $MsgParamScalars;

}

function homematicGateway_output($param_xmlRpcMsg) {
    $inhalt = date(LOGGING_DATE_TIME_STRING) . ":\n" . $param_xmlRpcMsg->serialize() . "\n\n";
    $handle = fopen("log.txt", "a");
    fwrite($handle, $inhalt);
    fclose($handle);

    return new xmlrpcresp(new xmlrpcval("", "string"));

}

function homematicGateway_blank($param_xmlRpcMsg) {
    return new xmlrpcresp(new xmlrpcval("", "string"));

}

function homematicGateway_eventScalar($param_xmlRpcScalar, SQL $param_sql = null) {
    $sql = null;
    if ( $param_sql != null ) {
        $sql = $param_sql;
    } else {
        $sql = new SQL();
    }

    $clientArray = array();
    $clientArray = $sql->queryRequest("SELECT Id, CallbackId, CallbackAddress FROM " . HCS_TABLE_PREFIX . "EventClients");

    foreach ( $clientArray as $client ) {
        $xmlRpc_client = new xmlrpc_client($client["CallbackAddress"]);
        $param_xmlRpcScalar[0] = $client["CallbackId"];
        $xmlRpcMsg = new xmlrpcmsg("event", $param_xmlRpcScalar);

        $xmlRpcResp = $xmlRpc_client->send($xmlRpcMsg, XMLRPC_TIMEOUT);
        if ( $xmlRpcResp === 0 ) {
            // some error occured during sending
            Logging::error("[Gateway] could not send event to client ID: " . $client["Id"] . "; CallbackID: " . $client["CallbackID"] . "; CallbackAddress: " . $client["CallbackAddress"] . "; ErrorNumber: \"" . $xmlRpc_client->errno . "\"; ErrorString: \"" . $xmlRpc_client->errstring . "\".");
            $sql->queryExec("DELETE FROM " . HCS_TABLE_PREFIX . "EventClients WHERE Id = '" . $client["Id"] . "';");
        }
    }
    return new xmlrpcresp(new xmlrpcval("", "string"));

}

function homematicGateway_event($param_xmlRpcMsg) {
    return homematicGateway_eventScalar(extractScalarParams($param_xmlRpcMsg));

}

function homematicGateway_init($param_xmlRpcMsg) {

    $scalarValues = extractScalarParams($param_xmlRpcMsg);
    $scalarCount = count($scalarValues);
    if ( $scalarCount == 2 ) {
        $sql = new SQL();
        $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . "EventClients", array("CallbackAddress" => $scalarValues[0], "CallbackId" => $scalarValues[1]));
    } else if ( $scalarCount == 1 ) {
        homematicGateway_deinit($param_xmlRpcMsg);
    }

    return new xmlrpcresp(new xmlrpcval("", "string"));

}

function homematicGateway_deinit($param_xmlRpcMsg) {
    $scalarValues = extractScalarParams($param_xmlRpcMsg);
    if ( count($scalarValues) == 1 ) {
        $sql = new SQL();
        $sql->queryExec("DELETE FROM " . HCS_TABLE_PREFIX . "EventClients WHERE CallbackAddress = '" . $scalarValues[0] . "';");
    }

    return new xmlrpcresp(new xmlrpcval("", "string"));

}

function homematicGateway_setValue($param_xmlRpcMsg) {
    $scalarValues = extractScalarParams($param_xmlRpcMsg);
    if ( count($scalarValues) == 3 ) {
        $sql = new SQL();
        $ccuIds = $sql->queryRequest("SELECT CcuId FROM " . HCS_TABLE_PREFIX . "Channels WHERE Address = '" . $scalarValues[0] . "';");
        foreach ( $ccuIds as $ccuId ) {
            HomematicControl::setValueBySn($ccuId["CcuId"], $scalarValues[0], $scalarValues[1], $scalarValues[2], "XML-RPC", $sql);
        }
    }
    return new xmlrpcresp(new xmlrpcval("", "string"));

}

?>