<?php

define('HCS_CALLABLE_FILE', true);
$HCS_ROOTPATH = '../';
$hcs_configFile = $HCS_ROOTPATH . 'config.php';

/* -------------------------------------------------------------------------- */


// <editor-fold defaultstate="collapsed" desc="CONFIG FILE INCLUDE">
// check if config file existst
if (!is_file($hcs_configFile)) {
    // try to create config file
    fopen($hcs_configFile, "w+");
    if (!is_file($hcs_configFile)) {
        // die if it is not possible to create the config file.
        die('Error: could not locate configuration file config.php in the HCS root folder. Create an empty config.php or set the install folder to chmod 777.');
    }
} else {
    // include config file
    include_once($hcs_configFile);
}
// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="PHP INCLUDES">

include_once($HCS_ROOTPATH . 'defaults.php');
include_once($HCS_ROOTPATH . 'includes/php/functions.php');
include_once($HCS_ROOTPATH . 'includes/php/common_functions.php');
include_once($HCS_ROOTPATH . 'includes/php/class_template.php');
// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="DEPRECATED FUNCTIONS">
@set_magic_quotes_runtime(0);
if (!get_magic_quotes_gpc()) {
    //$_GET = escapeData($_GET);
    //$_POST = escapeData($_POST);
}

// </editor-fold>

/* -------------------------------------------------------------------------- */

if (isset($_GET['language'])) {
    setLanguageCookie($_GET['language']);
    echo "<script>parent.window.location.href = 'index.php';</script>";
}
$installLanguage = (isset($_COOKIE['HCS_Language'])) ? $_COOKIE['HCS_Language'] : (isset($_DEFAULTS['default_lang']) ? $_DEFAULTS['default_lang'] : 'german');

if (!@include_once($HCS_ROOTPATH . 'language/' . $installLanguage . '/install.php')) {
    die('Could not include the language files! Check to make sure that "' . $HCS_ROOTPATH . 'language/' . $installLanguage . '/install.php" exists!');
}

if (isset($_GET['style'])) {
    setStyleCookie($_GET['style']);
    echo "<script>parent.window.location.href = 'index.php';</script>";
}
$installStyle = (isset($_COOKIE['HCS_Style'])) ? $_COOKIE['HCS_Style'] : 'homematic';

if (!@file_exists($HCS_ROOTPATH . 'style/' . $installStyle . '/style.css')) {
    die('Could not include the style files! Check to make sure that "' . $HCS_ROOTPATH . 'style/' . $installStyle . '/style.css" exists!');
}

$STEP = (isset($_POST['install_step'])) ? $_POST['install_step'] : (isset($_GET['install_step']) ? $_GET['install_step'] : '1');

// If HCS is already installed, don't install it again
if (defined('HCS_INSTALLED')) {
    $template = new InstallTemplate('install_message.html');
    $template->displayDieMessage($_LANGUAGE['install_header_error'], $_LANGUAGE['already_installed']);
    exit();
}

// View phpinfo() if requested
if ((isset($_GET['mode'])) && ($_GET['mode'] == 'phpinfo')) {
    phpinfo();
    exit;
}


/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="STEP SELECTION">
switch ($STEP) {
    case 1:
        installStep1();
        break;
    case 2:
        installStep2();
        break;
    case 3:
        installStep3();
        break;
    case 4:
        installStep4();
        break;
    case 5:
        installStep5();
        break;
    case 6:
        installStep6();
        break;
    case 7:
        installStep7();
        break;
    default:
        installStep1();
        break;
}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="STEP 1">
function installStep1()
{
    global $HCS_ROOTPATH, $DEFAULTS, $_LANGUAGE, $installLanguage, $installStyle, $hcs_configFile;

    $template = new InstallTemplate('install_step1.html');

    // Check to make sure config.php exists and is readable / writeable
    if (!file_exists($hcs_configFile)) {
        if (!@touch($hcs_configFile)) {
            $template->appendError($_LANGUAGE['conf_not_write']);
        } else {
            $template->appendMessage($template->StatusIcon('ok') . $_LANGUAGE['conf_written']);
        }
    } else {
        if ((!is_writeable($hcs_configFile)) || (!is_readable($hcs_configFile))) {
            if (!@chmod($hcs_configFile, 0666)) {
                $template->appendError($_LANGUAGE['conf_chmod']);
            } else {
                $template->appendMessage($template->StatusIcon('ok') . $_LANGUAGE['conf_writable']);
            }
        }
        // config file exists and is writeable, we're good to go
    }
    clearstatcache();

    // Check to make sure the required chmod 777 folders are writeable
    $directories = array('templates/cache', 'data', 'data/cache', 'data/log', 'data/export', 'data/import done');

    $passed['files'] = true;
    foreach ($directories as $dir) {
        $exists = $write = false;
        $MyDir = $HCS_ROOTPATH . $dir;

        // Try to create the directory if it does not exist
        if (!file_exists($MyDir)) {
            if (!@mkdir($MyDir, 0777)) {
                $template->appendError(sprintf($_LANGUAGE['templcache_notcreated'], $dir));
            } else {
                $template->appendMessage(sprintf($_LANGUAGE['templcache_created'], $dir));
            }
            @chmod($MyDir, 0777);
        }

        // Now really check
        if (file_exists($MyDir) && is_dir($MyDir)) {
            if (!@is_writable($MyDir)) {
                if (!@chmod($MyDir, 0777)) {
                    $template->appendError(sprintf($_LANGUAGE['templcache_notwritable'], $dir));
                } else {
                    $template->appendMessage($template->StatusIcon('ok') . $_LANGUAGE['templatecache_ok']);
                }
            }
            $exists = true;
        }

        // Now check if it is writable by storing a simple file
        $fp = @fopen($MyDir . '/test_lock', 'wb');
        if ($fp !== false) {
            $write = true;
        }
        @fclose($fp);

        @unlink($MyDir . '/test_lock');

        $passed['files'] = ($exists && $write && $passed['files']) ? true : false;

        $exists = ($exists) ? '<strong class="positive">' . $_LANGUAGE['inst_found'] . '</strong>' : '<strong class="negative">' . $_LANGUAGE['inst_notfound'] . '</strong>';
        $write = ($write) ? '<strong class="positive">' . $_LANGUAGE['inst_writable'] . '</strong>' : (($exists) ? '<strong class="negative">' . $_LANGUAGE['inst_unwritable'] . '</strong>' : '');

        $my_Icon = ($passed['files']) ? $template->StatusIcon('ok') : $template->StatusIcon('error');
        $template->appendMessage($my_Icon . sprintf($_LANGUAGE['cachefolder_out'], $dir, $exists, $write));
    }
    clearstatcache();

    //
    // Server settings
    //
    // HCS versions
    $currentHcsVersion = $_LANGUAGE['inst_unknown'];
    $plusfile = @file('http://hcs.xenorate.com/version_check');
    if (!$plusfile[0]) {
        $currentHcsVersion = $_LANGUAGE['connection_failed'];
    } else {
        $parse = explode('|', $plusfile[0]);
        if ($parse[0]) {
            $currentHcsVersion = $parse[0];
        }
    }

    // PHP Versions
    $our_php_version = ((phpversion() >= REQUIRED_PHP_VERSION) ? '<span class="positive">' : '<span class="negative">') . phpversion() . '</span>';
    $their_php_version = REQUIRED_PHP_VERSION . '+';

    // Modules
    $our_mysql = (extension_loaded('mysql')) ? '<span class="positive">' . $_LANGUAGE['inst_yes'] . '</span>' : '<span class="negative">' . $_LANGUAGE['inst_no'] . '</span>';
    $their_mysql = $_LANGUAGE['inst_yes'];
    $our_mssql = (extension_loaded('sqlsrv') || extension_loaded('mssql')) ? '<span class="positive">' . $_LANGUAGE['inst_yes'] . '</span>' : '<span class="negative">' . $_LANGUAGE['inst_no'] . '</span>';
    $their_mssql = $_LANGUAGE['inst_yes'];
    $our_xmlrpc = (extension_loaded('xmlrpc')) ? '<span class="positive">' . $_LANGUAGE['inst_yes'] . '</span>' : '<span class="negative">' . $_LANGUAGE['inst_no'] . '</span>';
    $their_xmlrpc = $_LANGUAGE['inst_yes'];
    // Safe Mode
    $our_safemode = (ini_get('safe_mode') == '1') ? '<span class="negative">' . $_LANGUAGE['inst_yes'] . '</span>' : '<span class="positive">' . $_LANGUAGE['inst_no'] . '</span>';
    $their_safemode = $_LANGUAGE['inst_no'];
    if (ini_get('safe_mode') == '1') {
        $template->appendMessage($template->StatusIcon('error') . '<b>' . $_LANGUAGE['lerror'] . ':</b> ' . $_LANGUAGE['safemode_on']);
    }


    // cURL Module
//    if ( function_exists('curl_version') ) {
//        $our_curl = curl_version();
//        if ( is_array($our_curl) ) {
//            $our_curl = '<span class="positive">' . $_LANGUAGE['inst_yes'] . ' (' . $our_curl['version'] . ')</span>';
//        } else {
//            $our_curl = '<span class="positive">' . $_LANGUAGE['inst_yes'] . ' (' . $our_curl . ')</span>';
//        }
//    } else {
//        $our_curl = '<span class="negative">' . $_LANGUAGE['inst_no'] . '</span>';
//        $template->appendMessage($template->StatusIcon('error') . '<b>' . $_LANGUAGE['lerror'] . ':</b> ' . $_LANGUAGE['curl_notavailable']);
//    }
//    $their_curl = $_LANGUAGE['inst_yes'];


    if (function_exists('fopen')) {
        $our_fopen = '<span class="positive">' . $_LANGUAGE['inst_yes'] . '</span>';
    } else {
        $our_fopen = '<span class="negative">' . $_LANGUAGE['inst_no'] . '</span>';
        $template->appendMessage($template->StatusIcon('error') . '<b>' . $_LANGUAGE['lerror'] . ':</b> ' . $_LANGUAGE['fopen_notavailable']);
    }
    $their_fopen = $_LANGUAGE['inst_yes'];

    if ((phpversion() < REQUIRED_PHP_VERSION) || ((!extension_loaded('mysql')) && (!extension_loaded('sqlsrv')) && (!extension_loaded('mssql')))) {
        $template->appendError('<span style="font-weight: bold; font-size: 14px;">' . $_LANGUAGE['minimal_requ_notfilled'] . '</span>');
    } else {
        $template->appendMessage($template->StatusIcon('ok') . $_LANGUAGE['minimal_requ_filled']);
    }

    // installation language
    $lang_path = $HCS_ROOTPATH . 'language/';
    $dir = @opendir($lang_path);
    if ($dir) {
        while ($file = @readdir($dir)) {
            $lang_folder = $lang_path . $file;
            if ((!is_file($lang_folder)) && (is_file($lang_folder . '/install.php')) && (!is_link($lang_folder)) && valid_folder($file)) {
                $template->assignBlockVariables('languageSelectRow', array(
                        'VALUE' => $file,
                        'OPTION' => ucfirst(strtolower($file)),
                        'SELECTED' => option_selected(strtolower($file) == strtolower($installLanguage)),
                    )
                );
            }
        }
    }

    // installation style
    $lang_path = $HCS_ROOTPATH . 'style/';
    $dir = @opendir($lang_path);
    if ($dir) {
        while ($file = @readdir($dir)) {
            $lang_folder = $lang_path . $file;
            if ((!is_file($lang_folder)) && (is_file($lang_folder . '/style.css')) && (!is_link($lang_folder)) && valid_folder($file)) {
                $template->assignBlockVariables('styleSelectRow', array(
                        'VALUE' => $file,
                        'OPTION' => ucfirst(strtolower($file)),
                        'SELECTED' => option_selected(strtolower($file) == strtolower($installStyle)),
                    )
                );
            }
        }
    }

    //
    // Output the page
    //
    $template->assignVariables(array(
            'HCS_VERSION' => HCS_VERSION,
            'CURRENT_HCS_VERSION' => (compareVersion($currentHcsVersion, HCS_VERSION) == 1) ? '<span class="negative" style="font-weight:bold;">' . $currentHcsVersion . '</span>' : '<span class="positive" style="font-weight:bold;">' . $currentHcsVersion . '</span>',
            'OUR_PHP_VERSION' => $our_php_version,
            'THEIR_PHP_VERSION' => $their_php_version,
            'OUR_MYSQL' => $our_mysql,
            'THEIR_MYSQL' => $their_mysql,
            'OUR_MSSQL' => $our_mssql,
            'THEIR_MSSQL' => $their_mssql,
            'OUR_XMLRPC' => $our_xmlrpc,
            'THEIR_XMLRPC' => $their_xmlrpc,
            'OUR_SAFEMODE' => $our_safemode,
            'THEIR_SAFEMODE' => $their_safemode,
            'OUR_FOPEN' => $our_fopen,
            'THEIR_FOPEN' => $their_fopen,
            'L_LANGUAGE_SELECT' => $_LANGUAGE['language_selector'],
            'L_INSTALL_LANGUAGE' => $_LANGUAGE['install_language'],
            'L_STYLE_SELECT' => $_LANGUAGE['style_selector'],
            'L_INSTALL_STYLE' => $_LANGUAGE['install_style'],
            'L_SERVER_VERSION' => $_LANGUAGE['inst_latest'],
            'L_HCSVERSION' => $_LANGUAGE['inst_hcsv'],
            'L_HCSNAME' => $_LANGUAGE['hcs_name'],
            'L_INST_PHP' => $_LANGUAGE['inst_php'],
            'L_VIEWPHPINFO' => $_LANGUAGE['inst_view'],
            'L_VERSION' => $_LANGUAGE['inst_version'],
            'L_REQUIRED' => $_LANGUAGE['inst_required'],
            'L_AVAILABLE' => $_LANGUAGE['inst_available'],
            'L_ENABLED' => $_LANGUAGE['inst_enabled'],
            'L_USING' => $_LANGUAGE['inst_using'],
            'L_MYSQLMODULE' => $_LANGUAGE['inst_mysqlmodule'],
            'L_MSSQLMODULE' => $_LANGUAGE['inst_mssqlmodule'],
            'L_XMLRPCMODULE' => $_LANGUAGE['inst_xmlrpcmodule'],
            'L_CURLMODULE' => $_LANGUAGE['inst_curlmodule'],
            'L_ZLIBMODULE' => $_LANGUAGE['inst_zlibmodule'],
            'L_SAFEMODE' => $_LANGUAGE['inst_safemode'],
            'L_FOPEN' => $_LANGUAGE['inst_fopen'],
            'L_INST_BUTTON1' => $_LANGUAGE['inst_button1'],
        )
    );

    $template->preparePageHeader();
    $template->prepratePageTailAndDisplay();

}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="STEP 2">
function installStep2()
{
    global $HCS_ROOTPATH, $_DEFAULTS, $_LANGUAGE, $_DBMS, $installLanguage, $installStyle, $hcs_configFile;
    $host = $_DBMS['mysql']['host'];
    $default_user = $_DBMS['mysql']['default_user'];
    $template = new InstallTemplate('install_step2.html');
    $extLoaded = false;
    $host = "";
    $default_user = "";
    // Build the database drop-down
    foreach ($_DBMS as $dbType => $db) {
        if ($dbType == 'mssql') {
            $extLoaded = (extension_loaded($db['module']) || extension_loaded('sqlsrv'));
        } else {
            $extLoaded = extension_loaded($db['module']);
        }
        if ($extLoaded) {
            if ($host == "") {
                $host = $db['host'];
            }
            if ($default_user == "") {
                $default_user = $db['default_user'];
            }
            $template->assignBlockVariables('dbmsRow', array(
                'VALUE' => $dbType,
                'HOST' => addslashes($db['host']),
                'OPTION' => $db['label'],
                'DEFAULT_USER' => $db['default_user']
            ));
        }
        $extLoaded = false;
    }
    unset($extLoaded);


    $template->assignVariables(array(
            'DB_HOST' => $host,
            'DB_USER' => $default_user,
            'TABLE_PREFIX' => $_DEFAULTS['table_prefix'],
            'L_DATABASE_CONF' => $_LANGUAGE['inst_database_conf'],
            'L_DATABASE_TYPE' => $_LANGUAGE['inst_dbtype'],
            'L_DATABASE_HOST' => $_LANGUAGE['inst_dbhost'],
            'L_DATABASE_NAME' => $_LANGUAGE['inst_dbname'],
            'L_DATABASE_USER' => $_LANGUAGE['inst_dbuser'],
            'L_DATABASE_PW' => $_LANGUAGE['inst_dbpass'],
            'L_DATABASE_PREFIX' => $_LANGUAGE['inst_table_prefix'],
            'L_BUTTON_2' => $_LANGUAGE['inst_button2'],
        )
    );

    $template->preparePageHeader();
    $template->prepratePageTailAndDisplay();

}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="STEP 3">
function installStep3()
{
    LoggingSingleton::getInstance()->logWarning = true;
    LoggingSingleton::getInstance()->logToArray = true;


    global $HCS_ROOTPATH, $_DEFAULTS, $_LANGUAGE, $_DBMS, $installLanguage, $installStyle, $hcs_configFile;
    $template = new InstallTemplate('install_step3.html');

    $dbtype = getPostIndexOrArrayValue('dbtype');
    $dbhost = getPostIndexOrArrayValue('dbhost');
    $dbname = getPostIndexOrArrayValue('dbname');
    $dbuser = getPostIndexOrArrayValue('dbuser');
    $dbpass = getPostIndexOrArrayValue('dbpass');
    $table_prefix = getPostIndexOrArrayValue('table_prefix', $_DEFAULTS);

    // prefix check
    if (strpos($table_prefix, '-') !== false || strpos($table_prefix, '.') !== false || strpos($table_prefix, "'") !== false || strpos($table_prefix, '\\') !== false || strpos($table_prefix, '/') !== false) {
        $template->appendError($_LANGUAGE['INST_ERR_PREFIX_INVALID']);
    }
    $prefixMaxLength = 36;
    if (strlen($table_prefix) > $prefixMaxLength) {
        $template->appendError(sprintf($_LANGUAGE['INST_ERR_PREFIX_TOO_LONG'], $prefixMaxLength));
    }

    // Database Check
    $error = array();
    $sql = @new SQL($dbtype, $dbname, $dbhost, $dbuser, $dbpass);

    // check if installation with prefix exists
    $suspiciousPrefix = strtolower($table_prefix);
    $suspiciousTables = array($suspiciousPrefix . 'devices', $suspiciousPrefix . 'cculist');
    $tables = array_map('strtolower', $sql->getTables());
    if (sizeof(array_intersect($tables, $suspiciousTables))) {
        Logging::warning($_LANGUAGE['INST_ERR_PREFIX']);
    }

    $myError = true;
    $db_message = '';

    if (!$table_prefix) {
        $db_message = $_LANGUAGE['inst_error_prefix'] . "<br /><br /><a id='redo' href='index.php'>" . $_LANGUAGE['inst_redoit'] . "</a>";
        $template->appendError($template->StatusIcon('error') . $db_message);
    } elseif (count(LoggingSingleton::getErrorArray()) > 0) {
        foreach (LoggingSingleton::getErrorArray() as $error) {
            $template->appendError($template->StatusIcon('error') . $error);
        }
        $template->appendError("<a id='redo' href='index.php'>" . $_LANGUAGE['inst_redoit'] . "</a>");
    } elseif (count(LoggingSingleton::getWarningArray()) > 0) {
        $myError = false;
        foreach (LoggingSingleton::getWarningArray() as $warning) {
            $template->appendMessage($template->StatusIcon('warning') . $warning);
        }
    } else {
        $myError = false;
        $db_message = $_LANGUAGE['insinfo_dbready'];
    }

    // Output the page


    if (!$myError) {
        switch ($dbtype) {
            case 'mysql':
                // I require MySQL version 4.0.4 minimum.
                $server_version = mysql_get_server_info();
                $client_version = mysql_get_client_info();
                break;

            case 'mssql':
                $query = $sql->queryRequest("SELECT @@VERSION;");
                $server_version = $query[0][''];
                break;

            default:
                break;
        }
        if ($db_message != '') {
            $template->appendMessage($template->StatusIcon('ok') . $db_message);
        }
        if ((isset($server_version) && isset($client_version) && ($dbtype == 'mysql'))) {
            $sql_infomssg = $template->StatusIcon('ok') . sprintf($_LANGUAGE['inst_mysqlinfo'], $server_version, $client_version);
        } elseif (isset($server_version) && ($dbtype == 'mssql')) {
            $sql_infomssg = $template->StatusIcon('ok') . sprintf($_LANGUAGE['inst_mssqlinfo'], $server_version);
        } else {
            $sql_infomssg = $template->StatusIcon('error') . sprintf($_LANGUAGE['inst_failedversioninfo'], $dbname, $dbuser, $dbhost);
        }
    }


    $template->assignVariables(array(
            'DB_PASS' => $dbpass,
            'DB_USER' => $dbuser,
            'DB_HOST' => $dbhost,
            'TABLE_PREFIX' => $table_prefix,
            'DB_NAME' => $dbname,
            'DB_TYPE' => $dbtype,
            'SQL_TITLE' => $_LANGUAGE['inst_sqlheaderbox'],
            'SQL_TEXT' => (!$myError) ? $sql_infomssg : '',
            'SHOW_SQLINFO' => ($myError) ? false : true,
            'L_BUTTON3' => $_LANGUAGE['inst_button3'],
        )
    );

    $template->preparePageHeader();
    $template->prepratePageTailAndDisplay();

}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="STEP 4">
function installStep4()
{
    global $HCS_ROOTPATH, $_DEFAULTS, $_LANGUAGE, $_DBMS, $installLanguage, $installStyle, $hcs_configFile;
    $template = new InstallTemplate('install_step4.html');

    $dbtype = getPostIndexOrArrayValue('dbtype');
    $dbhost = getPostIndexOrArrayValue('dbhost');
    $dbname = getPostIndexOrArrayValue('dbname');
    $dbuser = getPostIndexOrArrayValue('dbuser');
    $dbpass = getPostIndexOrArrayValue('dbpass');
    $table_prefix = getPostIndexOrArrayValue('table_prefix', $_DEFAULTS);


    $serverName = (!empty($_SERVER['SERVER_ADDR'])) ? $_SERVER['SERVER_ADDR'] : ((!empty($_SERVER['LOCAL_ADDR'])) ? $_SERVER['LOCAL_ADDR'] : ((!empty($_SERVER['SERVER_NAME'])) ? $_SERVER['SERVER_NAME'] : $_ENV['SERVER_ADDR']));
    if ((!empty($_SERVER['SERVER_PORT'])) || (!empty($_ENV['SERVER_PORT']))) {
        $serverPort = (!empty($_SERVER['SERVER_PORT'])) ? $_SERVER['SERVER_PORT'] : $_ENV['SERVER_PORT'];
    } else {
        $serverPort = '80';
    }
    $serverPath = str_replace('install/index.php', '', $_SERVER['PHP_SELF']);

    $template->assignVariables(array(
            'DB_PASS' => $dbpass,
            'DB_USER' => $dbuser,
            'DB_HOST' => $dbhost,
            'TABLE_PREFIX' => $table_prefix,
            'DB_NAME' => $dbname,
            'DB_TYPE' => $dbtype,
            'EVENT_SERVER_IP' => $serverName,
            'EVENT_SERVER_PORT' => $serverPort,
            'EVENT_SERVER_PATH' => $serverPath,
            'L_BUTTON_STEP_4' => $_LANGUAGE['L_BUTTON_STEP_4'],
            'L_EVENT_SERVER_HEADER' => $_LANGUAGE['L_EVENT_SERVER_HEADER'],
            'L_EVENT_SERVER_EXPLANATION' => $_LANGUAGE['L_EVENT_SERVER_EXPLANATION'],
            'L_EVENT_SERVER_IP' => $_LANGUAGE['L_EVENT_SERVER_IP'],
            'L_EVENT_SERVER_PORT' => $_LANGUAGE['L_EVENT_SERVER_PORT'],
            'L_EVENT_SERVER_PATH' => $_LANGUAGE['L_EVENT_SERVER_PATH'],
            'L_CCU_LIST_HEADER' => $_LANGUAGE['L_CCU_LIST_HEADER'],
            'L_CCU_EXPLANATION' => $_LANGUAGE['L_CCU_EXPLANATION'],
            'L_CCU_NAME' => $_LANGUAGE['L_CCU_NAME'],
            'L_CCU_IP' => $_LANGUAGE['L_CCU_IP'],
            'L_CCU_VERSION' => $_LANGUAGE['L_CCU_VERSION'],
            'L_CCU_EVENT_ABO' => $_LANGUAGE['L_CCU_EVENT_ABO'],
            'L_CCU_EVENT_WIRED' => $_LANGUAGE['L_CCU_EVENT_WIRED'],
            'L_CCU_EVENT_RF' => $_LANGUAGE['L_CCU_EVENT_RF'],
            'L_CCU_EVENT_SYSTEM' => $_LANGUAGE['L_CCU_EVENT_SYSTEM'],
            'L_CCU_ADD' => $_LANGUAGE['L_CCU_ADD'],
            'L_CCU_ADD_EMPTY_NAME' => $_LANGUAGE['L_CCU_ADD_EMPTY_NAME'],
            'L_CCU_ADD_EMPTY_IP' => $_LANGUAGE['L_CCU_ADD_EMPTY_IP'],
            'L_CCU_ADD_WIRED' => $_LANGUAGE['L_CCU_ADD_WIRED'],
            'L_CCU_ADD_RF' => $_LANGUAGE['L_CCU_ADD_RF'],
            'L_CCU_ADD_SYSTEM' => $_LANGUAGE['L_CCU_ADD_SYSTEM'],
            'L_CCU_DELETE' => $_LANGUAGE['L_CCU_DELETE'],
            'L_CCU_EDIT' => $_LANGUAGE['L_CCU_EDIT'],
            'L_CCU_ADD_CANCEL' => $_LANGUAGE['L_CCU_ADD_CANCEL'],
            'L_EVENT_SERVER_ADDRESS_WARNING' => $_LANGUAGE['L_EVENT_SERVER_ADDRESS_WARNING'],
            'L_EVENT_SERVER_IP_WARNING' => $_LANGUAGE['L_EVENT_SERVER_IP_WARNING'],
            'L_CCU_WARNING' => $_LANGUAGE['L_CCU_WARNING'],
            'L_WATING_TIME_INFO' => $_LANGUAGE['L_WATING_TIME_INFO'],
        )
    );

    $template->preparePageHeader();
    $template->prepratePageTailAndDisplay();

}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="STEP 5">
function installStep5()
{
    LoggingSingleton::getInstance()->logInfo = true;
    LoggingSingleton::getInstance()->logWarning = true;
    LoggingSingleton::getInstance()->logError = true;
    LoggingSingleton::getInstance()->logToArray = true;

    global $HCS_ROOTPATH, $_DEFAULTS, $_LANGUAGE, $_DBMS, $installLanguage, $installStyle, $hcs_configFile;
    $template = new InstallTemplate('install_step5.html');

    $dbtype = getPostIndexOrArrayValue('dbtype');
    $dbhost = getPostIndexOrArrayValue('dbhost');
    $dbname = getPostIndexOrArrayValue('dbname');
    $dbuser = getPostIndexOrArrayValue('dbuser');
    $dbpass = getPostIndexOrArrayValue('dbpass');
    $table_prefix = getPostIndexOrArrayValue('table_prefix', $_DEFAULTS);

    $ccus = getPostIndexOrArrayValue('ccus');

    $eventServerIp = getPostIndexOrArrayValue('eventServerIp');
    $eventServerPort = getPostIndexOrArrayValue('eventServerPort');
    $eventServerPath = getPostIndexOrArrayValue('eventServerPath');

    $callbackAddress = $eventServerIp . ':' . $eventServerPort . $eventServerPath . 'server.php';
    $installError = false;
    $installWarning = false;
    $installInfo = '';

    //wirte config file // SQLite, SQLite3, MySQL, MSSQL

    $config_file = "";
    $config_file .= "<?php\n\n";
    $config_file .= "define('HCS_SERVER_NAME', '" . $eventServerIp . "');\n";
    $config_file .= "define('HCS_SERVER_PORT', '" . $eventServerPort . "');\n";
    $config_file .= "define('HCS_SERVER_PATH', '" . $eventServerPath . "');\n";
    $config_file .= "\n";
    $config_file .= "define('HCS_TABLE_PREFIX', '" . $table_prefix . "');\n";
    $config_file .= "\n";
    $config_file .= "define('SQL_SERVER_TYPE', '" . $_DBMS[$dbtype]['name'] . "');\n";
    $config_file .= "define('SQL_SERVER_HOST', '" . $dbhost . "');\n";
    $config_file .= "define('SQL_SERVER_USERNAME', '" . $dbuser . "');\n";
    $config_file .= "define('SQL_SERVER_PASSWORD', '" . $dbpass . "');\n";
    $config_file .= "define('SQL_SERVER_DATABASE', '" . $dbname . "');\n";
    $config_file .= "define('SQL_SERVER_DATETIME_FORMAT', 'Y-m-d H:i:s');\n";
    $config_file .= "\n";
    $config_file .= "define('LOGGING_INFO_MESSAGE', false);\n";
    $config_file .= "define('LOGGING_WARNING_MESSAGE', false);\n";
    $config_file .= "define('LOGGING_DEBUG_MESSAGE', false);\n";
    $config_file .= "define('LOGGING_ERROR_MESSAGE', true);\n";
    $config_file .= "define('LOGGING_PRINT_TO_FILE', true);\n";
    $config_file .= "define('LOGGING_ECHO_TO_SCREEN', false);\n";
    $config_file .= "define('LOGGING_LOG_TO_GLOBAL', true);\n";
    $config_file .= "define('LOGGING_FILE_PATH', 'data/log');\n";
    $config_file .= "define('LOGGING_DATE_TIME_STRING', 'Y/m/d H:i:s');\n";
    $config_file .= "define('HCS_TIMEZONE', 'Europe/Berlin');\n";
    $config_file .= "date_default_timezone_set(HCS_TIMEZONE);\n";
    $config_file .= "\n";
    $config_file .= "define('XMLRPC_TIMEOUT', 30);\n";
    $config_file .= "define('REGAHSS_TIMEOUT', 30);\n";
    $config_file .= "define('PING_TIMEOUT', 1);\n";
    $config_file .= "\n";
    $config_file .= "define('MIN_CLEANUP_INTERVAL', 30);\n";
    $config_file .= "\n";
    $config_file .= "?" . ">";

    defined('HCS_SERVER_NAME') or define('HCS_SERVER_NAME', (string)$eventServerIp);
    defined('HCS_SERVER_PORT') or define('HCS_SERVER_PORT', (string)$eventServerPort);
    defined('HCS_SERVER_PATH') or define('HCS_SERVER_PATH', (string)$eventServerPath);

    defined('HCS_TABLE_PREFIX') or define('HCS_TABLE_PREFIX', (string)$table_prefix);

    defined('SQL_SERVER_TYPE') or define('SQL_SERVER_TYPE', (string)$_DBMS[$dbtype]['name']);
    defined('SQL_SERVER_HOST') or define('SQL_SERVER_HOST', (string)$dbhost);
    defined('SQL_SERVER_USERNAME') or define('SQL_SERVER_USERNAME', (string)$dbuser);
    defined('SQL_SERVER_PASSWORD') or define('SQL_SERVER_PASSWORD', (string)$dbpass);
    defined('SQL_SERVER_DATABASE') or define('SQL_SERVER_DATABASE', (string)$dbname);
    defined('SQL_SERVER_DATETIME_FORMAT') or define('SQL_SERVER_DATETIME_FORMAT', 'Y-m-d H:i:s');

    defined('HCS_TIMEZONE') or define('HCS_TIMEZONE', 'Europe/Berlin');
    date_default_timezone_set(HCS_TIMEZONE);

    defined('MIN_CLEANUP_INTERVAL') or define('MIN_CLEANUP_INTERVAL', 30);


    if (!$fp = @fopen($hcs_configFile, 'w')) {
        $template->appendMessage($template->StatusIcon('warning') . $_LANGUAGE['L_CONFIG_NOT_CREATED'] . '<br /><textarea rows="10" cols="50">' . $config_file . '</textarea>');
    } else {
        @fputs($fp, $config_file);
        @fclose($fp);

        $template->appendMessage($template->StatusIcon('ok') . $_LANGUAGE['L_CONFIG_CREATED']);
    }


    $sql = @new SQL($dbtype, $dbname, $dbhost, $dbuser, $dbpass);
    $sql->initializeSQL();

    $ccusString = explode("\n", $ccus);

    foreach ($ccusString as $ccuString) {
        $ccuParts = explode("\t", $ccuString);

        if (count($ccuParts) == 6) {
            if ($sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'CcuList', array(
                        'CcuName' => $ccuParts[0],
                        'CcuAddress' => $ccuParts[1],
                        'CcuVersion' => $ccuParts[2],
                        'CallbackAddress' => $callbackAddress,
                        'SubscribeWiredEvents' => XStringToSqlBoolean($ccuParts[3]),
                        'SubscribeRfEvents' => XStringToSqlBoolean($ccuParts[4]),
                        'SubscribeSystemEvents' => XStringToSqlBoolean($ccuParts[5]),
                    )) !== false
            ) {
                $template->appendMessage($template->StatusIcon('ok') . sprintf($_LANGUAGE['L_ADDED_CCU'], $ccuParts[0], $ccuParts[1]));
            }
        }
    }

    $ccuList = $sql->queryRequest("SELECT Id, CcuName, CcuAddress, SubscribeWiredEvents, SubscribeRfEvents, SubscribeSystemEvents FROM " . HCS_TABLE_PREFIX . "CcuList;");

    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'DB_Version', 'Value' => HCS_VERSION));
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'Style', 'Value' => $installStyle));
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'Language', 'Value' => $installLanguage));
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'DefaultStyle', 'Value' => $installStyle));
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'DefaultLanguage', 'Value' => $installLanguage));
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'UpdateCheck', 'Value' => 1));
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'UpdateCheckInterval', 'Value' => 3600));
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'ShowUpdateOnPage', 'Value' => 0));
    $sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Values', array('Name' => 'ExpertView', 'Value' => 0));

    //Device, DataPoint, Value, LoggingInterval, LifeSpan, OnlyLogChanges
    //Exclusions
    $sql->addRuleToSql(array(20, "*", "*", "WORKING", "*", 0, 0, false));
    $sql->addRuleToSql(array(19, "*", "*", "ERROR", "*", 0, 0, false));
    $sql->addRuleToSql(array(18, "*", "*", "STICKY_UNREACH", "*", 0, 0, false));
    $sql->addRuleToSql(array(17, "*", "*", "CONFIG_PENDING", "*", 0, 0, false));
    $sql->addRuleToSql(array(16, "*", "*", "LOWBAT", "*", 0, 0, false));
    $sql->addRuleToSql(array(15, "*", "*", "PRESS_SHORT", "*", 0, 0, false));
    $sql->addRuleToSql(array(14, "*", "*", "PRESS_LONG", "*", 0, 0, false));
    $sql->addRuleToSql(array(13, "*", "*", "INSTALL_TEST", "*", 0, 0, false));
    $sql->addRuleToSql(array(12, "*", "*", "ERROR_REDUCED", "false", 0, 0, false));
    $sql->addRuleToSql(array(11, "*", "*", "ERROR_OVERHEAT", "false", 0, 0, false));
    $sql->addRuleToSql(array(10, "*", "*", "ERROR_OVERLOAD", "false", 0, 0, false));
    $sql->addRuleToSql(array(9, "*", "*", "UNREACH", "false", 0, 0, false));
    $sql->addRuleToSql(array(8, "*", "*", "ADJUSTING_COMMAND", "*", 0, 0, false));
    $sql->addRuleToSql(array(7, "*", "*", "ADJUSTING_DATA", "*", 0, 0, false));

    $sql->addRuleToSql(array(6, "*", "*", "STATE", "*", 0, 2678400, false));
    $sql->addRuleToSql(array(5, "*", "*", "LEVEL", "*", 0, 2678400, false));
    $sql->addRuleToSql(array(4, "*", "*", "TEMPERATURE", "*", 0, 2678400, true));
    $sql->addRuleToSql(array(3, "*", "*", "HUMIDITY", "*", 0, 2678400, true));
    $sql->addRuleToSql(array(2, "*", "*", "BRIGHTNESS", "*", 0, 2678400, true));
    $sql->addRuleToSql(array(1, "*", "*", "MOTION", "*", 0, 2678400, true));
    $sql->addRuleToSql(array(0, "*", "*", "SETPOINT", "*", 0, 2678400, false));
    $sql->addRuleToSql(array(-1, "*", "*", "*", "*", 0, 2678400, true));

    $sql->addWidgetPacket("controls");
    $sql->addWidgetPacket("pic_controls");
    $sql->addWidgetPacket("hcs_diagrams");

    if (count(LoggingSingleton::getErrorArray()) <= 0) {
        $template->appendMessage($template->StatusIcon('ok') . $_LANGUAGE['L_ADDED_RULES']);
    }

    foreach ($ccuList as $ccu) {

        $regahss = new Homematic_ReGaHSS($ccu["CcuAddress"], Homematic_Interface::ReGaHSS);

        // read infos
        ProgramsArrayToSqlTable($ccu["Id"], $regahss->getProgramsList(), $sql);
        CraftsArrayToSqlTable($ccu["Id"], $regahss->getCraftsList(), $sql);
        RoomArrayToSqlTable($ccu["Id"], $regahss->getRoomList(), $sql);
        homematicDeviceArrayToSqlTable($ccu["Id"], $regahss->getDeviceList(), $sql);
        homematicSysVarArrayToSqlTable($ccu["Id"], $regahss->getSystemVariableList(), $sql);

        if (count(LoggingSingleton::getErrorArray()) <= 0) {
            $template->appendMessage($template->StatusIcon('ok') . sprintf($_LANGUAGE['L_RECIEVED_CCU_INFOS'], $ccu['CcuName']));
        }

        if ($ccu['SubscribeRfEvents']) {
            $ccuRf = new Homematic_XmlRpcInterface($ccu["CcuAddress"], Homematic_Interface::RF, $callbackAddress, $ccu["Id"]);
            if ($ccuRf->init()) {
                $template->appendMessage($template->StatusIcon('ok') . sprintf($_LANGUAGE['L_RF_SUBSCRIBED'], $ccu['CcuName']));
            }
        }
        if ($ccu['SubscribeWiredEvents']) {
            $ccuWired = new Homematic_XmlRpcInterface($ccu["CcuAddress"], Homematic_Interface::Wired, $callbackAddress, $ccu["Id"]);
            if ($ccuWired->init()) {
                $template->appendMessage($template->StatusIcon('ok') . sprintf($_LANGUAGE['L_WIRED_SUBSCRIBED'], $ccu['CcuName']));
            }
        }
        if ($ccu['SubscribeSystemEvents']) {
            $ccuSystem = new Homematic_XmlRpcInterface($ccu["CcuAddress"], Homematic_Interface::System, $callbackAddress, $ccu["Id"]);
            if ($ccuSystem->init()) {
                $template->appendMessage($template->StatusIcon('ok') . sprintf($_LANGUAGE['L_SYSTEM_SUBSCRIBED'], $ccu['CcuName']));
            }
        }
    }

    if (count(LoggingSingleton::getErrorArray()) > 0) {
        $installError = true;
        foreach (LoggingSingleton::getErrorArray() as $error) {
            $template->appendError($template->StatusIcon('error') . $error);
        }
    }
    if (count(LoggingSingleton::getWarningArray()) > 0) {
        $installWarning = true;
        foreach (LoggingSingleton::getWarningArray() as $warning) {
            $template->appendMessage($template->StatusIcon('warning') . $warning);
        }
    }
    if (count(LoggingSingleton::getInfoArray()) > 0) {
        foreach (LoggingSingleton::getInfoArray() as $info) {
            $installInfo .= utf8_encode($info) . '<br />';
        }
    }

    $template->assignVariables(array(
            'L_BUTTON_STEP_5' => $_LANGUAGE['L_BUTTON_STEP_5'],
            'L_CONFIG_CREATED' => $_LANGUAGE['L_CONFIG_CREATED'],
            'L_ADDED_CCU' => $_LANGUAGE['L_ADDED_CCU'],
            'L_ADDED_RULES' => $_LANGUAGE['L_ADDED_RULES'],
            'L_RF_SUBSCRIBED' => $_LANGUAGE['L_RF_SUBSCRIBED'],
            'L_WIRED_SUBSCRIBED' => $_LANGUAGE['L_WIRED_SUBSCRIBED'],
            'L_SYSTEM_SUBSCRIBED' => $_LANGUAGE['L_SYSTEM_SUBSCRIBED'],
            'L_RECIEVED_CCU_INFOS' => $_LANGUAGE['L_RECIEVED_CCU_INFOS'],
            'SQL_TITLE' => $_LANGUAGE['inst_sqlheaderbox'],
            'SQL_TEXT' => (!$installError) ? $installInfo : '',
            'SHOW_SQLINFO' => ($installError) ? false : true,
        )
    );

    $template->preparePageHeader();
    $template->prepratePageTailAndDisplay();

}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="STEP 6">
function installStep6()
{
    global $HCS_ROOTPATH, $_DEFAULTS, $_LANGUAGE, $_DBMS, $installLanguage, $installStyle, $hcs_configFile;
    $template = new InstallTemplate('install_step6.html');


    $template->assignVariables(array(
            'L_ADMINACCOUNT_TITLE' => $_LANGUAGE['L_ADMINACCOUNT_TITLE'],
            'L_ADMINACCOUNT_TEXT' => $_LANGUAGE['L_ADMINACCOUNT_TEXT'],
            'L_ADMINACCOUNT_USERNAME' => $_LANGUAGE['L_ADMINACCOUNT_USERNAME'],
            'L_ADMINACCOUNT_PASSWORD' => $_LANGUAGE['L_ADMINACCOUNT_PASSWORD'],
            'L_ADMINACCOUNT_PASSWORD_CONFIRM' => $_LANGUAGE['L_ADMINACCOUNT_PASSWORD_CONFIRM'],
            'L_ADMINACCOUNT_EMAIL' => $_LANGUAGE['L_ADMINACCOUNT_EMAIL'],
            'L_ACCOUNT_PASSWORD_NO_MATCH' => $_LANGUAGE['L_ACCOUNT_PASSWORD_NO_MATCH'],
            'L_ACCOUNT_EMAIL_ERROR' => $_LANGUAGE['L_ACCOUNT_EMAIL_ERROR'],
            'L_BUTTON_STEP_6' => $_LANGUAGE['L_BUTTON_STEP_6'],
        )
    );

    $template->preparePageHeader();
    $template->prepratePageTailAndDisplay();

}

// </editor-fold>

/* -------------------------------------------------------------------------- */

// <editor-fold defaultstate="collapsed" desc="STEP 7">
function installStep7()
{
    global $HCS_ROOTPATH, $_DEFAULTS, $_LANGUAGE, $_DBMS, $installLanguage, $installStyle, $hcs_configFile;
    $template = new InstallTemplate('install_step7.html');

    $adminUsername = getPostIndexOrArrayValue('adminUsername');
    $adminPassword = getPostIndexOrArrayValue('adminPassword');
    $adminPasswordConfirm = getPostIndexOrArrayValue('adminPasswordConfirm');
    $adminEMail = getPostIndexOrArrayValue('adminEMail');


    // create admin user in mysql database
    LoggingSingleton::getInstance()->logError = true;
    LoggingSingleton::getInstance()->logToArray = true;
    $sql = @new SQL();
    if ($sql->insertOrUpdateInTo(HCS_TABLE_PREFIX . 'Users', array(
                'Username' => escapeData($adminUsername),
                'Password' => escapeData(hasher($adminPassword)),
                'EMail' => escapeData($adminEMail),
                'Adminrights' => 1,
                'LastLogin' => date(SQL_SERVER_DATETIME_FORMAT),
                'Style' => escapeData($installStyle),
                'Language' => escapeData($installLanguage),
            )) !== false
    ) {
        $template->appendMessage($template->StatusIcon('ok') . $_LANGUAGE['L_ADMIN_USER_CREATED']);
    }


    //
    // Rewrite the config file to its final form
    //
    $config_file = file($hcs_configFile);
    $config_file = implode("", $config_file);
    $config_file = preg_replace('#\?' . '>$#', '', $config_file);
    $config_file .= 'define(\'HCS_INSTALLED\', true);' . "\n";
    $config_file .= "\n";
    $config_file .= '?' . '>';

    if (!$fp = @fopen($hcs_configFile, 'w')) {
        $template->appendMessage($template->StatusIcon('warning') . $_LANGUAGE['L_CONFIG_NOT_CREATED'] . '<br /><textarea rows="10" cols="50">' . $config_file . '</textarea>');
    } else {
        @fputs($fp, $config_file);
        @fclose($fp);
    }


    if (count(LoggingSingleton::getErrorArray()) > 0) {
        foreach (LoggingSingleton::getErrorArray() as $error) {
            $template->appendError($template->StatusIcon('error') . $error);
        }
    }

    $template->assignVariables(array(
            'USERNAME' => $adminUsername,
            'PASSWORD' => $adminPassword,
            'L_LOGIN' => $_LANGUAGE['L_LOGIN'],
            'L_USERNAME' => $_LANGUAGE['L_USERNAME'],
            'L_PASSWORD' => $_LANGUAGE['L_PASSWORD'],
            'L_REMEMBER_PASSWORD' => $_LANGUAGE['L_REMEMBER_PASSWORD'],
            'L_LOGIN_BUTTON' => $_LANGUAGE['L_LOGIN_BUTTON'],
        )
    );

    $template->preparePageHeader();
    $template->prepratePageTailAndDisplay();

}

// </editor-fold>


/* -------------------------------------------------------------------------- */
// EOF
?>