/**
 * Gray theme for Highcharts JS
 * @author Torstein Hønsi
 *
 *
 */

var contentColor = getCSS('color', 'ui-widget-content');
console.log(contentColor);


Highcharts.theme = {
	colors: ["#eca525","#ecd925","#be8615","#d2c4b7","#ec7225","#ecd525","#2585ec","#78ec25","#ec4e25"],
	chart: {
		backgroundColor: null,
		borderWidth: 0,
		borderRadius: 5,
		plotShadow: false,
        plotBackgroundColor: null,
		plotBorderWidth: 1,
        plotBorderColor: '#7b622a'
	},
	title: {
		style: {
			color: '#d2c4b7',
			font: 'bold 16px Segoe UI, Arial, sans-serif'
		}
	},
	subtitle: {
		style: {
			color: '#d2c4b7',
			font: '12px Segoe UI, Arial, sans-serif'
		}
	},
	xAxis: {
		gridLineWidth: 0,
		lineColor: '#7b622a',
		tickColor: '#7b622a',
		labels: {
			style: {
				color: '#d2c4b7',
				fontWeight: 'bold'
			}
		},
		title: {
			style: {
				color: '#d2c4b7',
				font: 'bold 12px Segoe UI, Arial, sans-serif'
			}
		}
	},
	yAxis: {
		alternateGridColor: null,
		minorTickInterval: null,
		gridLineColor: 'rgba(255, 255, 255, .1)',
		minorGridLineColor: 'rgba(255,255,255,0.07)',
		lineWidth: 1,
		tickWidth: 1,
        lineColor: '#7b622a',
        tickColor: '#7b622a',
		labels: {
			style: {
				color: '#d2c4b7',
				fontWeight: 'bold'
			}
		},
		title: {
			style: {
				color: '#d2c4b7',
				font: 'bold 12px Segoe UI, Arial, sans-serif'
			}
		}
	},
	legend: {
		itemStyle: {
			color: '#d2c4b7',
            fontWeight: 'normal'
		},
		itemHoverStyle: {
			color: '#d2c4b7',
            fontWeight: 'bold'
		},
		itemHiddenStyle: {
			color: '#333'
		}
	},
	labels: {
		style: {
			color: '#d2c4b7'
		}
	},
	tooltip: {
		backgroundColor: {
			linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
			stops: [
				[0, 'rgba(78, 58, 06, .8)'],
				[1, 'rgba(25, 18, 0, .8)']
			]
		},
		borderWidth: 0,
		style: {
			color: '#d2c4b7'
		}
	},


	plotOptions: {
		series: {
			shadow: true
		},
		line: {
			dataLabels: {
				color: '#CCC'
			},
			marker: {
				lineColor: '#333'
			}
		},
		spline: {
			marker: {
				lineColor: '#333'
			}
		},
		scatter: {
			marker: {
				lineColor: '#333'
			}
		},
		candlestick: {
			lineColor: 'white'
		}
	},

	toolbar: {
		itemStyle: {
			color: '#CCC'
		}
	},

	navigation: {
		buttonOptions: {
			symbolStroke: '#d2c4b7',
			hoverSymbolStroke: '#FFFFFF',
            symbolFill: '#271a07',
			theme: {
				fill: '#271a07',
				stroke: '#d2c4b7',
                r: 5,
                states: {
                    hover: {
                        stroke: '#FFF',
                        fill: '#271a07'
                    },
                    select: {
                        stroke: '#FFF',
                        fill: '#271a07'
                    }
                }
			}
		}
	},

    rangeSelector: {
        buttonTheme: {
            fill: 'none',
            stroke: '#000000',
            style: {
                color: '#CCC',
                fontWeight: 'bold'
            },
            states: {
                hover: {
                    fill: {
                        linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
                        stops: [
                            [0.4, '#BBB'],
                            [0.6, '#888']
                        ]
                    },
                    stroke: '#000000',
                    style: {
                        color: 'white'
                    }
                },
                select: {
                    fill: {
                        linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
                        stops: [
                            [0.1, '#000'],
                            [0.3, '#333']
                        ]
                    },
                    stroke: '#000000',
                    style: {
                        color: 'yellow'
                    }
                }
            }
        },
        inputStyle: {
            backgroundColor: '#191200',
            color: '#d2c4b7'
        },
        labelStyle: {
            color: '#d2c4b7'
        }
    },


	navigator: {
		handles: {
			backgroundColor: '#666',
			borderColor: '#AAA'
		},
		outlineColor: '#CCC',
		maskFill: 'rgba(16, 16, 16, 0.5)',
		series: {
			color: '#7798BF',
			lineColor: '#A6C7ED'
		}
	},

	scrollbar: {
		barBackgroundColor: {
				linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
				stops: [
					[0.4, '#3b2a0c'],
					[0.6, '#271a07']
				]
			},
		barBorderColor: '#7b622a',
		buttonArrowColor: '#d2c4b7',
		buttonBackgroundColor: {
				linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
				stops: [
                    [0.4, '#3b2a0c'],
                    [0.6, '#271a07']
				]
			},
		buttonBorderColor: '#7b622a',
		rifleColor: '#d2c4b7',
		trackBackgroundColor: {
			linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
			stops: [
				[0, '#191200'],
				[1, '#271a07']
			]
		},
		trackBorderColor: '#3b2a0c'
	},

	// special colors for some of the demo examples
	legendBackgroundColor: 'rgba(48, 48, 48, 0.8)',
	legendBackgroundColorSolid: 'rgb(70, 70, 70)',
	dataLabelsColor: '#444',
	textColor: '#E0E0E0',
	maskColor: 'rgba(255,255,255,0.3)'
};

// Apply the theme
var highchartsOptions = Highcharts.setOptions(Highcharts.theme);
