/**
 * Gray theme for Highcharts JS
 * @author Torstein Hønsi
 */



var fontFamily = getCSS('font-family', 'ui-widget');
var chartBgColor = getCSS('background-color', 'ui-widget-content');
var borderColor = getCSS('border-top-color', 'ui-widget-content');
var borderWidth = parseInt(getCSS('border-top-width', 'ui-widget-content'));
var borderRadius = parseInt(getCSS('border-top-left-radius', 'ui-corner-all'));
var textActiveColor = getCSS('color', 'ui-state-default');
var textColor = getCSS('color', 'ui-widget-content');
var textDisableColor = changeAlpha(textColor, .5);
var lineColor = textColor;
var gridLineColor = changeAlpha(lineColor, .1);
var minorGridLineColor = changeAlpha(lineColor, .05);

var bgColorTop = chartBgColor;
var bgColor = chartBgColor;
var bgColorBottom = getCSS('background-color', 'ui-widget-header');

var navLineColor = textActiveColor;
var navAreaColor = changeAlpha(navLineColor, .1);

Highcharts.theme = {
    colors: getHarmonicColors(navLineColor),
    chart: {
        backgroundColor: chartBgColor,
        borderWidth: borderWidth,
        borderRadius: borderRadius,
        borderColor: borderColor,
        plotBackgroundColor: null,
        plotShadow: false,
        plotBorderWidth: 0
    },
    title: {
        style: {
            color: textActiveColor,
            font: '16px ' + fontFamily
        }
    },
    subtitle: {
        style: {
            color: textColor,
            font: '12px ' + fontFamily
        }
    },
    xAxis: {
        gridLineWidth: 0,
        lineColor: lineColor,
        tickColor: lineColor,
        gridLineColor: gridLineColor,
        minorGridLineColor: minorGridLineColor,
        labels: {
            style: {
                color: textColor,
                fontWeight: 'bold'
            }
        },
        title: {
            style: {
                color: textActiveColor,
                font: 'bold 12px ' + fontFamily
            }
        }
    },
    yAxis: {
        lineColor: lineColor,
        tickColor: lineColor,
        gridLineColor: gridLineColor,
        minorGridLineColor: minorGridLineColor,
        alternateGridColor: null,
        minorTickInterval: null,
        lineWidth: 1,
        tickWidth: 1,
        labels: {
            style: {
                color: textColor,
                fontWeight: 'bold'
            }
        },
        title: {
            style: {
                color: textActiveColor,
                font: 'bold 12px ' + fontFamily
            }
        }
    },
    legend: {
        itemStyle: {
            color: textColor
        },
        itemHoverStyle: {
            color: textActiveColor
        },
        itemHiddenStyle: {
            color: textDisableColor
        }
    },
    labels: {
        style: {
            color: textColor
        }
    },
    tooltip: {
        backgroundColor: {
            linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
            stops: [
                [0, bgColorTop],
                [1, bgColorBottom]
            ]
        },
        borderWidth: 0,
        borderRadius: borderRadius,
        style: {
            color: textColor
        }
    },


    plotOptions: {
        series: {
            shadow: true
        },
        line: {
            dataLabels: {
                color: textColor
            },
            marker: {
                lineColor: textDisableColor
            }
        },
        spline: {
            marker: {
                lineColor: textDisableColor
            }
        },
        scatter: {
            marker: {
                lineColor: textDisableColor
            }
        },
        candlestick: {
            lineColor: textActiveColor
        }
    },

    toolbar: {
        itemStyle: {
            color: textColor
        }
    },

    navigation: {
        buttonOptions: {
            symbolStroke: textColor,
            hoverSymbolStroke: textActiveColor,
            theme: {
                fill: {
                    linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
                    stops: [
                        [0.4, bgColorTop],
                        [0.6, bgColorBottom]
                    ]
                },
                stroke: borderColor
            }
        }
    },

    // scroll charts
    rangeSelector: {
        buttonTheme: {
            fill: {
                linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
                stops: [
                    [0.4, bgColorTop],
                    [0.6, bgColorBottom]
                ]
            },
            stroke: borderColor,
            style: {
                color: textColor,
                fontWeight: 'bold'
            },
            states: {
                hover: {
                    fill: {
                        linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
                        stops: [
                            [0.4, bgColorTop],
                            [0.6, bgColorBottom]
                        ]
                    },
                    stroke: borderColor,
                    style: {
                        color: textActiveColor
                    }
                },
                select: {
                    fill: {
                        linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
                        stops: [
                            [0.1, bgColorBottom],
                            [0.3, bgColorTop]
                        ]
                    },
                    stroke: borderColor,
                    style: {
                        color: textActiveColor
                    }
                }
            }
        },
        inputStyle: {
            backgroundColor: bgColor,
            color: textColor
        },
        labelStyle: {
            color: textColor
        }
    },

    navigator: {
        handles: {
            backgroundColor: bgColorTop,
            borderColor: lineColor
        },
        outlineColor: lineColor,
        maskFill: 'rgba(0, 0, 0, 0.2)',
        series: {
            color: navAreaColor,
            lineColor: navLineColor
        }
    },

    scrollbar: {
        barBackgroundColor: {
            linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
            stops: [
                [0.4, bgColorTop],
                [0.6, bgColorBottom]
            ]
        },
        barBorderColor: textColor,
        buttonArrowColor: textColor,
        buttonBackgroundColor: {
            linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
            stops: [
                [0.4, bgColorTop],
                [0.6, bgColorBottom]
            ]
        },
        buttonBorderColor: textColor,
        rifleColor: textColor,
        trackBackgroundColor: {
            linearGradient: { x1: 0, y1: 0, x2: 0, y2: 1 },
            stops: [
                [0, chartBgColor],
                [1, bgColorTop]
            ]
        },
        trackBorderColor: borderColor
    },

    // special colors for some of the demo examples
    legendBackgroundColor: 'rgba(48, 48, 48, 0.8)',
    legendBackgroundColorSolid: 'rgb(70, 70, 70)',
    dataLabelsColor: '#444',
    textColor: '#E0E0E0',
    maskColor: 'rgba(255,255,255,0.3)'
};

// Apply the theme
var highchartsOptions = Highcharts.setOptions(Highcharts.theme);